#!/usr/bin/perl

use strict;

my $yes_answer = 0;
if ($ARGV[0] eq "-y")
  {
    $yes_answer = 1;
  }

my $pslist=`ps xawww`;

my @eximconf=('/etc/mail/exim.conf','/etc/exim/exim.conf','/usr/local/etc/exim/configure','/etc/exim/exim4.conf');
my @eximbin=('/usr/bin/exim','/usr/sbin/exim','/usr/local/bin/exim','/usr/local/sbin/exim','/usr/sbin/exim4');
my @sendmailconf=('/etc/mail/sendmail.cf','/etc/sendmail.cf','/usr/local/etc/sendmail.cf');
my @postfixconf=('/usr/local/etc/postfix/master.cf','/etc/postfix/master.cf','/etc/mail/master.cf');

if ($pslist=~/CGServer|Communigate/i)
  {
    print "Running Communigate Pro detected.\nPlease install KAS client software according to User Manual\n\n";
    exit 1;
  }

my %MTAS;

$pslist="exim qmail sendmail  master";

if ($pslist=~/exim/) 
  {
    my $data;
    $data->{type} = 'exim';
    for my $conf (@eximconf)
      {
        if(-f $conf) { $data->{config} = $conf; last;}
      }
    for my $xbin (@eximbin)
      {
        if(-x $xbin) { $data->{bin} = $xbin; last}
      }

    if ($data->{bin} && $data->{config})
      {
        $MTAS{exim}=$data;
      }
    else
      {
        #
        # Trying to detect Debian-style installation
        #

        if (-d "/etc/exim4/conf.d")
          {
            $data->{type} = "exim (debian style installation)";
            $data->{config} = "/etc/exim4";

            $MTAS{exim}=$data;
          }
      }
  }


if ($pslist=~/qmail/) 
  {
    my $data;
    $data->{type} = 'qmail';
    $data->{config}='/var/qmail' if -d '/var/qmail';
    $MTAS{qmail}=$data if $data->{config};
  } 

if ($pslist=~/[ \[\/]master/) 
  {
    my $data;
    $data->{type} = 'postfix';
    for my $conf (@postfixconf)
      {
        if(-f $conf) { $data->{config} = $conf; last;}
      }
    $MTAS{postfix}=$data if $data->{config};
  }

if ($pslist=~/sendmail/) 
  {
    my $data;
    $data->{type} = 'sendmail';
    for my $conf (@sendmailconf)
      {
        if(-f $conf) { $data->{config} = $conf; last;}
      }
    $MTAS{sendmail}=$data if $data->{config};
}

# try to find config files if no MTA found running
if(scalar keys %MTAS<1) 
  { 
    my $data = undef;
    unless ($MTAS{qmail})
      {
        if( -d '/var/qmail') 
          {
            $data->{type} = 'qmail';
            $data->{config}='/var/qmail';
            $MTAS{qmail}=$data if $data->{config};
            undef $data;
          }
      }
    # postfix
    unless ($MTAS{postfix})
      {
        for my $conf (@postfixconf)
          {
            if(-f $conf)
              {
                $data->{type}='postfix';
                $data->{config} = $conf;
                $MTAS{postfix}=$data if $data->{config};
                undef $data;
            last;
              }
          }
      }
    unless ($MTAS{exim})
      {
        my ($config,$bin)=(undef,undef);
        for my $conf (@eximconf)
          {
            if(-f $conf) { $config = $conf; last;}
          }
        for my $xbin (@eximbin)
          {
            if(-x $xbin) { $bin = $xbin; last}
          }
        if ($bin && $config)
          {
                $data->{type}='exim';
                $data->{config} = $config;
                $data->{bin} = $bin;
                $MTAS{exim}=$data;
                undef $data;
          }
      }
  }

my $mtadata=undef;
if (scalar keys %MTAS<0)
  {
    print "Cannot find supported MTA. Please read user manual and configure your mail server by yourself\n";
    exit 1;
  }
else
  {
    $|=1;
    if (!$yes_answer)
      {
        # MANY MTAS
        print "Choose mail server for installation:\n";
        my @msl = ({type=>'Cancel installation'});
        
        foreach ('postfix','exim','qmail','sendmail')
          {
            push @msl, $MTAS{$_} if exists $MTAS{$_};
          }

        foreach my $i(0..$#msl)
          {
            print "  $i) ".(ucfirst $msl[$i]->{type}).($i?" (config: $msl[$i]->{config})":"")."\n";
          }

        print "\nPlease select mail server to install Kaspersky Antispam [0]: ";
        my $s =<>;
        my $idx = $1 if $s=~/(\d+)/;
        exit 0 unless $idx>0 && $idx<=$#msl;
        $mtadata = $msl[$idx];
      }
    else
      {
        my @msl;
        print "Next MTAs was found:\n";
        foreach ('postfix','exim','qmail','sendmail')
          {
            push @msl, $MTAS{$_} if exists $MTAS{$_};
          }

        foreach my $i(0..$#msl)
          {
            print "  $i) ".(ucfirst $msl[$i]->{type}).($i?" (config: $msl[$i]->{config})":"")."\n";
          }
        print "Automatic selection was forced, choosing $msl[0]->{type} (config: $msl[0]->{config})\n";
        $mtadata = $msl[0];
      }
  }

print "Installing Kaspersky Antispam into ".ucfirst $mtadata->{type}." mail server\n";

my $cmdline="";
if ($mtadata->{type} eq 'exim')
  {
    $cmdline="/usr/local/ap-mailfilter3/bin/config-exim.pl $mtadata->{config} $mtadata->{bin}";
  }
elsif ($mtadata->{type} =~ /exim.*debian/)
  {
    $cmdline="/usr/local/ap-mailfilter3/bin/config-exim-debian.pl $mtadata->{config} $mtadata->{bin}";
    
  }
else
  {
    $cmdline="/usr/local/ap-mailfilter3/bin/config-$mtadata->{type}.pl $mtadata->{config}";
  }

system($cmdline);
my $exit_value  = $? >> 8;
exit $exit_value;
