#!/usr/bin/perl

use FindBin qw($Bin);
require "$Bin/config-lib.pl";
use strict;

my $exim_dir= $ARGV[0];

unless ($exim_dir && -d $exim_dir)
  {
    print "Usage: $0 /path/to/exim-conf-dir\n";
    exit 1;
  }

unless ($< == 0) {
 print "$0 should be run as root\n";
 exit 1;
}

#my $autogenerated_conf="/var/lib/exim4/config.autogenerated";
my $exim_conf_template="$exim_dir/exim4.conf.template";
my $exim_conf_dir="$exim_dir/conf.d";

my $router_file="$exim_conf_dir/router/099_exim4-config_kas30router";
my $transport_file="$exim_conf_dir/transport/30_exim4-config_kas30transport";

unless (-r "$exim_conf_template" && -d "$exim_conf_dir")
  {
    print "Directory $exim_dir isn't Debian exim configuration directory.\n";
    exit 1;
  }

#unless (-r $autogenerated_conf)
#  {
#    print "Couldn't find autogenerated config file, $autogenerated_conf.\n";
#    exit 1;
#  }

if (-f $router_file)
  {
    print "File $router_file already exist.\n";
    exit 1;
  }

if (-f $transport_file)
  {
    print "File $transport_file already exist.\n";
    exit 1;
  }

my $eb = $ARGV[1] || "/usr/sbin/exim4";
while (! -x $eb) {
 printf "Cannot find your exim binary file\n";
 print "Enter path to exim binary (enter to quit) [$eb]:";
 $eb=<STDIN>;
 chomp $eb;
 exit 0 if length $eb < 4;
}

my $outcommand = "$eb -bs";

#check_exim_config($autogenerated_conf);
check_exim_config($exim_conf_template);

print "Creating kas-pipe config...\n";
make_pipe_conf(1,0,0,0,"exec:$outcommand");

print "Editing exim configuration files...\n";

# patch_exim_config("$autogenerated_conf", "$autogenerated_conf.tmp");
patch_exim_config("$exim_conf_template", "$exim_conf_template.tmp");

open ROUTER, ">$router_file" or die;
print ROUTER exim_get_router();
close ROUTER;

open TRANSPORT, ">$transport_file" or die;
print TRANSPORT exim_get_transport();
close TRANSPORT;

init_backup_data();

my $backuped = mta_backuped('exim');

# submit_orig_file('exim',"$autogenerated_conf") unless ($backuped);
# unlink "$autogenerated_conf" if -f "$autogenerated_conf";
# rename "$autogenerated_conf.tmp","$autogenerated_conf";
# submit_changed_file('exim',"$autogenerated_conf") unless $backuped;

submit_orig_file('exim',"$exim_conf_template") unless ($backuped);
unlink "$exim_conf_template" if -f "$exim_conf_template";
rename "$exim_conf_template.tmp","$exim_conf_template";
submit_changed_file('exim',"$exim_conf_template") unless $backuped;

submit_created_file('exim', $router_file) unless $backuped;
submit_created_file('exim', $transport_file) unless $backuped;

my $rpath = "/etc/init.d/exim4 restart";

if ($rpath)
  {
    submit_restart('exim',$rpath);
    system($rpath);
  }


save_backup() unless $backuped;

print "Ok.\n";

