#!/usr/bin/perl

use FindBin qw($Bin);
require "$Bin/config-lib.pl";
use strict;

my $pcf= $ARGV[0];

unless ($pcf && -r $pcf)
  {
    print "Usage: $0 /path/to/exim.configfile\n";
    exit 1;
  }

my $s;

unless ($< == 0) {
 print "$0 should be run as root\n";
 exit 1;
}

$|=1;
RETRY:
unless ( -r $pcf ) {
RETR2:
 print "Cannot find your exim.conf configuration file\n";
 print "Please enter path to it (enter to quit):";
 $pcf=<STDIN>;
 chomp $pcf;
 exit 1 if length $pcf<2;
 goto RETRY;
}


my $ok=0;
my $exim4 = 0;
open F, $pcf;

while (<F>)
{
 $exim4 = 1 if /^begin\s+transports/;
}
close F;

if (!$exim4) 
{
  print "$pcf is not a Exim 4.x configuration file\n";
  goto RETR2;
}

my $eb = $ARGV[1] || "/usr/sbin/exim";
while (! -x $eb) {
 printf "Cannot find your exim binary file\n";
 print "Enter path to exim binary (enter to quit) [$eb]:";
 $eb=<STDIN>;
 chomp $eb;
 exit 0 if length $eb < 4;
}

# check for KAV
#
# XXX This code have to be checked with newer versions of KAV

print "Config: $pcf\n";

my $outcommand =undef;
my $lmtpout = 0;
my $lmtpin = 1;
my $no_l_kav = 0;
my $haskav = 0;

for my $file (("$pcf.listen.kav","$pcf.listen"))
  {
    if (-f $file)
      {
	open F,"$file" or die;
	my $inkav=0;
	while (<F>)
	  {
	    my $s=$_;
	    if($s=~/^kav_lmtp_transport/)
	      {
		$haskav = $inkav = 1;
		next;
	      }
	    if($inkav) 
	      {
		$outcommand = $1 if ($s=~/command\s*=\s*(\S.*)/);
		$inkav = 0 if ($s=~/^[a-z]/);
	      }
	  }
	close F;
	if ($haskav )
	  {
	    $no_l_kav = 1 if $file =~/listen\.kav/;
	    last;
	  }
      }
  }

if($haskav)
  {
    if($outcommand)
      {
	print "You have installed Kaspersky Anti-Virus.\n";
	print "Kaspersky Anti-Spam will run before Kaspersky Anti-Virus\n";
      }
    else
      {
	print "You have installed Kaspersky Anti-Virus.\n";
	print "Unfortunately, installation script cannot parse your configuration file\n";
	print "You should install Kaspersky Anti-Spam client into your Exim configuration yourself\n";
	exit 1;
      }
  }
$lmtpout = 1 if $outcommand;
$outcommand = "$eb -bs" unless $outcommand;

#
# Localscan client check
#

my $has_localscan = 0;

open F, "$pcf" or die;
while (<F>)
  {
    if (/^\s*spamtest_address\s*=/ || /^\s*kas_connect_to\s*=/)
      {
        $has_localscan = 1;
        last;
      }
  }
close(F);

if ($has_localscan)
  {
    print "You exim configuration file $pcf already contains kas-exim\n";
    print "local_scan configuration parameters. If your Exim hasn't been\n";
    print "integrated with kas-exim, remove all local_scan parameters and\n";
    print "try again.\n";
    exit 1;
  }

check_exim_config($pcf);

print "Creating kas-pipe config...\n";
make_pipe_conf($lmtpin,$lmtpout,0,0,"exec:$outcommand");

print "Editing exim.conf file...\n";

patch_exim_config("$pcf", "$pcf.tmp");

init_backup_data();

my $backuped = mta_backuped('exim');

submit_orig_file('exim',"$pcf") unless ($backuped);

unlink "$pcf" if -f "$pcf";
rename "$pcf.tmp","$pcf";

submit_changed_file('exim',"$pcf") unless $backuped;

my @exim_startup = ('/etc/init.d/exim', '/etc/init.d/exim4', '/usr/local/etc/rc.d/exim.sh', '/usr/local/etc/rc.d/exim4.sh');

for my $startup (@exim_startup)
  {
    if(-x $startup) { 
      my $rpath = "$startup restart";
      submit_restart('exim', $rpath);
      system($rpath);
      last
    }
  }

save_backup() unless $backuped;

print "Ok.\n";
