use Data::Dumper;

my $md5='/usr/local/ap-mailfilter3/bin/md5calc';
my $bkppath="/usr/local/ap-mailfilter3/conf/mtabackup";
my $bkfile="$bkppath/.list";

sub make_pipe_conf
  {
  my ($inlmtp,$outlmtp,$xf,$oport,$oaddress)=@_;
  die "Cannot create config" unless $oport || $oaddress;

  $oaddress ||="tcp:127.0.0.1:$oport";
  my $ilmtp = $inlmtp?"lmtp":"smtp";
  my $olmtp = $outlmtp?"lmtp":"smtp";
  my $fname="/usr/local/ap-mailfilter3/etc/filter.conf";
  my $xforward = $xf?"yes":"no";

  open IN,"$fname" or die;
  open OUT,">$fname.new" or die;
  my ($in,$out,$outad) = (0,0,0);
  # commenting old settings
  while (<IN>)
    {
      $s=$_;
      if($s=~/^\s*#/) { print OUT $s; next;}
      if($s=~/^\s*Pipe/i) { print OUT "# $s"; next;}
      print OUT $s;
    }
  print OUT <<EOM;

## Added by config-lib.pl
PipeInProtocol $ilmtp
PipeOutProtocol $olmtp
PipeOutgoingAddr $oaddress
PipeUseXForward $xforward

EOM
  close IN;
  close OUT;
  rename $fname,"$fname.bak" or die;
  rename "$fname.new", $fname or die;
  my  ($login,$pass,$uid,$gid) = getpwnam('mailflt3')
      or die "mailflt3 not in passwd file";
  chown $uid,$gid,$fname or die "Cannot change owner of $fname to mailflt3";
}

sub kas_milter_monitoring
  {
    my ($monitor)=@_;
    my $filterconf_name="/usr/local/ap-mailfilter3/etc/filter.conf";

    open IN,"$filterconf_name" or die;
    open OUT,">$filterconf_name.new" or die;
    
    while (<IN>)
      {
        $s=$_;
        if($s=~/^\s*#/) { print OUT $s; next;}
        if($s=~/^\s*MonitoringKasMilter/i) { print OUT "# $s"; next;}
        print OUT $s;
      }
    print OUT <<EOM;

## Added by config-sendmail.pl
MonitoringKasMilter $monitor
EOM

    close IN;
    close OUT;
    
    rename $filterconf_name,"$filterconf_name.bak" or die;
    rename "$filterconf_name.new", $filterconf_name or die;
    my  ($login,$pass,$uid,$gid) = getpwnam('mailflt3')
      or die "mailflt3 not in passwd file";
    chown $uid,$gid,$filterconf_name or die "Cannot change owner of $filterconf_name to mailflt3";
    
  }

my $exim_router="\n# ROUTER ADDED BY KAS 3.0 INSTALLER
kas30router:
  driver = accept
  local_parts = passwd;\$local_part : lsearch
  condition = \"\${if !eq {\$received_protocol}{local-esmtp}{yes}}\"
  transport = kas30transport
";

my $exim_transport="\n# TRANSPORT ADDED BY KAS 3.0 INSTALLER
kas30transport:
  driver = lmtp
  batch_max = 100
  command = /usr/local/ap-mailfilter3/bin/kas-pipe
  return_path_add = false
";

sub exim_get_transport
{
	return $exim_transport;
}

sub exim_get_router
{
	return $exim_router;
}

sub check_exim_config
{
    my ($infile)=@_;
    open IN,"$infile" or die;
    while(<IN>) {
      if(/^\s*kas30router:/) {
         print "You configuration file $infile already have kas30router.\n";
         exit 1;
      }
      if(/^\s*kas30transport:/) {
         print "You configuration file $infile already have kas30transport.\n";
         exit 1;
      }
    }
    close IN;

}

sub patch_exim_config
  {
    my ($infile, $outfile)=@_;
    my $s;

    open IN,"$infile" or die;
    open OUT,">$outfile" or die;

    my $old_mode = ((stat $infile)[2] & 0777) || 0644;
    chmod $old_mode,$outfile;

    while(<IN>)
      {
        $s=$_;
        print OUT $s;
        
        if($s=~/^begin\s+routers/) 
          { 
            print OUT $exim_router;
          };
        
        if($s=~/^begin\s+transports/) 
          { 
            print OUT $exim_transport;
          };
        
      }

    close(IN);
    close(OUT);
    
  }

my $data;
#    
#  :
# mtaname|restart command|path:md5:savedpath:mode[|path:md5:savedpath:mode]
sub init_backup_data
  {
    open IN,$bkfile;
    while (<IN>)
      {
        chomp;
        my ($mta,$restart,@remain)=split(/\|/,$_);
        $data->{$mta}->{list} = parse_bkp(\@remain);
        $data->{$mta}->{restart}=$restart;
      }
  }

sub save_backup
  {
    open O,">$bkfile" or die;
    foreach my $mta(keys %{$data})
      {
        print O "$mta|$data->{$mta}->{restart}|";
        foreach my $file(values %{$data->{$mta}->{list}})
          {
            print O "$file->{path}:$file->{md5}:$file->{saved}:".sprintf("%d",$file->{mode})."|";
          }
        print O "\n";
       }
    close O;
  }
sub parse_bkp
  {
    my $l = shift or return ();
    my $ret={};
    foreach my $i (@{$l})
      {
        my $h;
        ($h->{path},$h->{md5},$h->{saved},$h->{mode})=split(/:/,$i);
        $ret->{$h->{path}}=$h if length $h->{path}>1;
        undef $h;
      }
    return $ret;
  }

sub mta_backuped
  {
    my $mta = shift;
    return (exists $data->{$mta})?1:0;
  }

# set original file data;
sub submit_orig_file
  {
    my ($mta,$fname)=@_;
    return undef unless $fname;
    $data->{$mta}->{list}->{$fname}->{path}=$fname;
    $data->{$mta}->{list}->{$fname}->{mode} = ((stat $fname)[2] & 0777) || 0644;
    my $cnt = sprintf("%04d",scalar keys %{$data->{$mta}->{list}});
    my $outn = "$mta.$cnt";
    open OUT, ">$bkppath/$outn";
    open IN, $fname;
    print OUT while <IN>;
    close IN;
    $data->{$mta}->{list}->{$fname}->{saved}=$outn;
}

sub submit_restart
  {
    my ($mta,$fname)=@_;
    $data->{$mta}->{restart}=$fname if length $fname>2;
  }
sub submit_changed_file
  {
    my ($mta,$fname)=@_;

    my $md5str= `$md5 $fname`;
    my ($md5,@r)=split(/ /,$md5str);
    $data->{$mta}->{list}->{$fname}->{md5}=$md5;
  }

sub submit_created_file
  {
    my ($mta,$fname)=@_;

    my $md5str= `$md5 $fname`;
    my ($md5,@r)=split(/ /,$md5str);
    $data->{$mta}->{list}->{$fname}->{path}=$fname;
    $data->{$mta}->{list}->{$fname}->{md5}=$md5;
    $data->{$mta}->{list}->{$fname}->{mode}="99999";
  }

sub unconfig_mta
  {
    my $mta = shift;
    return unless exists $data->{$mta};
    # checking files
    foreach my $file( values %{$data->{$mta}->{list}})
      {
        unless (-w $file->{path})
          {
            print STDERR "No permissions to write into file $file->{path}, cannot continue\n";
            return;
          }
        my ($md5) = split(/\s+/,`$md5 $file->{path}`);
        unless ($md5 eq $file->{md5})
          {
            print STDERR "File $file->{path} has changed after KAS installation, cannot restore initial state\n";
            return;
          }
      }
    # all files OK, restoring original
    foreach my $file( values %{$data->{$mta}->{list}})
      {
        if($file->{mode} == 99999)
          {
            unlink $file->{path};
          }
        else
          {
            chmod $file->{mode},$file->{path} if $file->{mode};
            open IN,"$bkppath/$file->{saved}";
            open OUT,">$file->{path}";
            print OUT while <IN>;
            close IN;
            close OUT;
            unlink "$bkppath/$file->{saved}";
            chmod $file->{mode},$file->{path} if $file->{mode};
          }
      }
    if ($data->{$mta}->{restart})
      {
        system($data->{$mta}->{restart});
      }
    delete $data->{$mta};

    if ($mta =~ /sendmail/)
      {
        kas_milter_monitoring("no");
      }
  }

sub unconfig_all
  {
    foreach my $mta(keys %{$data})
      {
        print "unconfiguring KAS for $mta....\n";
        unconfig_mta($mta);
        print "done\n";
      }
  }

1;
