#!/usr/bin/perl

use FindBin qw($Bin);
require "$Bin/config-lib.pl";
use strict;
#0. Check configuration
my $pcf= $ARGV[0];

unless ($pcf && -r $pcf)
  {
    print "Usage: $0 /path/to/postfix/master.cf\n";
    exit 1;
  }

my $s;

unless ($< == 0) {
 print "$0 should be run as root\n";
 exit 1;
}

$|=1;
RETRY:
unless ( -r $pcf ) {
RETR2:
print "Cannot find postfix configuration file (master.cf).\n";
print "Please locate it manually.\n";
print "\n";
print "Enter full path including directory and file name (press Enter to quit):\n";
 $pcf=<>;
 chomp $pcf;
 exit 1 if length $pcf<2;
 goto RETRY;
}
my $ok=0;
my $hascf=0;
my %ports;
my $oport=9025;
my $lport=9026;
my $haskav=0;
my $kavport=0;


open F, $pcf;
L1:
while (<F>)
{
  $s=$_;
  if($s=~/(KASPERSKY\s+ANTI-SPAM\s+BEGIN)|(SPAMTEST\s+BEGIN)/)
     {
       while($s=<F>)
	 {
	   last if $s=~/(KASPERSKY\s+ANTI-SPAM\s+END)|(SPAMTEST\s+END)/;
	 }
       next L1;
     }
  if ($s=~/## Added by Kaspersky Anti-Virus Installer ##/)
    {
      $haskav=1;
      while ($s=<F>)
	{
	  $kavport = $1 if ($s=~/content_filter=lmtp:127.0.0.1:(\d+)/);
	  last if $s=~/## Added by Kaspersky Anti-Virus Installer ##/;
	}
    }

  if (/^\s*#/)
    {
      next;
    }

 $ok=1 if $s=~/^proxymap\s+/;
 $ok=1 if $s=~/^pickup\s+/;
 $hascf=1 if $s=~/content_filter=\S/;

 $ports{$1}=1 if $s=~/^(\d+)\s+inet/;
 $ports{$1}=1 if $s=~/^.*:(\d+)\s+inet/;
 $ports{$1}=1 if $s=~/^.*:(\S+)\s+inet/;
}
unless ($ok)
{
  print "$pcf is not a Postfix configuration file\n";
  goto RETR2;
}

if($hascf || ($haskav && !$kavport)) 
  {
    print "Your Postfix MTA is already configured with content filtering feature.\n";
    print "It is not possible to automate configuration process.\n";
    print "Please, read KAS documentation and configure postfix yourself\n";
    exit 1;
}
for my $srv  (keys %ports)
  {
    next if $srv=~/^\d+$/;
    my @a = getservbyname($srv,"tcp");
    $ports{$a[3]}=1;
  }


while(exists $ports{$oport} || exists $ports{$lport} && $oport <65000)
  {
    $oport++;
    $lport++;
  }
if($oport >= 65000)
  {
    print "Cannot find two free TCP ports for KAS services\n";
    exit 1;
  }

init_backup_data();

my $backuped = mta_backuped('postfix');
submit_orig_file('postfix',$pcf) unless ($backuped);

if ($haskav && $kavport)
  {
    make_pipe_conf(1,1,0,$kavport);
  }
else
  {
    make_pipe_conf(0,0,1,$oport);
  }



# 2. Patching master.cf
print "Editing master.cf file...";

open IN,"$pcf" or die;
open OUT,">$pcf.tmp" or die;

my $old_mode = ((stat $pcf)[2] & 0777) || 0644;
chmod $old_mode,"$pcf.tmp";

my $proto = "kas3scan";
if ($haskav && $kavport)
  {
    $proto="lmtp";
  }

LOOP:
while(<IN>)
{
  $s=$_;

  # comment-out KAV-lines
  if ($haskav && $kavport && $s=~/## Added by Kaspersky Anti-Virus Installer ##/)
    {
      print OUT $s;
   INLOOP:
     while ($s=<IN>)
     {
       if ($s=~/## Added by Kaspersky Anti-Virus Installer ##/)
	 {
	   print OUT $s;
	   last INLOOP;
	 }
       else
	 {
	   print OUT "# $s";
	 }
     }
     next;
    }
  # reinstallation
  if($s=~/(KASPERSKY\s+ANTI-SPAM\s+BEGIN)|(SPAMTEST\s+BEGIN)/)
     {
      while($s=<IN>)
      {
	last if $s=~/(KASPERSKY\s+ANTI-SPAM\s+END)|(SPAMTEST\s+END)/;
      }
      next LOOP;
     }
  if($s=~/^\s*#/)
     {
      print OUT $s;
      next;
     }

  if($s=~/(^smtp\s+inet\s)|(^25\s+inet\s)|(^pickup\s+fifo)|(^\S+:25)|(^\S+:smtp)/)
  {
    print OUT $s;
    print OUT "### KASPERSKY ANTI-SPAM  BEGIN ###\n";
    print OUT "  -o content_filter=$proto:127.0.0.1:$lport\n";
    print OUT "### KASPERSKY ANTI-SPAM  END ###\n";
    next;
  }
  print OUT $s;
}
print OUT "### KASPERSKY ANTI-SPAM  BEGIN ###\n";
print OUT "127.0.0.1:$lport inet n  n       n       -       20      spawn\n";
print OUT "   user=mailflt3 argv=/usr/local/ap-mailfilter3/bin/kas-pipe\n";
if (!$haskav || !$kavport)
  {
print OUT "127.0.0.1:$oport inet    n       -       n       -       25      smtpd\n";
print OUT <<EOM;
    -o smtpd_authorized_xforward_hosts=127.0.0.0/8
    -o local_recipient_maps=
    -o relay_recipient_maps=
    -o smtpd_restriction_classes=
    -o smtpd_client_restrictions=
    -o smtpd_helo_restrictions=
    -o smtpd_sender_restrictions=
    -o smtpd_recipient_restrictions=permit_mynetworks,reject
    -o mynetworks=127.0.0.0/8
    -o strict_rfc821_envelopes=no
    -o smtpd_error_sleep_time=0
    -o smtpd_soft_error_limit=1001
    -o smtpd_hard_error_limit=1000
    -o receive_override_options=no_address_mappings

kas3scan      unix  -       -       n       -       10      smtp
        -o smtp_send_xforward_command=yes
EOM
  }
print OUT "### KASPERSKY ANTI-SPAM  END ###\n";
close IN; close OUT;
rename $pcf,"$pcf.save" or die;
rename "$pcf.tmp",$pcf or die;

submit_changed_file('postfix',$pcf) unless $backuped;
print "done\n";

my $rpath=undef;

if ( -x "/etc/rc.d/init.d/postfix" )
  { 
    $rpath = "/etc/rc.d/init.d/postfix restart";
  } 
elsif (-x "/etc/init.d/postfix")
  {
  $rpath ="/etc/init.d/postfix restart";
  }
elsif (-x "/etc/rc.d/postfix")
  {
  $rpath = "/etc/rc.d/postfix restart";
  }
elsif (-x "/usr/local/etc/rc.d/postfix.sh")
  {
  $rpath = "/usr/local/etc/rc.d/postfix.sh stop; sleep 1; /usr/local/etc/rc.d/postfix.sh start";
  }
elsif (-x "/usr/local/sbin/postfix")
  {
    $rpath = "/usr/local/sbin/postfix reload";
  }
elsif (-x "/usr/sbin/postfix")
  {
    $rpath = "/usr/sbin/postfix reload";
  }
else
  {
    $rpath = "postfix reload";
  }
if ($rpath)
  {
    submit_restart('postfix',$rpath);
    system($rpath);
  }
save_backup() unless $backuped;
