#!/usr/bin/perl

use FindBin qw($Bin);
require "$Bin/config-lib.pl";

# 0. Check configuration
my $smcf= $ARGV[0];
my $smmc=$ARGV[1];

unless($smcf && -r $smcf)
  {
    print <<EOM;
Usage: $0 /path/to/sendmail.cf [/path/to/sendmail.mc]
EOM
    exit 1;
  }


unless (-r $smmc)
  {
    $smmc = $smcf;
    $smmc=~s/\.cf$/\.mc/;
  }

unless ($< == 0) {
 print "$0 should be run as root\n";
 exit 1;
}

$|=1;
RETRY:
unless ( -r $smcf ) {
RETR2:
 print "Cannot find your sendmail.cf configuration file $smcf\n";
 print "Please enter path to it (enter to quit):";
 $smcf=<STDIN>;
 chomp $smcf;
 exit 1 if length $smcf<2;
 goto RETRY;
}
$ok=0;
open F, $smcf;
while (<F>)
{
 $ok=1 if /#\s+SENDMAIL CONFIGURATION FILE/;

}
close F;
unless ($ok)
{
  print "$smcf is not a Sendmail configuration file\n";
  goto RETR2;
}

# 1. Start kas-milter and add to startup
if ( -x "/sbin/chkconfig" ) {
 # RH 7-9 or compatible
 system ("/sbin/chkconfig --add kas3-milter");

 my $milter_script = "";

 if ( -f "/etc/rc.d/init.d/kas3-milter" ) 
   {
     $milter_script = "/etc/rc.d/init.d/kas3-milter";
   } 
 elsif ( -f "/etc/init.d/kas3-milter" ) 
  {
    $milter_script = "/etc/init.d/kas3-milter";
  } else
    {
      print "Error: couldn't find kas3-milter startup script, install it in startup directory and try again.";
      exit(1);
    }


 if ( -f "/var/lock/subsys/kas3-milter" ) {
   system ("$milter_script restart");
 } else {
   system ("$milter_script start");
 }
 
}else {
 # manual installation of startup scripts
  my $uname=`uname`;
  if ( $uname=~/FreeBSD/i && -d "/usr/local/etc/rc.d")
    {
      system("/usr/bin/install -o root -m 0755 /usr/local/ap-mailfilter3/bin/scripts/kas3-milter.simple-init /usr/local/etc/rc.d/kas3-milter.sh");
      system "/usr/local/etc/rc.d/kas3-milter.sh restart"
    }
  elsif ( -f "/etc/init.d/functions" )
    {
      # RH-compatible init
      system("/usr/bin/install -o root -m 0755 /usr/local/ap-mailfilter3/bin/scripts/kas3-milter.redhat-init /etc/init.d/kas3-milter");
      if (-x "/sbin/chkconfig")
	{
	  system("/sbin/chkconfig --add kas3-milter");
	}
      else
	{
	  symlink "/etc/init.d/kas3-milter","/etc/rc3.d/S30kas3-milter";
	  symlink "/etc/init.d/kas3-milter","/etc/rc3.d/K30kas3-milter";
	}
      system "/etc/init.d/kas3-milter restart";
    } 
  elsif (-d "/etc/init.d" && -f "/etc/rc.status")
    {
      # SuSe
      system("/usr/bin/install -o root -m 0755 /usr/local/ap-mailfilter3/bin/scripts/kas3-milter.suse-init /etc/init.d/kas3-milter");
      if (-x "/sbin/chkconfig")
	{
	  system("/sbin/chkconfig --add kas3-milter");
	}
      else
	{
	  symlink "/etc/init.d/kas3-milter","/etc/rc3.d/S30kas3-milter";
	  symlink "/etc/init.d/kas3-milter","/etc/rc3.d/K30kas3-milter";
	}
      system "/etc/init.d/kas3-milter restart";
    }
  elsif (-d "/etc/init.d"  && -x "/sbin/start-stop-daemon" )
    {
      # debian
      system("/usr/bin/install -o root -m 0755 /usr/local/ap-mailfilter3/bin/scripts/kas3-milter.debian-init /etc/init.d/kas3-milter");
      system "/etc/init.d/kas3-milter restart";
      if ( -x "/usr/sbin/update-rc.d" )
	{
	  system("/usr/sbin/update-rc.d kas3-milter defaults");
	}
      else
	{
	  symlink "/etc/init.d/kas3-milter","/etc/rc3.d/S30kas3-milter";
	}
    }
  elsif ( -d "/etc/init.d" && -d "/etc/rc3.d" ) 
    {
      system("/usr/bin/install -o root -m 0755 /usr/local/ap-mailfilter3/bin/scripts/kas3-milter.simple-init /etc/init.d/kas3-milter");
      system "/etc/init.d/kas3-milter restart";
      symlink "/etc/init.d/kas3-milter","/etc/rc3.d/S30kas3-milter";
    } 
  elsif ( -d "/etc/rc.d" && -d "/etc/rc.d/rc3.d" ) 
    {
      system("/usr/bin/install -o root -m 0755 /usr/local/ap-mailfilter3/bin/scripts/kas3-milter.simple-init /etc/rc.d/kas3-milter");
      symlink "/etc/rc.d/kas3-milter","/etc/rc.d/rc3.d/S30kas3-milter";
      system "/etc/rc.d/kas3-milter restart"
    }
}

#_ 2. Patch /etc/mail/sendmail.cf

init_backup_data();
my $backuped = mta_backuped('sendmail');
unless ($backuped)
  {
    submit_orig_file('sendmail',$smcf);
  }

print "Patching sendmail config $smcf...";
$configured = 0;
$xc=0;
$v10=0;
open IN,"$smcf" or die;
open OUT,">$smcf.tmp" or die ;

my $old_mode = ((stat $smcf)[2] & 0777) || 0644;
chmod $old_mode,"$smcf.tmp";

while (<IN>)
{
 $s=$_;
 $v10=1 if($s=~/^V10/);
 
 if ($s=~/^V/ && !$xc) {
  print OUT $s;
  print OUT "Xkasfilter, S=local:/var/run/kas-milter.socket, T=C:10s;S:20s;R:30s\n";
#  print OUT "O Milter.LogLevel = 1\n" if $v10;
  $xc=1;
  next;
  }
 if($s=~/^Xkasfilter/) { next; }
 if($s=~/Milter.LogLevel/i) { next; }

 if($s=~/^O\s+InputMailFilters/ && !$configured){ 
  chomp $s;
  if($s=~/kasfilter/) {
    $s.="\n";
  } else {
    $s.=",kasfilter\n"; 
  }
  print OUT $s;
  $configured=1;
  next;
 }
 print OUT $s;
}
print OUT "Xkasfilter, S=local:/var/run/kas-milter.socket, T=C:10s;S:20s;R:30s\n" unless $xc;
print OUT "O InputMailFilters=kasfilter\n" unless $configured;
#print OUT "O Milter.LogLevel = 1\n" if ! $xc && $v10;
close IN;
close OUT;
print "done\n";
rename $smcf,"$smcf.save";
rename "$smcf.tmp",$smcf;
print "Your old $smcf saved as $smcf.save\n";

my $restart = undef;

if (-f "/etc/rc.sendmail")
  {
    $restart = "sh /etc/rc.sendmail restart";
  }
elsif ( -f "/etc/rc.d/init.d/sendmail" ) 
  {
    $restart = "/etc/rc.d/init.d/sendmail restart";
  } 
elsif ( -f "/etc/init.d/sendmail" ) 
  {
    $restart = "/etc/init.d/sendmail restart";
  } 

if ($restart)
  {
    submit_restart('sendmail',$restart);
    system($restart);
  }
else 
  {
    print "Cannot restart your sendmail, please restart it yourself\n";
  }

submit_changed_file('sendmail',$smcf) unless $backuped;

# 3. patch sendmail.mc
if ( -r $smmc ) 
  {
    submit_orig_file('sendmail',$smmc);
    print "Patching $smcf template file..";
    open IN, $smmc;
    open OUT, ">$smmc.tmp";
    $done=0;
    while(<IN>)
      {
        $s=$_;
        if (/^FEATURE/ && !$done)
          {
            print OUT <<EOM;
## KASPERSKY ANTI-SPAM BEGIN
INPUT_MAIL_FILTER(`kasfilter',`S=local:/var/run/kas-milter.socket, T=C:10s;S:20s;R:30s')
define(`confINPUT_MAIL_FILTERS',`kasfilter')
EOM
print OUT "define(\`confMILTER_LOG_LEVEL\',\`1\')\n" if $v10;
print OUT "## KASPERSKY ANTI-SPAM END\n";
            $done=1;
          }
        if (/^INPUT_MAIL_FILTER.*kasfilter/) { next; };
        if (/confINPUT_MAIL_FILTERS.*kasfilter/) { next; };
        if (/confMILTER_LOG_LEVEL/) { next; };
        print OUT $s;
      }
    close IN;
    close OUT;
    print "done\n";
    rename $smmc,"$smmc.save";
    rename "$smmc.tmp",$smmc;
    submit_changed_file('sendmail',$smmc) unless $backuped;
}
save_backup() unless $backuped;

# 4. Edit filter.conf

kas_milter_monitoring("yes");


