#!/bin/sh

. `dirname $0`/scripts-common.sh

LOG_TAG=enable-updates.sh

#############################################################################
print_usage() {
  echo 'Script for enabling automatic updates and UDS'
  echo ''
  echo 'Usage:'
  echo 'enable-updates.sh [-qdvlh] [-V verbose_level] [-L log_level] [-c config_file] [-k kas-conf_utility]'
  echo '    -q   -- print only error messages (min. verbose level)'
  echo '    -d   -- print all debug messages (max. verbose level)'
  echo '    -v   -- increase verbose level'
  echo '    -V n -- set verbose level to n'
  echo '    -l   -- increase log level (used for syslog messages)'
  echo '    -L n -- set log level to n'
  echo '    -c   -- config file to use'
  echo '    -k   -- full path to kas-conf utility'
  echo '    -h   -- print this help'
  echo ''
}

#############################################################################
# get and check options

while getopts qdvlhV:L:c:k: OPT ; do
case $OPT in
  q) disp_verbose_set 0 ;;
  d) disp_verbose_set 5 ;;
  v) disp_verbose_increase ;;
  l) log_verbose_increase;;
  V) disp_verbose_set $OPTARG ;;
  L) log_verbose_set $OPTARG ;;
  c) CONFFILE=$OPTARG ;;
  k) KASCONF=$OPTARG ;;

  h) print_usage
     if [ $# -eq 1 ] && [ $1 = '-h' ] ;
         then exit 0
         else exit 1
     fi
     ;;

esac
done

if [ $# -ge $OPTIND ] ; then
  eecho "Invalid arguments"
  print_usage
  exit 1
fi

adjust_tunables

#############################################################################
# check user
check_user $0 "$*"

get_path_name KASINITCONF initconf
check_executable $KASINITCONF
get_path_name UDSRTTS uds-rtts-sh
check_executable $UDSRTTS

iecho "Enabling UDS..."
ed -s $CONFFILE <<EOF 1>/dev/null 2>/dev/null
,s/^ *FilterUDSEnabled/# FilterUDSEnabled/
w
EOF
$UDSRTTS

iecho "Enabling automatic updates..."

$KASINITCONF -c

iecho "============================================================="
iecho "You can adjust automatic updates settings via control center."
iecho "============================================================="

finish "Automatic updates and UDS are now enabled."