#
# Default parameters for all KAS 3.x scripts
#

ROOTDIR=/usr/local/ap-mailfilter3
CONFFILE=$ROOTDIR/etc/filter.conf
KASCONF=$ROOTDIR/bin/kas-conf

#
# Logger settings
#
# Tunables are:
#   LOGGER, LOG_TAG, LOG_FACILITY, USE_LOGGER,
#   LOG_VERBOSE, DISP_VERBOSE, PREPEND_MY_MESSAGES
#
# Defaults for them are DEFAULT_ variables.
#
# Have to call adjust_tunables after command line parsing.
#

DEFAULT_LOG_TAG=kas-script
DEFAULT_LOG_FACILITY=mail
DEFAULT_LOG_VERBOSE=2
DEFAULT_LOGGER=logger
DEFAULT_USE_LOGGER=1

DEFAULT_DISP_VERBOSE=3
DEFAULT_PREPEND_MY_MESSAGES=0

#
# User and group of KAS 
#
# Tunables are KAS_USER and KAS_GROUP
#

DEFAULT_KAS_USER=mailflt3
DEFAULT_KAS_GROUP=mailflt3

#
# Pid files for services
#
# Default values
#

PID_FILE_SERVER=$ROOTDIR/run/ap-process-server.pid
PID_FILE_LICENSE=$ROOTDIR/run/kas-license.pid
PID_FILE_SPFD=$ROOTDIR/run/ap-spfd.pid
PID_FILE_MILTER=$ROOTDIR/run/kas-milter.pid
PID_FILE_HTTPD=$ROOTDIR/run/kas-thttpd.pid

write_log() {
  if [ "$USE_LOGGER" = "1" ] ; then
    eval "$LOGGER $* 2>&1"
    if [ $? -ne 0 ] ; then
      USE_LOGGER=0
      wecho "Error while sending message to syslog: $RET; syslog messages are disabled"
    else
      USE_LOGGER=2
    fi
  elif [ "$USE_LOGGER" = "2" ] ; then
    eval "$LOGGER $*"
  fi
}

eecho() {
  write_log "-p $LOG_FACILITY.error -t $LOG_TAG \"KASERROR $*\""
  echo "ERROR: $*" >&2
}

wecho() {
  if [ 0$LOG_VERBOSE -ge 1 ] ; then
     write_log "-p $LOG_FACILITY.warning -t $LOG_TAG \"KASWARNING $*\""
  fi
  if [ 0$DISP_VERBOSE -ge 1 ] ; then
    echo "WARNING: $*" >&2
  fi
}

necho() {
  if [ 0$LOG_VERBOSE -ge 2 ] ; then
    write_log "-p $LOG_FACILITY.info -t $LOG_TAG \"KASNOTICE $*\""
  fi

  if [ 0$DISP_VERBOSE -ge 2 ] ; then
    if [ 0$PREPEND_MY_MESSAGES -eq 1 ] ; then
      echo "NOTICE: $*"
    else
      echo "$*"
    fi
  fi
}

iecho() {
  if [ 0$LOG_VERBOSE -ge 3 ] ; then
    write_log "-p $LOG_FACILITY.info -t $LOG_TAG \"KASINFO $*\""
  fi

  if [ 0$DISP_VERBOSE -ge 3 ] ; then
    if [ 0$PREPEND_MY_MESSAGES -eq 1 ] ; then
      echo "INFO: $*"
    else
      echo "$*"
    fi
  fi
}

decho() {
  if [ 0$LOG_VERBOSE -ge 4 ] ; then
    write_log "-p $LOG_FACILITY.debug -t $LOG_TAG \"KASDEBUG $*\""
  fi

  if [ 0$DISP_VERBOSE -ge 4 ] ; then
    echo "DEBUG: $*"
  fi
}

decho2() {
  if [ 0$LOG_VERBOSE -ge 5 ] ; then
    write_log "-p $LOG_FACILITY.debug -t $LOG_TAG \"KASDEBUG $*\""
  fi

  if [ 0$DISP_VERBOSE -ge 5 ] ; then
    echo "DEBUG: $*"
  fi
}

log_verbose_increase() {
  if [ x$LOG_VERBOSE = x ] ; then LOG_VERBOSE=$DEFAULT_LOG_VERBOSE; fi
  LOG_VERBOSE=`expr $LOG_VERBOSE + 1`
}

log_verbose_set() {
  LOG_VERBOSE=$1
}

disp_verbose_increase() {
  if [ x$DISP_VERBOSE = x ] ; then DISP_VERBOSE=$DEFAULT_DISP_VERBOSE; fi

  DISP_VERBOSE=`expr $DISP_VERBOSE + 1`

  if [ $DISP_VERBOSE -ge 4 ] ; then
    PREPEND_MY_MESSAGES=1
  fi
}

disp_verbose_set() {
  DISP_VERBOSE=$1
  if [ ! x$DISP_VERBOSE = x0 ] && [ ! x$DISP_VERBOSE = x1 ] && [ ! x$DISP_VERBOSE = x2 ] && [ ! x$DISP_VERBOSE = x3 ] && [ ! x$DISP_VERBOSE = x4 ] && [ ! x$DISP_VERBOSE = x5 ] && [ ! x$DISP_VERBOSE = x6 ] && [ ! x$DISP_VERBOSE = x7 ] && [ ! x$DISP_VERBOSE = x8 ] && [ ! x$DISP_VERBOSE = x9 ] && [ ! x$DISP_VERBOSE = x10 ]; then
    echo "invalid options"
    exit 1
  fi
  if [ $DISP_VERBOSE -ge 4 ] ; then
    PREPEND_MY_MESSAGES=1
  fi
}

#
# Exit functions
#


ON_EXIT=""

fail() {
  ret=$1

  if [ x"$ON_EXIT" != x"" ] ; then
    decho "on exit: $ON_EXIT"
    `$ON_EXIT`
  fi

  release_all_locks

  if [ x"$ret" = x"" ] ; then
    ret=1
  fi
  decho "FAILED with code $ret"
  exit $ret  
}

finish() {

  if [ x"$ON_EXIT" != x"" ] ; then
    decho "on exit: $ON_EXIT"
    `$ON_EXIT`
  fi

  release_all_locks

  if [ x"$1" != x"" ] ; then
    necho "$1"
  fi
  exit 0 
}


kas_user_set() {
  if [ x$1 = x ] ; then 
    KAS_USER=$DEFAULT_KAS_USER
  else
    KAS_USER=$1
  fi
  KAS_UID=`id -u $KAS_USER`

  if [ $? -gt 0 ] ; then
    eecho "Couldn't find filter user $KAS_USER account"
    fail 1
  fi
}

kas_group_set() {
  if [ x$1 = x ] ; then 
    if [ x$KAS_USER != x ] ; then 
      KAS_GROUP=`id -gn $KAS_USER`
      KAS_GID=`id -g $KAS_USER`
      return 0
    else
      KAS_GROUP=$DEFAULT_KAS_GROUP
    fi
  else
    KAS_GROUP=$1
  fi
  KAS_GID=`grep ^$KAS_GROUP: /etc/group | cut -d ':' -f 3`

#  if [ $? -gt 0 ] ; then
#    eecho "Couldn't find filter group $KAS_GROUP account"
#    fail 1
#  fi
}

#
# User check and restart of the script
#

check_user() {
  
  decho "check_user: start"
 
  UNAME=`uname`
  MYUID=`id -u`
  decho "Current UID is $MYUID"

  if [ 0$MYUID -eq 0 ] ; then 
    iecho "Restarting as $KAS_USER"
    if [ x"$UNAME" = x"FreeBSD" ] ; then
      su -m $KAS_USER -c "$*"
    else
      su -s /bin/sh $KAS_USER -c "$*"
    fi
    exit $?
  else
    if [ 0$MYUID -ne 0$KAS_UID ] ; then
      if [ x"$UNAME" = x"FreeBSD" ] ; then
        eecho "Can't switch to $KAS_USER user (run su -m $KAS_USER -c '$1 [options]')"
      else
        eecho "Can't switch to $KAS_USER user (run su -s /bin/sh $KAS_USER -c '$1 [options]')"
      fi
      fail 1
    fi
  fi
  decho "check_user: finish"
}

#
# Check for process existence 
#

is_pid_running() {
  local LOCKER_PID=$1
  ps $LOCKER_PID > /dev/null
  if [ $? -eq 0 ] ; then
    return 0
  else
    return 1
  fi
}

#
# Lock and unlock lock-file
#

DEFAULT_LOCK_STRICT=0
# Additional tries (not including first try)
DEFAULT_LOCK_TRIES=2

OBTAINED_LOCKS=""

get_lock() {

  local LOCK_FILE=$1
  local NTRY=0
  decho "get_lock: in, LOCK_FILE=$LOCK_FILE"

  while [ 0$NTRY -le 0$LOCK_TRIES ] ; do

    [ $NTRY -gt 0 ] && {
      sleep 1
      decho "get_lock: try number $NTRY"
    }

    NTRY=`expr $NTRY + 1`

    if [ -e $LOCK_FILE ] ; then 

      decho "get_lock: lock file $LOCK_FILE exist."

      if [ 0$LOCK_STRICT -eq 1 ] ; then
        eecho "get_lock: couldn't obtain lock $LOCK_FILE, file already exist: remove it."
        if [ 0$NTRY -le $LOCK_TRIES ] ; then
          continue
        else
          return 1
        fi
      fi

      local LOCKER_PID=`cat $LOCK_FILE`;
      if [ x$LOCKER_PID = x ]; then
        iecho "lock file is empty. removing it"
        break
      fi

      if is_pid_running `cat $LOCK_FILE` ; then 
        iecho "get_lock: couldn't obtain lock $LOCK_FILE, process with pid $LOCKER_PID already have it"
        
        if [ 0$NTRY -le $LOCK_TRIES ] ; then
          continue
        else
          return 1
        fi
      fi
    fi

    break

  done
 
  echo -n "$$" > $LOCK_FILE
  if [ $? -ne 0 ] ; then
    eecho "get_lock: couldn't create lock $LOCK_FILE."
    return 1
  fi

  decho "get_lock: lock $LOCK_FILE obtained."

  OBTAINED_LOCKS="$LOCK_FILE $OBTAINED_LOCKS"

  return 0

}

release_lock() {

  local LOCK_FILE=$1
  decho "release_lock: in, LOCK_FILE=$LOCK_FILE"

  local HAS_LOCK=`echo $OBTAINED_LOCKS | grep $LOCK_FILE | wc -l | awk '{ print $1; }'`
  if [ 0$HAS_LOCK -eq 0 ] ; then
    wecho "release_lock: request for release lock, which wasn't obtained"
  fi

  if [ ! -f $LOCK_FILE ] ; then
    wecho "release_lock: can't find lock file $LOCK_FILE."
    # Trying to remove it anyway, may be it is directory?
    rm -rf $LOCK_FILE
    return 1
  fi

  local LOCKER_PID=`cat $LOCK_FILE`

  if [ $$ -ne 0$LOCKER_PID ] ; then
    wecho "release_lock: lock $LOCK_FILE locked by another process with pid $LOCKER_PID."
    is_pid_running $LOCKER_PID || {
      iecho "release_lock: process $LOCKER_PID doesn't exist, removing lock anyway."
      rm -rf $LOCK_FILE
    }
    return 1
  fi

  rm -r $LOCK_FILE
  if [ $? -ne 0 ] ; then
    wecho "release_lock: couldn't delete lock $LOCK_FILE"
    return 1
  fi

  decho "release_lock: lock $LOCK_FILE released."

  if [ 0$HAS_LOCK -ne 0 ] ; then
    OBTAINED_LOCKS=`echo $OBTAINED_LOCKS | sed "s+$LOCK_FILE++"`
  fi

  return 0
}

release_all_locks() {

  decho "release_all_locks: in"

  for LOCK in $OBTAINED_LOCKS ; do 
    release_lock $LOCK
  done

  decho "release_all_locks: done"
}

#
# Work with kas-conf utility
#

get_path_name() {
  eval $1=`$KASCONF -c $CONFFILE -p $2`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't get value of $1."
    fail
  fi

  eval local GOT=\$$1
  if [ x$GOT = x ] ; then
    eecho "Couldn't get value of $1."
    fail
  fi

  eval decho "$1=\$$1"
}

get_server_pid() {
  local GOT=`$KASCONF -c $CONFFILE -a ServerPidFile 2&> /dev/null`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't read $CONFFILE"
    fail
  fi

  if [ x$GOT = x ] ; then
    local PID_FILE
    get_path_name PID_FILE process-server-pid
    if [ x$PID_FILE = x ] ; then
      wecho "Server pid file name is not defined, using default"
    else
      PID_FILE_SERVER=$PID_FILE
    fi
  else
    PID_FILE_SERVER=$GOT
  fi
  decho "PID_FILE_SERVER=$PID_FILE_SERVER"
}

get_license_pid() {
  local GOT=`$KASCONF -c $CONFFILE -a LicensePidFile 2&> /dev/null`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't read $CONFFILE"
    fail
  fi

  if [ x$GOT = x ] ; then
    local PID_FILE
    get_path_name PID_FILE kas-license-pid
    if [ x$PID_FILE = x ] ; then
      wecho "License pid file name is not defined, using default"
    else
      PID_FILE_LICENSE=$PID_FILE
    fi
  else
    PID_FILE_LICENSE=$GOT
  fi
  decho "PID_FILE_LICENSE=$PID_FILE_LICENSE"
}

get_milter_pid() {
  local GOT=`$KASCONF -c $CONFFILE -a SendMailPIDFile 2&> /dev/null`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't read $CONFFILE"
    fail
  fi

  if [ x$GOT = x ] ; then
    local PID_FILE
    get_path_name PID_FILE milter-pid
    if [ x$PID_FILE = x ] ; then
      wecho "kas-milter pid file name is not defined, using default"
    else
      PID_FILE_MILTER=$PID_FILE
    fi
  else
    PID_FILE_MILTER=$GOT
  fi
  decho "PID_FILE_MILTER=$PID_FILE_MILTER"
}

get_httpd_pid() {
  local GOT=`$KASCONF -c $CONFFILE -a HTTPServerPIDFile 2&> /dev/null`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't read $CONFFILE"
    fail
  fi

  if [ x$GOT = x ] ; then
    local PID_FILE
    get_path_name PID_FILE httpd-pid
    if [ x$PID_FILE = x ] ; then
      wecho "kas-thttpd pid file name is not defined, using default"
    else
      PID_FILE_HTTPD=$PID_FILE
    fi
  else
    PID_FILE_HTTPD=$GOT
  fi
  decho "PID_FILE_HTTPD=$PID_FILE_HTTPD"
}

get_spfd_pid() {
  local GOT=`$KASCONF -c $CONFFILE -a SPFDPidFile 2&> /dev/null`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't get spfd pid file."
    fail
  fi

  if [ x$GOT = x ] ; then
    necho "SPFD pid file name (SPFDPidFile) is not defined."
  else
    PID_FILE_SPFD=$GOT
  fi
}

get_keepup2date_ini() {
  local GOT=`$KASCONF -c $CONFFILE -a UpdaterIniPath 2&> /dev/null`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't read $CONFFILE"
    fail
  fi

  if [ x$GOT = x ] ; then
    get_path_name GOT updater-ini
    if [ x$GOT = x ] ; then
      eecho "keepup2date conf file name is not defined"
      fail
    fi
  fi
  eval $1=$GOT
  eval decho "$1=\$$1"
}

get_httpd_ini() {
  local GOT=`$KASCONF -c $CONFFILE -a HTTPServerIniPath 2&> /dev/null`
  if [ $? -ne 0 ] ; then
    eecho "Couldn't read $CONFFILE"
    fail
  fi

  if [ x$GOT = x ] ; then
    get_path_name GOT thttpd-ini
    if [ x$GOT = x ] ; then
      eecho "kas-thttpd conf file name is not defined"
      fail
    fi
  fi
  eval $1=$GOT
  eval decho "$1=\$$1"
}

#
# Create output directories and adjust access rights
#

create_dir() {
  DIR=$1

  if [ ! -d $DIR ] ; then

    rm -rf $DIR || {
      eecho "Couldn't erase file $DIR"
      fail
    }

    mkdir -m 0755 $DIR || {
       eecho "Couldn't create directory $DIR"
       fail
    }

  fi
}

#
# File checks
#

check_readable()
{
  if [ x$1 = x ] ; then 
    eecho "check_readable: wrong parameters"
    fail 1
  fi

  if [ ! -r $1 ] ; then
    eecho "Couldn't access file: $1"
    fail 1
  fi

}

check_executable()
{
  if [ x$1 = x ] ; then 
    eecho "check_executable: wrong parameters"
    fail 1
  fi

  if [ ! -x $1 ] ; then
    eecho "can't find or access $1"
    fail 1
  fi

}

check_directory()
{
  if [ x$1 = x ] ; then 
    eecho "check_directory: wrong parameters"
    fail 1
  fi

  if [ ! -d $1 ] ; then
    eecho "Directory $1 does not exist"
    fail 1
  fi

}

#
# "Sure" wrappers
#

chdir_sure() {

  decho "chdir to $1"
  cd $1
  if [ $? -ne 0 ] ; then
    eecho "can't change dir to $1"
    fail 1
  fi

}

mv_sure() {

  decho "mv from $1 to $2"
  mv -f $1 $2
  if [ $? -ne 0 ] ; then
    eecho "can't move $1 to $2"
    fail 1
  fi
}

cp_sure() {

  decho "cp from $1 to $2"
  cp -f $1 $2
  if [ $? -ne 0 ] ; then
    eecho "can't cp $1 to $2"
    fail 1
  fi
}

cp_sure_r() {

  decho "recursive copy from $1 to $2"
  local CURDIR=`pwd`

  cd $1 || {
    eecho "cp_sure_r: can't cd tp $1"
    fail 1
  }

  tar cf - . | ( cd $2 ; tar xf -) || {
    eecho "cp_sure_r: can't cp contents of $1 to $2"
    fail 1
  }
  
}

#
# Remove all files in directory (but left subdirectories as is)
#
dir_cleanup() {
  local DIR=$1

  decho "Cleaning dir $DIR"

  find $DIR -type f -exec rm -f {} \;
  if [ 0`find $DIR -type f | wc -c | awk '{ print $1; }'` -gt 0 ] ; then
    eecho "Couldn't clear directory $DIR"
    fail 1
  fi

  decho "All files in $DIR are removed"

  return 0
}

#
# After correct initalisation (locating kas-conf utility)
# adjusting all tunables in this file to filter.conf.
#

adjust_local_tunable() {
  TUNABLE=$1
  eval "if [ x\$$TUNABLE = x ] ; then $TUNABLE=\$DEFAULT_$TUNABLE ; fi"
  eval decho $TUNABLE=\$$TUNABLE
}

adjust_global_tunable() {
  TUNABLE=$1
  GLOBALNAME=$2

  eval "if [ x\$$TUNABLE = x ] ; then \
         $TUNABLE=`$KASCONF -c $CONFFILE -a $GLOBALNAME` ; \
         if [ x\$$TUNABLE = x ] ; then \
           $TUNABLE=\$DEFAULT_$TUNABLE ; \
         fi ; \
       fi"

  eval decho $TUNABLE=\$$TUNABLE
}

adjust_tunables() {

  decho "adjust_tunables: start"
  adjust_local_tunable DISP_VERBOSE
  adjust_local_tunable PREPEND_MY_MESSAGES

  decho "KASCONF=$KASCONF"
  decho "CONFFILE=$CONFFILE"

  if [ ! -x $KASCONF ] ; then
    eecho "Couldn't run $KASCONF utility"
    fail
  fi

  if [ ! -r $CONFFILE ] || [ ! -f $CONFFILE ]; then
    eecho "Couldn't read config file $CONFFILE"
    fail
  fi

  ROOTDIR=`$KASCONF -c $CONFFILE -p main-root`
  decho "ROOTDIR=$ROOTDIR"

  if [ x$KAS_USER = x ] ; then
    KAS_USER=`$KASCONF -c $CONFFILE -a User`
  fi
  kas_user_set $KAS_USER
  decho "KAS_USER=$KAS_USER; KAS_UID=$KAS_UID"

  if [ x$KAS_GROUP = x ] ; then
    KAS_GROUP=`$KASCONF -c $CONFFILE -a Group`
  fi
  kas_group_set $KAS_GROUP
  decho "KAS_GROUP=$KAS_GROUP; KAS_GID=$KAS_GID"

  adjust_local_tunable LOCK_STRICT
  adjust_local_tunable LOCK_TRIES

  adjust_global_tunable LOG_FACILITY LogFacility
  adjust_local_tunable LOG_TAG
  adjust_global_tunable LOG_VERBOSE  LogLevel

  adjust_local_tunable LOGGER
  adjust_local_tunable USE_LOGGER
  if [ 0$USE_LOGGER -eq 1 ] ; then 
     which $LOGGER > /dev/null
     if [ $? -ne 0 ] ; then
       USE_LOGGER=0
       wecho "Can't find $LOGGER utility, syslog messages are disabled"
     else
       decho "syslog messages is on"
     fi 
  fi 

  decho "adjust_tunables: finish"
}

#
# EOF
#
