#!/bin/sh

. `dirname $0`/scripts-common.sh

LOG_TAG=uds-rtts.sh
PRINT_RTTS_OUTPUT=no

#############################################################################
print_usage() {
  echo 'Usage:'
  echo 'uds-rtts.sh [-aqdvlh] [-V verbose_level] [-L log_level] [-c config_file] [-k kas-conf_utility]'
  echo '    -a   -- output availability of uds servers'
  echo '    -q   -- print only error messages (min. verbose level)'
  echo '    -d   -- print all debug messages (max. verbose level)'
  echo '    -v   -- increase verbose level'
  echo '    -V n -- set verbose level to n'
  echo '    -l   -- increase log level (used for syslog messages)'
  echo '    -L n -- set log level to n'
  echo '    -c   -- config file to use'
  echo '    -k   -- full path to kas-conf utility'
  echo '    -h   -- print this help'
  echo ''
}

#############################################################################
# get and check options

while getopts qdvlahV:L:c:k: OPT ; do
case $OPT in
  q) disp_verbose_set 0 ;;
  d) disp_verbose_set 5; PRINT_RTTS_OUTPUT=yes ;;
  v) disp_verbose_increase ;;
  l) log_verbose_increase;;
  V) disp_verbose_set $OPTARG ;;
  L) log_verbose_set $OPTARG ;;
  a) PRINT_RTTS_OUTPUT=yes ;;
  c) CONFFILE=$OPTARG ;;
  k) KASCONF=$OPTARG ;;

  h) print_usage
     if [ $# -eq 1 ] && [ $1 = '-h' ] ;
         then exit 0
         else exit 1
     fi
     ;;

esac
done

if [ $# -ge $OPTIND ] ; then
  eecho "Invalid arguments"
  print_usage
  exit 1
fi

adjust_tunables

#############################################################################
# check user
check_user $0 "$*"

#############################################################################
# check executable file

get_path_name UDSRTTS uds-rtts
check_executable $UDSRTTS

UDS_ENABLED=`$KASCONF -c $CONFFILE -a FilterUDSEnabled`
if [ x$UDS_ENABLED = xno ] ; then 
  iecho "UDS is disabled."
  finish
fi

UDS_FILE=`$KASCONF -c $CONFFILE -a FilterUDSCfgFile`
if [ x$UDS_FILE = x ] ; then
    get_path_name UDS_FILE  name-uds
fi

UDS_TIMEOUT=`$KASCONF -c $CONFFILE -a FilterUDSTimeout`
if [ x$UDS_TIMEOUT = x ] ; then 
    UDS_TIMEOUT=10
fi

UDS_TIMEOUT=`expr $UDS_TIMEOUT \* 3`


#############################################################################
# run utility
if [ x$PRINT_RTTS_OUTPUT = xyes ] ; then
$UDSRTTS -t $UDS_TIMEOUT $UDS_FILE
else
$UDSRTTS -q -q -t $UDS_TIMEOUT $UDS_FILE
fi
if [ $? -ne 0 ] ; then
  wecho "uds-rtts failed"
  fail 1
else
  iecho "uds-rtts finished successfully"
  finish
fi

#############################################################################
