#!/usr/local/ap-mailfilter3/bin/perl -w
#######################################################################
# Default paths and constants

use strict;
use FindBin qw($Bin);
use POSIX qw(setlocale);

use vars qw($CONST $ROOT $CONFIG_PATH $CONFIG $LANGS $LANG @ERR $ERRLOG_PATH $LANG_FORCE);

BEGIN {
  require 'root.pl';
  require 'const.pl';
  require 'conffiles.pl';

  srand( time() ^ ($$ + ($$ << 15)) );
};

$CONFIG = {
            ConfigPath        => $CONFIG_PATH,  # Fixed for KAS v3
            RootPath          => $ROOT,
            ErrlogPath        => $ERRLOG_PATH,

            BinRootPath       => undef,
            EtcRootPath       => undef,
            FilterConfigPath  => undef,
            FilterCFDataPath  => undef,
            CCRootPath        => undef,
            RunRootPath       => undef,
            LogsRootPath      => undef,
            FilterBinConfigPath => undef,
            FilterDataConfigPath => undef,
            FilterDefConfigPath => undef,
            FilterLicConfigPath => undef,
            FilterSrcConfigPath => undef,
            FilterTmpConfigPath => undef,
            FilterCFDataMainsetPath => undef,
            FilterPath        => undef,
            FilterStatLogFile => undef,
            FilterUserLogFile => undef,

            User              => $CONST->{user}{mailflt_name},
            Group             => $CONST->{user}{mailflt_name},

            ###
            LogFacility             => $CONST->{defaults}{LogFacility},
            LogLevel                => $CONST->{defaults}{LogLevel},

            ServerMaxFilters        => $CONST->{defaults}{ServerMaxFilters},
            ServerStartFilters      => $CONST->{defaults}{ServerStartFilters},
            ServerSpareFilters      => $CONST->{defaults}{ServerSpareFilters},

            FilterMaxMessages       => $CONST->{defaults}{FilterMaxMessages},
            FilterRandMessages      => $CONST->{defaults}{FilterRandMessages},
            FilterMaxIdle           => $CONST->{defaults}{FilterMaxIdle},
            FilterDelayedExit       => $CONST->{defaults}{FilterDelayedExit},

            FilterDNSTimeout        => $CONST->{defaults}{FilterDNSTimeout},
            FilterReceivedHeadersLimit=> $CONST->{defaults}{FilterReceivedHeadersLimit},
            FilterParseMSOffice     => $CONST->{defaults}{FilterParseMSOffice},
            FilterParseRTF          => $CONST->{defaults}{FilterParseRTF},
            FilterUDSEnabled        => $CONST->{defaults}{FilterUDSEnabled},
            FilterUDSTimeout        => $CONST->{defaults}{FilterUDSTimeout},

            ClientFilteringSizeLimit=> $CONST->{defaults}{ClientFilteringSizeLimit},
            ClientOnError           => $CONST->{defaults}{ClientOnError},
            ClientDefaultDomain     => $CONST->{defaults}{ClientDefaultDomain},
            ClientConnectTimeout    => $CONST->{defaults}{ClientConnectTimeout},
            ClientDataTimeout       => $CONST->{defaults}{ClientDataTimeout},

            AlertsTo                => $CONST->{defaults}{AlertsTo},
            Lang                    => 'en',

            MonitoringHttpd         => $CONST->{defaults}{MonitoringHttpd},
            MonitoringKasMilter     => $CONST->{defaults}{MonitoringKasMilter},

            ###

            CCDontModifyCrontab     => $CONST->{defaults}{CCDontModifyCrontab},
            UseKasDNSBL             => $CONST->{defaults}{UseKasDNSBL},

            # Program modified
            rc_path => undef,
            charset => undef,
            CompilerLog => undef,
          };

#########################
# Read configuration file

ReadConfFile( $CONFIG, $CONFIG->{ConfigPath}, $CONST->{config_map} );

#########################
# Build File path

# Based paths
$ROOT = $CONFIG->{RootPath};
$CONFIG->{BinRootPath}      = "$ROOT/bin"     unless defined($CONFIG->{BinRootPath});
$CONFIG->{EtcRootPath}      = "$ROOT/etc"     unless defined($CONFIG->{EtcRootPath});
$CONFIG->{FilterConfigPath} = "$ROOT/conf"    unless defined($CONFIG->{FilterConfigPath});
$CONFIG->{FilterCFDataPath} = "$ROOT/cfdata"  unless defined($CONFIG->{FilterCFDataPath});
$CONFIG->{CCRootPath}       = "$ROOT/control" unless defined($CONFIG->{CCRootPath});
$CONFIG->{RunRootPath}      = "$ROOT/run"     unless defined($CONFIG->{RunRootPath});
$CONFIG->{LogsRootPath}     = "$ROOT/log"     unless defined($CONFIG->{LogsRootPath});

# Related path
$CONFIG->{FilterBinConfigPath} = "$CONFIG->{FilterConfigPath}/bin"        unless defined($CONFIG->{FilterBinConfigPath});
$CONFIG->{FilterDataConfigPath} = "$CONFIG->{FilterConfigPath}/data"      unless defined($CONFIG->{FilterDataConfigPath});
$CONFIG->{FilterDefConfigPath} = "$CONFIG->{FilterConfigPath}/def"        unless defined($CONFIG->{FilterDefConfigPath});
$CONFIG->{FilterLicConfigPath} = "$CONFIG->{FilterConfigPath}/lk-license" unless defined($CONFIG->{FilterLicConfigPath});
$CONFIG->{FilterSrcConfigPath} = "$CONFIG->{FilterConfigPath}/src"        unless defined($CONFIG->{FilterSrcConfigPath});

$CONFIG->{FilterCFDataMainsetPath}  = "$CONFIG->{FilterCFDataPath}/mainset" unless defined($CONFIG->{FilterCFDataMainsetPath});
$CONFIG->{FilterCFDataReceivedPath} = "$CONFIG->{FilterCFDataPath}/received_updates" unless defined($CONFIG->{FilterCFDataReceivedPath});
$CONFIG->{FilterCFDataBackupPath}   = "$CONFIG->{FilterCFDataPath}/backup"  unless defined($CONFIG->{FilterCFDataBackupPath});
$CONFIG->{FilterCFDataPatchPath}    = "$CONFIG->{FilterCFDataPath}/patches"  unless defined($CONFIG->{FilterCFDataPatchPath});

$CONFIG->{CCBinPath}     = "$CONFIG->{CCRootPath}/bin"     unless defined($CONFIG->{CCBinPath});
$CONFIG->{CCLibPath}     = "$CONFIG->{CCRootPath}/lib"     unless defined($CONFIG->{CCLibPath});
$CONFIG->{CCStatPath}    = "$CONFIG->{CCRootPath}/stat"    unless defined($CONFIG->{CCStatPath});
$CONFIG->{CCWwwRootPath} = "$CONFIG->{CCRootPath}/www"     unless defined($CONFIG->{CCWwwRootPath});
$CONFIG->{CCRCLibPath}   = "$CONFIG->{CCLibPath}/rc"       unless defined($CONFIG->{CCRCLibPath});
$CONFIG->{CCWwwStatPath} = "$CONFIG->{CCWwwRootPath}/stat" unless defined($CONFIG->{CCWwwStatPath});
$CONFIG->{FilterTmpConfigPath} = "$CONFIG->{CCRootPath}/tmp" unless defined($CONFIG->{FilterTmpConfigPath});

# Configurable files path
$CONFIG->{FilterPath}    = "$CONFIG->{BinRootPath}/ap-mailfilter" unless defined($CONFIG->{FilterPath});

# Conf-files
$CONFIG->{UpdaterIniPath} = "$CONFIG->{EtcRootPath}/keepup2date.conf";
$CONFIG->{StatConfPath}   = "$CONFIG->{CCStatPath}/stat.conf";

$CONFIG->{ProtectedDomainsPath} = "$CONFIG->{FilterConfigPath}/protected_domains";

$CONFIG->{FilterUserLogFile} = undef unless defined($CONFIG->{FilterUserLogFile});  # Normally output to syslog
$CONFIG->{FilterStatLogFile} = "$CONFIG->{LogsRootPath}/filter.log" unless defined($CONFIG->{FilterStatLogFile});  # Normally output to syslog
$CONFIG->{UpdaterLogPath}    = "$CONFIG->{LogsRootPath}/updater.log" unless defined($CONFIG->{UpdaterLogPath});
$CONFIG->{SFUpdatesLogPath}  = "$CONFIG->{LogsRootPath}/sfupdates.log";

# Executables
$CONFIG->{RestartScript}        = "$CONFIG->{BinRootPath}/kas-restart";
$CONFIG->{CompilerPath}         = "$CONFIG->{BinRootPath}/mkconf";
$CONFIG->{PrecompilerPath}      = "$CONFIG->{CCBinPath}/ap-mflt-precompiler";
$CONFIG->{MkProfilesPath}       = "$CONFIG->{BinRootPath}/mkprofiles";
$CONFIG->{LicInfoPath}          = "$CONFIG->{BinRootPath}/kas-show-license -x";
$CONFIG->{InstallKey}           = "$CONFIG->{BinRootPath}/install-key";
$CONFIG->{RemoveKey}            = "$CONFIG->{BinRootPath}/remove-key";

$CONFIG->{UpdaterScriptPath}    = "$CONFIG->{BinRootPath}/sfupdates";
$CONFIG->{MonitoringScriptPath} = "$CONFIG->{CCBinPath}/sfmonitoring";
$CONFIG->{DoLogsPath}           = "$CONFIG->{CCBinPath}/dologs.sh";
$CONFIG->{DoGraphPath}          = "$CONFIG->{CCBinPath}/dograph.sh";
$CONFIG->{LogRotatePath}        = "$CONFIG->{CCBinPath}/logrotate.sh";
$CONFIG->{UdsRttsPath}          = "$CONFIG->{BinRootPath}/uds-rtts.sh";

# Application Logs
$CONFIG->{FilterBaseIDPath} = "$CONFIG->{FilterDataConfigPath}/data.set.info";

# Control center data files
$CONFIG->{CCSyslogLogPath}    = "$CONFIG->{CCStatPath}/syslog.log";
$CONFIG->{CCUpdaterLogPath}   = "$CONFIG->{CCStatPath}/updater.log";
$CONFIG->{CCKeepup2dateLogPath} = "$CONFIG->{CCStatPath}/keepup2date.log";
$CONFIG->{CCSLicenseLogPath}  = "$CONFIG->{CCStatPath}/license.log";
$CONFIG->{CCStatBasePath}     = "$CONFIG->{CCStatPath}/__all";
$CONFIG->{CCEventsPath}       = "$CONFIG->{CCStatPath}/events.dat";

# PID-files
$CONFIG->{ServerPIDFile}      = "$CONFIG->{RunRootPath}/ap-process-server.pid";
$CONFIG->{SpfDaemonPIDFile}   = "$CONFIG->{RunRootPath}/ap-spfd.pid";
$CONFIG->{KasLicensePIDFile}   = "$CONFIG->{RunRootPath}/kas-license.pid";
$CONFIG->{HttpDaemonPIDFile}  = "$CONFIG->{RunRootPath}/kas-thttpd.pid";
$CONFIG->{KasMilterPIDFile}   = "$CONFIG->{RunRootPath}/kas-milter.pid";

$CONFIG->{MkProfilesPIDFile}  = "$CONFIG->{RunRootPath}/ap-mflt-mkprofiles.pid";

# Updatable dnsbl list
$CONFIG->{KasDNSBlackList}    = "$CONFIG->{FilterSrcConfigPath}/dnsblacklists/kas_dnsbl.xml";

#############################
# Get User and Group

$CONFIG->{User} = $CONST->{user}{mailflt_name} unless $CONFIG->{User};
$CONFIG->{UserID} = scalar getpwnam($CONFIG->{User});
unless( defined($CONFIG->{UserID}) )
{
  $CONFIG->{User} = $CONST->{user}{mailflt_name};
  $CONFIG->{UserID} = scalar getpwnam($CONFIG->{User});
}

if( defined($CONFIG->{UserID}) )
{
  # Patch $CONST entries
  $CONST->{mailflt_name} = $CONFIG->{User};
  $CONST->{mailflt_uid}  = $CONFIG->{UserID};
  $CONST->{group_name}   = $CONFIG->{Group};

  if( $< == $CONFIG->{UserID} )
  {
    $CONST->{bin}{crontab_get} = 'crontab -l 2>/dev/null';
    $CONST->{bin}{crontab_set} = 'crontab - >/dev/null 2>/dev/null';
  }
  else
  {
    $CONST->{bin}{crontab_get} = "crontab -u $CONFIG->{User} -l 2>/dev/null";
    $CONST->{bin}{crontab_set} = "crontab -u $CONFIG->{User} - >/dev/null 2>/dev/null";
  }
}


#############################
# Get language resources info

my @RCFILES = qw(info.pl nls.pl);

opendir( RCDIR, $CONFIG->{CCRCLibPath} ) || die "Can't open directory $CONFIG->{CCRCLibPath}: $!\n";
my @rcdirs = grep { -d "$CONFIG->{CCRCLibPath}/$_" && $_ !~ /^\./ } readdir RCDIR;
closedir RCDIR;

foreach my $dir (@rcdirs)
{
  my $ok = 1;
  foreach my $file (@RCFILES)
  {
    $ok = undef unless( -r "$CONFIG->{CCRCLibPath}/$dir/$file" );
  }

  if( $ok )
  {
    my $res = do "$CONFIG->{CCRCLibPath}/$dir/info.pl";
    if( $@ || !$res )
    {
      $ok = undef;
      delete $LANGS->{$dir} if( exists $LANGS->{$dir} );
    }
  }
}

die "Language 'en' not found in resources direcrory $CONFIG->{CCRCLibPath}\n" unless( defined $LANGS->{en} );


########################################
# Verify language value & resources path

$CONFIG->{Lang} = lc( $LANG_FORCE? $LANG_FORCE: $CONFIG->{Lang} );
$CONFIG->{Lang} = 'en' unless( defined $LANGS->{$CONFIG->{Lang}} );
$CONFIG->{rc_path} = $CONFIG->{CCRCLibPath}.'/'.$CONFIG->{Lang};
$CONFIG->{charset} = $LANGS->{$CONFIG->{Lang}}{charset};
$CONFIG->{charset} = 'utf-8' unless( defined $CONFIG->{charset} );

require $CONFIG->{rc_path} . '/nls.pl';

$LANG->{locale} = "C" unless $LANG->{locale};
$ENV{LANG} = $ENV{LC_ALL} = $LANG->{locale};
POSIX::setlocale(&POSIX::LC_ALL, $LANG->{locale} );

########################################
# Build other configuration data

$CONFIG->{CompilerLog} = $CONFIG->{FilterTmpConfigPath} . "/.compiler.$$.log";
$CONFIG->{AlertsTo} = $CONST->{defaults}{AlertsTo} unless( defined($CONFIG->{AlertsTo}) && $CONFIG->{AlertsTo} ne '' );

1;
__END__

################################
# DEBUG

#print Dumper($CONFIG), "\n";

#use Data::Dumper;
#print Data::Dumper->Dump( [$LANGS, $CONFIG, $LANG], [qw(LANGS CONFIG LANG)] ), "\n\n";

#$CONFIG->{Lang} = 'ru';
#WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $CONST->{config_map} );
#WriteConfFile( $CONFIG, $CONFIG->{ConfigPath} );
