##############################################################################
use strict;
use Sys::Hostname;

use vars qw($CONST $ROOT);

BEGIN {
  require 'root.pl';
};


sub oneof(@)
{
  my $file;

  foreach (@_)
  {
    if ( -f $_ or -l $_ )
    {
      $file = $_;
      last;
    }
  }

  return $file;
}


$CONST =  {
            path => {
                      root    => $ROOT,
                      'umask' => 0007,
                    },

            cgi  => {
                      expires           => '-300d',
                      cache_control     => 'no-cache',
                      log_expires       => '+20s',
                      log_cache_control => undef, #  'no-cache'
                      refresh_time      => 600,
                    },

            config_map => {  # Map of parameter names in ap-mailfilter.conf
                      RootPath                => 'RootPath',
                      BinRootPath             => 'BinRootPath',
                      EtcRootPath             => 'EtcRootPath',
                      FilterConfigPath        => 'FilterConfigPath',
                      FilterCFDataPath        => 'FilterCFDataPath',
                      CCRootPath              => 'CCRootPath',
                      RunRootPath             => 'RunRootPath',
                      LogsRootPath            => 'LogsRootPath',
                      FilterBinConfigPath     => 'FilterBinConfigPath',
                      FilterDataConfigPath    => 'FilterDataConfigPath',
                      FilterDefConfigPath     => 'FilterDefConfigPath',
                      FilterLicConfigPath     => 'FilterLicConfigPath',
                      FilterSrcConfigPath     => 'FilterSrcConfigPath',
                      FilterTmpConfigPath     => 'FilterTmpConfigPath',
                      FilterCFDataMainsetPath => 'FilterCFDataMainsetPath',
                      FilterPath              => 'FilterPath',
                      FilterStatLogFile       => 'FilterStatLogFile',
                      FilterUserLogFile       => 'FilterUserLogFile',

                      User                    => 'User',
                      Group                   => 'Group',
                      ##########

                      LogFacility             => 'LogFacility',
                      LogLevel                => 'LogLevel',

                      ServerMaxFilters        => 'ServerMaxFilters',
                      ServerStartFilters      => 'ServerStartFilters',
                      ServerSpareFilters      => 'ServerSpareFilters',

                      FilterMaxMessages       => 'FilterMaxMessages',
                      FilterRandMessages      => 'FilterRandMessages',
                      FilterMaxIdle           => 'FilterMaxIdle',
                      FilterDelayedExit       => 'FilterDelayedExit',

                      FilterDNSTimeout        => 'FilterDNSTimeout',
                      FilterReceivedHeadersLimit=> 'FilterReceivedHeadersLimit',
                      FilterParseMSOffice     => 'FilterParseMSOffice',
                      FilterParseRTF          => 'FilterParseRTF',
                      FilterUDSEnabled        => 'FilterUDSEnabled',
                      FilterUDSTimeout        => 'FilterUDSTimeout',

                      ClientFilteringSizeLimit=> 'ClientFilteringSizeLimit',
                      ClientOnError           => 'ClientOnError',
                      ClientDefaultDomain     => 'ClientDefaultDomain',
                      ClientConnectTimeout    => 'ClientConnectTimeout',
                      ClientDataTimeout       => 'ClientDataTimeout',

                      AlertsTo                => 'ControlCenterSendAlertsTo',
                      Lang                    => 'ControlCenterLang',

                      MonitoringHttpd         => 'MonitoringHttpd',
                      MonitoringKasMilter     => 'MonitoringKasMilter',
                      ##########

                      CCDontModifyCrontab     => 'CCDontModifyCrontab',
                      UseKasDNSBL             => 'UseDefaultKasDNSBL',
                    },
            defaults => {
                      # defaults and limits
                      LogFacility             => 'mail',
                      LogLevel                => 2,

                      ServerMaxFilters        => 10,
                      ServerMaxFilters_min    => 1,
                      ServerMaxFilters_max    => 200,

                      ServerStartFilters      => 0,
                      ServerSpareFilters      => 0,

                      FilterMaxMessages       => 300,
                      FilterMaxMessages_min   => 10,
                      FilterMaxMessages_max   => 1000,

                      FilterRandMessages      => 30,
                      FilterRandMessages_min  => 0,
                      FilterRandMessages_max  => 50,

                      FilterMaxIdle           => 300,
                      FilterMaxIdle_min       => 30,
                      FilterMaxIdle_max       => 3600,

                      FilterDelayedExit       => 0,
                      FilterDelayedExit_min   => 0,
                      FilterDelayedExit_max   => 30,

                      FilterDNSTimeout        => 10,
                      FilterDNSTimeout_min    => 1,
                      FilterDNSTimeout_max    => 60,

                      FilterReceivedHeadersLimit     => 2,
                      FilterReceivedHeadersLimit_min => 0,
                      FilterReceivedHeadersLimit_max => 100,

                      FilterParseMSOffice     => 'no',
                      FilterParseRTF          => 'no',
                      FilterUDSEnabled        => 'yes',
                      FilterUDSTimeout        => 10,
                      FilterUDSTimeout_min    => 1,
                      FilterUDSTimeout_max    => 60,

                      ClientFilteringSizeLimit      => 500,
                      ClientFilteringSizeLimit_min  => 0,
                      ClientFilteringSizeLimit_max  => 10000,

                      ClientOnError           => 'accept',
                      ClientDefaultDomain     => 'localhost',

                      ClientConnectTimeout      => 40,
                      ClientConnectTimeout_min  => 10,
                      ClientConnectTimeout_max  => 100,

                      ClientDataTimeout         => 30,
                      ClientDataTimeout_min     => 10,
                      ClientDataTimeout_max     => 100,

                      Keepup2DateConnectTimeout         => 20,

                      Keepup2DatePassiveFtp             => 0,
                      Keepup2DateUseProxy               => 0,
                      Keepup2DateUseUpdateServerUrl     => 0,
                      Keepup2DateUseUpdateServerUrlOnly => 0,
                      Keepup2DateProxyAddress           => '',
                      Keepup2DateRegion                 => '',
                      Keepup2UpdateServerUrl            => '',

                      Keepup2DateLogLevel               => 4,

                      UpdateTime              => 3,

                      AlertsTo                => 'postmaster',

                      MonitoringHttpd         => 'yes',
                      MonitoringKasMilter     => 'no',

                      ##########

                      CCDontModifyCrontab     => 0,
                      UseKasDNSBL             => 'yes',
                    },
            kasupdaterini_map => {
                      Keepup2DateDateFormat              => 'locale/DateFormat',
                      Keepup2DateTimeFormat              => 'locale/TimeFormat',

                      Keepup2DateBasesPath               => 'path/BasesPath',
                      Keepup2DateLicensePath             => 'path/LicensePath',

                      Keepup2DateConnectTimeout          => 'updater.options/ConnectTimeout',
                      Keepup2DatePassiveFtp              => 'updater.options/PassiveFtp',
                      Keepup2DateUseProxy                => 'updater.options/UseProxy',
                      Keepup2DateUseUpdateServerUrl      => 'updater.options/UseUpdateServerUrl',
                      Keepup2DateUseUpdateServerUrlOnly  => 'updater.options/UseUpdateServerUrlOnly',
                      Keepup2DateProxyAddress            => 'updater.options/ProxyAddress',
                      Keepup2DateRegion                  => 'updater.options/RegionSettings',
                      Keepup2UpdateServerUrl             => 'updater.options/UpdateServerUrl',

                      Keepup2UpdateBackUpPath            => 'updater.path/BackUpPath',
                      Keepup2UpdateUploadPatchPath       => 'updater.path/UploadPatchPath',

                      Keepup2DateAppendLog               => 'updater.report/Append',
                      Keepup2DateLogPath                 => 'updater.report/ReportFileName',
                      Keepup2DateLogLevel                => 'updater.report/ReportLevel',
                    },
            detected_classes => {
                      normal    => 'Not Detected',
                      spam      => 'Spam',
                      probable  => 'Probable Spam',
                      formal    => 'Formal',
                      trust     => 'Trusted',
                      black     => 'Blacklisted',
                      other     => 'Other'
                    },
            actions_defaults => {
                      ACTION_SPAM_MODE                  => 0,  # accept
                      ACTION_SPAM_EMAIL                 => '',
                      ACTION_SPAM_SUBJECT_PREFIX        => '[!! SPAM] ',
                      ACTION_SPAM_SUBJECT_SUFFIX        => '',
                      ACTION_SPAM_KEYWORDS              => 'SPAM',
                      ACTION_SPAM_USERINFO              => 'SPAM',

                      ACTION_PROBABLE_MODE              => 0,  # accept
                      ACTION_PROBABLE_EMAIL             => '',
                      ACTION_PROBABLE_SUBJECT_PREFIX    => '[?? Probable Spam] ',
                      ACTION_PROBABLE_SUBJECT_SUFFIX    => '',
                      ACTION_PROBABLE_KEYWORDS          => 'PROBABLE',
                      ACTION_PROBABLE_USERINFO          => 'Probable Spam',

                      ACTION_FORMAL_MODE                => 0,  # accept
                      ACTION_FORMAL_EMAIL               => '',
                      ACTION_FORMAL_SUBJECT_PREFIX      => '[--Formal Message--] ',
                      ACTION_FORMAL_SUBJECT_SUFFIX      => '',
                      ACTION_FORMAL_KEYWORDS            => 'FORMAL',
                      ACTION_FORMAL_USERINFO            => 'Formal',

                      ACTION_BLACKLISTED_MODE           => 0,  # accept
                      ACTION_BLACKLISTED_EMAIL          => '',
                      ACTION_BLACKLISTED_SUBJECT_PREFIX => '[!! BLACKLISTED] ',
                      ACTION_BLACKLISTED_SUBJECT_SUFFIX => '',
                      ACTION_BLACKLISTED_KEYWORDS       => 'BLACKLISTED',
                      ACTION_BLACKLISTED_USERINFO       => 'Blacklisted',

                      ACTION_TRUSTED_MODE               => 0,  # accept
                      ACTION_TRUSTED_EMAIL              => '',
                      ACTION_TRUSTED_SUBJECT_PREFIX     => '',
                      ACTION_TRUSTED_SUBJECT_SUFFIX     => '',
                      ACTION_TRUSTED_KEYWORDS           => 'TRUSTED',
                      ACTION_TRUSTED_USERINFO           => 'Trusted',

                      ACTION_NORMAL_MODE                => 0,  # accept
                      ACTION_NORMAL_EMAIL               => '',
                      ACTION_NORMAL_SUBJECT_PREFIX      => '',
                      ACTION_NORMAL_SUBJECT_SUFFIX      => '',
                      ACTION_NORMAL_KEYWORDS            => '',
                      ACTION_NORMAL_USERINFO            => 'Not detected',
                    },
            bin  => {
                      ps => 'ps ax',
                      crontab_get => 'crontab -l 2>/dev/null',
                      crontab_set => 'crontab - >/dev/null 2>/dev/null',
                      uname       => 'uname -smr',
                      uptime      => 'uptime',
                      sendmail    => 'sendmail',
#                      sendmail    => oneof( '/usr/lib/sendmail', '/usr/local/bin/sendmail', '/usr/sbin/sendmail', '/usr/local/sbin/sendmail', '/usr/local/lib/sendmail', '/var/qmail/bin/sendmail' ),
                    },
            user => {
                      root_uid => 0,
                      mailflt_name => 'mailflt3',
                      mailflt_uid  => scalar(getpwnam('mailflt3')),
                      group_name   => 'mailflt3',
                    },
            crontab => {
                      monitoring  => "*/5 * * * *",
                      dologs      => "* * * * *",
                      dograph     => "*/5 * * * *",
                      logrotate   => "7 */12 * * *",
                    },
            etc  => {
                      info_label   => 'info',
                      load_average => { warning=>6, error=>12 },
                      update_time  => { warning=>24, error=>24 }, # in hours
                      license_time => { warning=>15, error=>0 },  # in days - warn if value is less
                      logtime_limit=> 24*3600,
                      logcount_limit=> 4096,
                      logsize_limit=> 1*1024*1024,
                      timespan_long_limits => 4,
                      groupname_limit => 50,
                      subject_prefix_limit => 32,
                      x_header_limit => 72,
                      max_group_desc_length => 500,
                    },
            cachetime => {
                      system_info   => 5*60,
                      apfilters     => 60,
                      spfd          => 60,
                      approc_server => 60,
                      licdaemon     => 60,
                      crontab       => 5*60,
                      update        => 2*60,
                      license       => 5*60,
                      httpdaemon    => 60,
                      kasmilter     => 60,
                    },
            messages  => {
                      bounce => 'This message violates our content filtration policy.',
                      reject => 'The message is rejected by spam filtering engine.',
                    },
          };
