#!/usr/local/ap-mailfilter3/bin/perl -w
#######################################################################
# Default paths and constants

use strict;
use POSIX qw(strftime mktime);
use Data::Dumper;
use File::Copy;
use Time::Local;

use vars qw($CONST $CONFIG @ERR);

BEGIN {
  require 'const.pl';
  require 'config.pl';
  require 'conffiles.pl';
  require 'utils.pl';
  require 'events.pl';
};


sub LoadCritEvents()
{
  my $path = $CONFIG->{CCEventsPath};
  my %events;

  if($path =~ s/events\.dat\s*$/events_crit.dat/){
   
    if( open( FILE, "<$path" ) ) {
      # event_name:check_time:count:start_time:mail_time:value

      while(<FILE>) {
        chomp;
        if( /^([^:]+):(\d+):(\d+):(\d+):(\d+):(.*)$/ ) {
          my ($event,$checktime,$count,$starttime,$mailtime,$value) = ($1,int($2),int($3),int($4),int($5),$6);
          $events{$event} = { check_time=>$checktime, mail_time=>$mailtime, start_time=>$starttime, count=>$count, value=>$value };
          $events{$event}{value} = undef if $events{$event}{value} eq '';
        }
      }
      close(FILE);
    }
    else {
#      warn "Can't open event file $path: $!\n";
    }
  }
  else {
#    warn "Can't convert event file path\n";
  }

  return \%events;
}


sub SaveCritEvents($)
{
  my ($events) = @_;
  return undef unless defined $events;

  my $path = $CONFIG->{CCEventsPath};

  if($path =~ s/events\.dat\s*$/events_crit.dat/){


    my $tmppath = $path.".$$.tmp";

    if( open( FILE, ">$tmppath" ) ) {
    # event_name:check_time:count:start_time:mail_time:value

    foreach my $event (sort keys(%{$events}))
    {
      next unless $event;
      $events->{$event}{count} = 0 unless( defined($events->{$event}{count}) );
      $events->{$event}{check_time} = 0 unless $events->{$event}{check_time};
      $events->{$event}{start_time} = 0 unless $events->{$event}{start_time};
      $events->{$event}{mail_time} = 0 unless $events->{$event}{mail_time};

      if( $events->{$event}{count} ) {
        $events->{$event}{start_time} = time() unless $events->{$event}{start_time};
      } else {
	  if($event ne "db_updated") {
             $events->{$event}{start_time} = $events->{$event}{mail_time} = 0;
	  }
      }

      $events->{$event}{value} = '' unless defined $events->{$event}{value};
      $events->{$event}{value} =~ s/^\s+//;
      $events->{$event}{value} =~ s/\s+$//;
      $events->{$event}{value} =~ s/[\n\r\t]+/ /g;
      $events->{$event}{value} =~ s/\s+/ /g;

      print FILE "$event:",int($events->{$event}{check_time}),":",int($events->{$event}{count}),":",int($events->{$event}{start_time}),":",int($events->{$event}{mail_time}),":$events->{$event}{value}\n";
    }
    close(FILE);

    if( move $tmppath, $path ){
      return 1;
    }
    else {
#      warn "Can't move $tmppath to $path: $!\n";
      push @ERR, "Can't move $tmppath to $path: $!";
    }
  }
  else
  {
#    warn "Can't open file $tmppath: $!\n";
    push @ERR, "Can't open file $tmppath: $!";
  }

  }

  return undef;
}


sub GetCritEvents(%)
{
  my (%param) = @_;
  my %crit_events;
  my $modified = 0;

  open(OLDERR,'>&STDERR');
  open(STDERR,'>/dev/null');

  {
    my $ref_data = LoadCritEvents();
    %crit_events = %{$ref_data} if $ref_data;
  }


  my $events=GetEvents(all=>1, dont_save=>1);
  my ($item,$warn,$err);
  my ($st_time_min,$count);
  my @crstr=();
  $st_time_min=$count=0;
  

# critical error

  my $time = time();

  $crit_events{critical} = {} unless defined $crit_events{critical};

  foreach $item (qw(load_average_level approc_server spfd_exist licdaemon apfilters
                    crontab_status crontab_updater httpdaemon kasmilter)) {   
    if(defined($events->{$item}) && defined($events->{$item}{message})){
      if( $events->{$item}{message}{alert} eq 'error' ){
        my $err_msg = $events->{$item}{message}{text};
        push(@crstr, $item." (".$err_msg.")");
        if($st_time_min==0) {
          $st_time_min=$events->{$item}{start_time};
          $count=$events->{$item}{count};
        }
        elsif($events->{$item}{start_time}>0 && $events->{$item}{start_time}<$st_time_min) { 
           $st_time_min=$events->{$item}{start_time}; 
           $count=$events->{$item}{count};
        }
      }
    }
   }

#TODO  crontab_status <-> count, start_time

  if($#crstr >=0 ) { 
       my ($i, $str1);
       $str1="";
       for( $i=0; $i<=$#crstr; $i++ ){
         $str1=$str1.$crstr[$i];
         if($i<$#crstr) { $str1=$str1.","; }
       }
       $crit_events{critical}{value}="event(s) signalled - ".$str1;
       $crit_events{critical}{start_time}=$st_time_min;
       $crit_events{critical}{count}=$count;

#       print "critical, events signalled: $str1\n"; 
   }
   else {
       $crit_events{critical}{value}=undef;
       $crit_events{critical}{start_time}=0;
       $crit_events{critical}{count}=0;
   }

   $crit_events{critical}{mail_time}=0;
   $crit_events{critical}{check_time}=$time;


# license_expired

  $crit_events{license_expired} = {} unless defined $crit_events{license_expired};

  if(defined($events->{lic_valid}) && defined($events->{lic_valid}{message})){
      if( $events->{lic_valid}{message}{alert} eq 'error' ){
	  $crit_events{license_expired}{value}=1;
          if(defined($events->{lic_valid}{start_time}) && $events->{lic_valid}{start_time}>0){
             $crit_events{license_expired}{start_time}=$events->{lic_valid}{start_time};	     
	  }
	  else{
	      $crit_events{license_expired}{start_time}=0;
	  }
      
          if(defined($events->{lic_valid}{count}) && $events->{lic_valid}{count}>0){
              $crit_events{license_expired}{count}=$events->{lic_valid}{count};
	  }
      }
      else {
        $crit_events{license_expired}{value}=0;
        $crit_events{license_expired}{start_time}=0;
        $crit_events{license_expired}{count}=0;
      }
  }
  else{
        $crit_events{license_expired}{value}=0;
        $crit_events{license_expired}{start_time}=0;
        $crit_events{license_expired}{count}=0;
  }

   $crit_events{license_expired}{check_time}=$time;
   $crit_events{license_expired}{mail_time}=0;


# db_expired

  $crit_events{db_expired} = {} unless defined $crit_events{db_expired};  

  if(defined($events->{update_time}) && defined($events->{update_time}{message})){
      if( $events->{update_time}{message}{alert} eq 'error' ){
	  $crit_events{db_expired}{value}=1;
          $crit_events{db_expired}{start_time}=$events->{update_time}{start_time};
          $crit_events{db_expired}{count}=$events->{update_time}{count};
      }
      else{
	  $crit_events{db_expired}{value}=0;
          $crit_events{db_expired}{start_time}=0;
          $crit_events{db_expired}{count}=0;
      }  
  }
  else{
      $crit_events{db_expired}{value}=0;
      $crit_events{db_expired}{start_time}=0;
      $crit_events{db_expired}{count}=0;
  }

  $crit_events{db_expired}{check_time}=$time;
  $crit_events{db_expired}{mail_time}=0;


#  db_updated

  $crit_events{db_updated} = {} unless defined $crit_events{db_updated};

  $crit_events{db_updated}{start_time}=0 unless defined $crit_events{db_updated}{start_time};

  if($crit_events{db_updated}{start_time} < $events->{update_time}{value}){
      $crit_events{db_updated}{value}=1;
  }
  else {
      $crit_events{db_updated}{value}=0;
  }
  $crit_events{db_updated}{start_time}=$events->{update_time}{value};
  $crit_events{db_updated}{mail_time}=0;
  $crit_events{db_updated}{check_time}=$time;


  close(STDERR);
  open(STDERR,'>&OLDERR');
  close(OLDERR);

  if( $param{dont_save} )
  {
     
      ;
#    $events{modified_flag} = $modified;
  }
  else
  {
     SaveCritEvents(\%crit_events);
  }

  return \%crit_events;
}


1;
__END__
