#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

#use FindBin qw($Bin);
use CGI qw(:standard);
use CGI::Carp;
use POSIX qw(strftime);
use Data::Dumper;

use vars qw($CONST $CONFIG $LANG $MENU $PARAM @ERR);

#######################################################################
# Load menu description

BEGIN {
  require 'cgiutil.pl';
  require 'rc/menu.pl';
}

#######################################################################
# Common data
my $PAGEID = undef;

my %MODES_ICO = (
              error=>   { small=>'icon_err.gif',   med=>'icon_err_24.gif',   large=>'icon_err_32.gif' },
              warning=> { small=>'icon_atten.gif', med=>'icon_atten_24.gif', large=>'icon_atten_32.gif' },
              notice => { small=>'icon_ok.gif',    med=>'icon_ok_24.gif',    large=>'icon_ok_32.gif' },
              info   => { small=>'icon_info.gif',  med=>'icon_info_24.gif',  large=>'icon_info_32.gif' }
            );

my $NAME_FORM_WIDTH = '30%';
my $NAME_INFO_WIDTH = '25%';

my $ACTION_TABLE_WIDTH = 550;
my $ACTION_LEFT_MARGINE = 30;
my $ACTION_NAME_WIDTH   = 190;
my $ACTION_CELLPADDING = 3;
my $ACTION_NAME_CTRL = $ACTION_TABLE_WIDTH - $ACTION_NAME_WIDTH - $ACTION_LEFT_MARGINE - 4*$ACTION_CELLPADDING;
my $ACTION_SELECT_WIDTH = $ACTION_TABLE_WIDTH - $ACTION_LEFT_MARGINE - 3*$ACTION_CELLPADDING;

my $TABLE_BGCOLOR_DEFAULT = '#f2f2f2';
my $TABLE_BGCOLOR_ALERT   = '#f9d5b6';

my @SUBSTITUTES = qw(group_name rules_group_name);
my $SUBSTITUTES_RX = '(' . join(')|(', @SUBSTITUTES) . ')';


#######################################################################

sub HTMLString
{
  my @res;

  foreach (@_)
  {
    next unless defined;
    s/^\s+//;
    s/\s+$//;
    push @res, SafeEscapeHTML($_);
  }

  return @res;
}

#######################################################################

sub GetPageInfo($)
{
  my $page = shift;
  my $res = {
              title => $LANG->{menu}{title}{default},
              help  => undef,
              path  => [],
            };

  my ($lvl1, $lvl2, $lvl3, $lvl4);

LOOP:
  foreach $lvl1 (@$MENU)
  {
    if( $lvl1->{page} eq $page )
    {
      $lvl1->{active} = 1;

      $res->{title} = $lvl1->{title};
      $res->{help} = $lvl1->{help};
      push @{$res->{path}}, $lvl1;

      last LOOP;
    }

    if( defined $lvl1->{menu} && scalar(@{$lvl1->{menu}}) )
    {
      foreach $lvl2 (@{$lvl1->{menu}})
      {
        if( $lvl2->{page} eq $page )
        {
          $lvl1->{active} = $lvl2->{active} = 1;

          $res->{title} = $lvl2->{title};
          $res->{help} = $lvl2->{help};
          push @{$res->{path}}, $lvl1, $lvl2;

          last LOOP;
        }

        if( defined $lvl2->{menu} && scalar(@{$lvl2->{menu}}) )
        {
          foreach $lvl3 (@{$lvl2->{menu}})
          {
            if( $lvl3->{page} eq $page )
            {
              $lvl1->{active} = $lvl2->{active} = $lvl3->{active} = 1;

              $res->{title} = $lvl3->{title};
              $res->{help} = $lvl3->{help};
              push @{$res->{path}}, $lvl1, $lvl2, $lvl3;

              last LOOP;
            }

            if( defined $lvl3->{menu} && scalar(@{$lvl3->{menu}}) )
            {
              foreach $lvl4 (@{$lvl3->{menu}})
              {
                if( $lvl4->{page} eq $page )
                {
                  $lvl1->{active} = $lvl2->{active} = $lvl3->{active} = $lvl4->{active} = 1;

                  $res->{title} = $lvl4->{title};
                  $res->{help} = $lvl4->{help};
                  push @{$res->{path}}, $lvl1, $lvl2, $lvl3, $lvl4;

                  last LOOP;
                }
              }
            }
          }
        }
      }
    }
  }

  return %$res;
}

sub MakePath(@)
{
  my @path = @_;
  my $res  = '';
  my $pos = 0;
  my $last = scalar(@path);

  if( $last )
  {
    $last--;
    foreach my $item (@path)
    {
      if(exists($item->{skip_path}) && $item->{skip_path} == 1)
      {
        $pos++;
        next;
      }

#      $res .= ' <span class=cookie>&#8594;</span> ' if $pos;
      $res .= ' <span class=cookie>&rarr;</span> ' if $pos;

      if( $pos != $last )
      {
        if(! $item->{page} || $item->{disabled_path})
        {
          $res .= qq|<span class=cookie>$item->{name}</span>|;
        }
        else
        {
          my $url = $item->{page};
          $url =~ s/^ref\://i;
          $res .= qq|<a class=cookie href="$url">$item->{name}</a>|;
        }
      }
      else
      {
        $res .= qq|<b class=cookie>$item->{name}</b>|;
      }

      $pos++;
    }
  }
  else
  {
    my $item = ${@{$MENU}}[0];
    my $url = $item->{page};
    $url =~ s/^ref\://i;
    $res .= qq|<a class=cookie href="$url">$item->{name}</a>|;
  }

  return $res;
}

sub MakeTabbing
{
  my $res = '';
  my $count = scalar( @{$MENU} );

  return $res unless $count;

  my $width = int(100/$count);
  $width .= '%';

  my $pos = 1;
  foreach my $item ( @{$MENU} )
  {
    if( $item->{active} )
    {
      my $sty = ($pos != 1)? '': '_2';
      $res .= qq|<td width=$width class=tab_sel$sty align=center>$item->{name}</td>|;
    }
    else
    {
      my $url = $item->{page};
      $url =~ s/^ref\://i;
      $res .= qq|<td width=$width class=tab_pas align=center><a class=tab_pas_a href=$url>$item->{name}</a></td>|;
    }

    if( $item->{active} || $pos==$count )
    {
      $res .= qq|<td width=1><img src=image/tab_pas_3.gif border=0 width=1 height=25><br></td>|;
    }
    else
    {
      $res .= qq|<td width=1><img src=image/tab_pas_2.gif border=0 width=1 height=25><br></td>|;
    }

    $pos++;
  }

  return $res;
}

sub MakeRubricator(@)
{
    my @path = @_;
    return undef if( scalar(@path) < 2 );
    my $res = '';
    my $hidden_level = ( scalar(@path) > 3 );
    my $menu = shift @path;
    $menu = $menu->{menu};

    $res .= qq|<table cellpadding=0 cellspacing=0 width=75% border=0>|;

    foreach my $lvl1 (@{$menu})
    {
      next if($lvl1->{hidden});
      if( $lvl1->{active} )
      {
        $res .= qq|
    <tr bgcolor=#e96e01><td class=nav_sel>$lvl1->{name}</td></tr>
    <tr height=1><td><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>
    <tr height=1 bgcolor=#e96e01><td><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>|;
      }
      else
      {
        my $url = $lvl1->{page};
        $url =~ s/^ref\://i;
        $res .= qq|
    <tr bgcolor=#177264><td class=nav_pas><a class=pas href=$url>$lvl1->{name}</a></td></tr>
    <tr height=1><td><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>
    <tr height=1 bgcolor=#177264><td><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>|;
      }

      if( defined($lvl1->{menu}) && scalar(@{$lvl1->{menu}}) )
      {
        $res .= qq|
    <tr><td class=nav_box>|;

        foreach my $lvl2 (@{$lvl1->{menu}})
        {
          next if($lvl2->{hidden});

          my $url = $lvl2->{page};
          $url =~ s/^ref\://i;

          if( $lvl2->{active} )
          {
            if( $hidden_level )
            {
              $res .= qq|
      <table cellpadding=0 cellspacing=0 width=100% border=0>
      <tr bgcolor=#ffffff valign=top>
      <td><img src=image/e.gif border=0 alt="" width=11 height=1><br></td>
      <td><img src=image/bul_sel.gif border=0 alt="" width=4 height=4 hspace=4 vspace=6><br></td>
      <td width=80%><a href=$url class=sel>$lvl2->{name}</a></td>
      <td><img src=image/e.gif border=0 alt="" width=10 height=1><br></td>
      </tr>
      </table>
|;
            }
            else
            {
              $res .= qq|
      <table cellpadding=0 cellspacing=0 width=100% border=0>
      <tr bgcolor=#ffffff valign=top>
      <td><img src=image/e.gif border=0 alt="" width=11 height=1><br></td>
      <td><img src=image/bul_sel.gif border=0 alt="" width=4 height=4 hspace=4 vspace=6><br></td>
      <td width=80%><span class=sel>$lvl2->{name}</span></td>
      <td><img src=image/e.gif border=0 alt="" width=10 height=1><br></td>
      </tr>
      </table>
|;
            }
          }
          elsif( $lvl2->{disabled} )
          {
            $res .= qq|
      <table cellpadding=0 cellspacing=0 width=100% border=0>
      <tr valign=top>
      <td><img src=image/e.gif border=0 alt="" width=11 height=1><br></td>
      <td><img src=image/bul_gr.gif border=0 alt="" width=4 height=4 hspace=4 vspace=6><br></td>
      <td width=80%><span class=gray>$lvl2->{name}</span></td>
      <td><img src=image/e.gif border=0 alt="" width=10 height=1><br></td>
      </tr>
      </table>
|;
          }
          else
          {
            $res .= qq|
      <table cellpadding=0 cellspacing=0 width=100% border=0>
      <tr valign=top>
      <td><img src=image/e.gif border=0 alt="" width=11 height=1><br></td>
      <td><img src=image/bul_pas.gif border=0 alt="" width=4 height=4 hspace=4 vspace=6><br></td>
      <td width=80%><a href=$url>$lvl2->{name}</a></td>
      <td><img src=image/e.gif border=0 alt="" width=10 height=1><br></td>
      </tr>
      </table>
|;
          }
        }

        $res .= qq|
    </td></tr>|;
      }
    }

    $res .= qq|</table>|;

  return $res;
}


sub PageOpen($;%)
{
  my ($page,%param) = @_;
  $PAGEID = $page;

  my %info = GetPageInfo($page);

  if(scalar @{$info{path}} && !defined($param{redirect}))
  {
    my %cache_control;
    $cache_control{-CACHE_CONTROL} = $CONST->{cgi}{cache_control} if defined($CONST->{cgi}{cache_control});

    print header ( -TYPE => "text/html; charset=$CONFIG->{charset}",
                   -EXPIRES => $CONST->{cgi}{expires},
                   %cache_control );
  }
  else
  {
    print redirect( -uri => ( (defined($param{redirect}) && $param{redirect} ne '')? $param{redirect} : '/') );
    if( scalar @ERR )
    {
      warn "Error dump in redirected page $page\n";
      foreach (@ERR)
      {
        warn "$_\n";
      }
      warn "End dump\n";
    }
  }

  my $title = $info{title};
  my $path  = MakePath(@{$info{path}});

  my $do_substitutes;
  foreach (@SUBSTITUTES)
  {
    if( defined $param{$_} )
    {
      $do_substitutes = 1;
      ($param{$_}) = HTMLString $param{$_};
    }
    else
    {
      $param{$_} = '';
    }
  }

  if( $do_substitutes )
  {
    $title =~ s/\$($SUBSTITUTES_RX)/$param{$1}/go;
    $path  =~ s/\$($SUBSTITUTES_RX)/$param{$1}/go;
  }

  my $onload = (defined($param{onload}) && $param{onload} ne '')? qq| onload="$param{onload}"| : '';
  my $reload = (defined($param{reload}) && $param{reload} ne '')? qq|<meta http-equiv="refresh" content=$param{reload}>|: '';

  print qq|<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
 <meta http-equiv="Content-Type" content="text/html; charset=$CONFIG->{charset}">$reload
 <title>$title</title>
 <link rel="stylesheet" type="text/css" href="style.css">
 <link rel="Shortcut Icon" type="image/x-icon" href="image/favicon.ico">|;

  print qq|
 <script language=javascript src="$param{script}" type="text/javascript"></script>| if defined($param{script}) && $param{script} ne '';

  print qq|
</head>
<body bgcolor="#ffffff" link="#005447" alink="#FF0000" vlink="#FF0000" text="#000000" topmargin=0 leftmargin=0 rightmargin=0 marginwidth=0 marginheight=0$onload>|;

  print qq|<table cellpadding=0 cellspacing=0 width=760 border=0>
<!-- head -->
<tr valign=top bgcolor=#005447>
<td width=760 height=60>
  <table cellpadding=0 cellspacing=0 width=760 height=100% border=0>
  <tr>
  <td width=200 style="background-image:url(image/bg_1.gif); background-position:50% 0%;">
    <table cellpadding=0 cellspacing=0 width=100% height=100% border=0>
    <tr height=100%><td align=center><img src=image/top_logo.gif border=0 alt="$LANG->{static}{logo}" width=165 height=48><br></td></tr>
    <tr height=1 bgcolor=#002822><td><img src=image/e.gif border=0 width=1 height=1><br></td></tr>
    </table>
  </td>
  <td style="background-image:url(image/bg_2.gif);">
    <table cellpadding=0 cellspacing=0 width=100% height=100% border=0>
    <!-- path to section -->
    <tr height=20><td>
      <table cellpadding=0 cellspacing=0 width=100% height=20 border=0 bgcolor=#005447>
      <tr>
      <td width=5 bgcolor=#f7f7f7><img src=image/e.gif border=0 width=7 height=20><br></td>
      <td width=5 bgcolor=#002822><img src=image/e.gif border=0 width=5 height=20><br></td>
      <td width=1 bgcolor=#ffffff><img src=image/e.gif border=0 width=1 height=20><br></td>
      <td style="padding:0 5 3 5;">$path</td>
      </tr>
      </table>
    </td></tr>
    <tr><td>&nbsp;</td></tr>
    <!-- Tabbing -->
    <tr height=25><td>
      <table cellpadding=0 cellspacing=0 width=100% height=25 border=0>
      <tr>|;

  print MakeTabbing();

  print qq|
      </tr>
      </table>
    </td></tr>
    </table>
  </td>
  <td width=60 align=center>|;

  if( defined($info{help}) && $info{help} ne '' )
  {
    my $help_uri = $info{help};
    print qq|
    <table cellpadding=0 cellspacing=0 width=100% height=100% border=0>
    <tr valign=bottom><td align=center><a href="$help_uri"><img src=image/icon_big_help.gif border=0 alt="$LANG->{static}{help}" width=28 height=29></a><br></td></tr>
    <tr valign=top height=22><td align=center><a class=cookie href="$help_uri"><b>$LANG->{static}{help}</b></a></td></tr>
    <tr height=1 bgcolor=#002822><td><img src=image/e.gif border=0 width=1 height=1><br></td></tr>
    </table>|;
  }

  print qq|
  </td>
  </tr>
  </table>
</td>
<td width=100%>
  <table cellpadding=0 cellspacing=0 width=100% height=100% border=0>
  <tr height=100%><td>&nbsp;</td></tr>
  <tr height=1 bgcolor=#002822><td><img src=image/e.gif border=0 width=1 height=1><br></td></tr>
  </table>
</td>
</tr>
|;

  print qq|
<!-- Main area -->
<tr valign=top height=100%>
<td>
  <table cellpadding=0 cellspacing=0 width=100% height=100% border=0>
  <tr valign=top>|;

  my $rubr = MakeRubricator(@{$info{path}});

  if( defined $rubr )
  {
    print qq|
  <!-- Rubricator in left -->
  <td width=200 align=center>
    <table cellpadding=0 cellspacing=0 width=100% border=0><tr><td><img src=image/e.gif border=0 alt="" width=1 height=20><br></td></tr></table>
    $rubr
    <table cellpadding=0 cellspacing=0 width=100% border=0><tr><td><img src=image/e.gif border=0 alt="" width=1 height=20><br></td></tr></table>
  </td>|;
  }
  else
  {
    print qq|<td width=25></td>|;
  }

  print qq|
  <!-- Content area -->
  <td>|;
}


sub PageClose()
{
  print qq|
    <table cellpadding=0 cellspacing=0 width=100% border=0><tr><td><img src=image/e.gif border=0 alt="" width=1 height=20><br></td></tr></table>
  </td>
  </tr>
  </table>
</td>
<td>&nbsp;</td>
</tr>
<!-- footer -->
<tr><td colspan=2 bgcolor=#002822><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>
<tr><td colspan=2 bgcolor=#ffffff><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>
<tr valign=top height=48 bgcolor=#005447>
<td>
  <table cellpadding=0 cellspacing=0 width=760 height=48 border=0>
  <tr>
  <td width=200 align=center style="background-image:url(image/bg_1.gif); background-position:50% 0%;">&nbsp;</td>
  <td style="background-image:url(image/bg_2.gif);">
  <p class=copy>$LANG->{static}{copyright}
  </td>
  <td width=60>&nbsp;</td>
  </tr>
  </table>
</td>
<td>&nbsp;</td>
</tr>
<tr><td colspan=2 bgcolor=#ffffff><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>
<tr><td colspan=2 bgcolor=#002822><img src=image/e.gif border=0 alt="" width=1 height=1><br></td></tr>
</table>

</body>
</html>
|;
}


#######################################################################


sub DrawHeader(%)
{
  my (%param) = @_;
  my ($Title,$Memo,$time) = ($param{title},$param{memo},$param{'time'});
  $Title = '' unless defined $Title;

  my $do_substitutes;
  foreach (@SUBSTITUTES)
  {
    if( defined $param{$_} )
    {
      $do_substitutes = 1;
      ($param{$_}) = HTMLString $param{$_};
    }
    else
    {
      $param{$_} = '';
    }
  }

  if( $do_substitutes )
  {
    $Title =~ s/\$($SUBSTITUTES_RX)/$param{$1}/go;
    $Memo =~  s/\$($SUBSTITUTES_RX)/$param{$1}/go if defined($Memo) && $Memo ne '';
  }

  print qq|
    <table cellpadding=0 cellspacing=0 width=100% border=0><tr><td><img src=image/e.gif border=0 alt="" width=1 height=12><br></td></tr></table>|;

  if( $time )
  {
    $time = strftime( $LANG->{timefmt}{time_short}, localtime(time) );

    print qq|
    <table cellpadding=0 cellspacing=0 width=100% border=0>
    <tr>
    <td>
    <h1 class=table_title>$Title</h1>|;

    print qq|
    <span class=comment>$Memo</span>| if defined($Memo) && $Memo ne '';

    print qq|
    </td>
    <td width=60 align=right>
      <table cellpadding=0 cellspacing=0 border=0 align=center><tr><td nowrap><h1 class=table_title>$time</h1></td></tr></table>
    </td>
    </tr>
    </table>|;
  }
  else
  {
    print qq|
    <h1 class=table_title>$Title</h1>|;
  print qq|
    <span class=comment>$Memo</span>| if defined($Memo) && $Memo ne '';
  }

  print qq|
    <table cellpadding=0 cellspacing=1 width=100% border=0 bgcolor=#ffffff>
    <tr bgcolor=#f9d5b6>
    <td><img src=image/e.gif border=0 alt="" width=1 height=5><br></td>
    </tr>
    </table>|;
}


#######################################################################


sub InfoOpen(%)
{
  my (%attr) = @_;

  DrawHeader(%attr);
  print qq|
    <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>|;
}

sub InfoClose(%)
{
  print qq|
    </table>|;
}

sub InfoBlock(%)
{
  my (%param) = @_;

  my $txt = (HTMLString($param{text}))[0];
  my $out;
  if( defined($param{more_url}) && $param{alert} =~ /^(error|warning)$/i )
  {
    my $dddot = '';
    $dddot = '...' if( $txt =~ s/\.\.\.$// );
#    $out = qq|<table width=100% cellpadding=0 cellspacing=0 border=0><tr valign=top><td>$txt</td><td align=right><a href="$param{more_url}">$LANG->{static}{more_info}</a>$dddot&nbsp;</td></tr></table>|;
    $out = qq|<a href="$param{more_url}">$txt</a>$dddot|;
  }
  else
  {
    $out = $txt;
  }

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td width=5%>|, ((defined($param{alert}) && defined($MODES_ICO{$param{alert}}))? qq|<img src=image/$MODES_ICO{$param{alert}}{small} border=0 alt="" width=16 height=16 hspace=5 vspace=2>|: '<img src=image/e.gif border=0 alt="" width=16 height=16 hspace=5 vspace=2>'), qq|<br></td>
    <td width=$NAME_INFO_WIDTH nowrap><span class=gray_bold>$param{title}:</span></td>
    <td>$out</td>
    </tr>
  |;
}


#######################################################################


sub ListOpen(%)
{
  my (%param) = @_;
  DrawHeader(%param);
  print qq|
    <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>|;
}

sub ListClose(%)
{
  print qq|
    </table>|;
}

sub RulesCommonListItem(%)
{
  my (%param) = @_;
  my ($title,$memo) = HTMLString( $param{title}, $param{memo} );
  my $reset_msg = $LANG->{static}{resetdef_confirm};
  $reset_msg =~ s/\$group/$title/g;
  $reset_msg =~ s/(['"\\])/\\$1/g;

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td align=center width=5%>|, ((defined $param{position})? (($param{position}+1).'.'): '&nbsp;'), qq|</td>
    <td><span class=gray_bold><a href=common_rules.cgi?id=$param{position}>$title</a></span>|, ((defined $memo)? "<br><span class=comment>$memo</span>": '') , qq|<br><span class=comment>$LANG->{static}{rules}:&nbsp;$param{rules_total}|, (($param{rules_modified})? ",&nbsp;$LANG->{static}{rules_modified}:&nbsp;$param{rules_modified}" : ''), qq|</span></td>
    <td width=12% nowrap>&nbsp;|, (($param{rules_modified})? qq|<a href=common_rules.cgi?id=$param{position}><img src=image/btn_editmod.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{modify}"></a><a href="common_rules.cgi?id=$param{position}&default=1&ok=1" onclick='return confirm("$reset_msg")'><img src=image/btn_revert.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{resetdef}"></a>|: qq|<a href=common_rules.cgi?id=$param{position}><img src=image/btn_edit.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{modify}"></a>|), qq|&nbsp;</td>
    </tr>|;
#    <td width=15% nowrap>|, (($param{rules_modified})? qq|&nbsp;<a href=common_rules.cgi?id=$param{position}><b>$LANG->{static}{modified}</b></a>&nbsp;<br>&nbsp;<a href="common_rules.cgi?id=$param{position}&default=1&ok=1" onclick='return confirm("$reset_msg")'>$LANG->{static}{resetdef}</a>&nbsp;|: qq|&nbsp;<a href=common_rules.cgi?id=$param{position}>$LANG->{static}{modify}</a>&nbsp;<br>&nbsp;|), qq|<span class=cookie><br>&nbsp;</span></td>
}


sub RulesGroupListItem(%)
{
  my (%param) = @_;
  my $gid = $param{gid};
  my ($title,$memo) = HTMLString( $param{title}, $param{memo} );
  my $reset_msg = $LANG->{static}{resetdef_confirm};
  $reset_msg =~ s/\$group/$title/g;
  $reset_msg =~ s/(['"\\])/\\$1/g;

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td align=center width=5%>|, ((defined $param{position})? (($param{position}+1).'.'): '&nbsp;'), qq|</td>
    <td><span class=gray_bold><a href=group_rules.cgi?gid=$gid&id=$param{position}>$title</a></span>|, ((defined $memo)? "<br><span class=comment>$memo</span>": '') , qq|<br><span class=comment>$LANG->{static}{rules}:&nbsp;$param{rules_total}|, (($param{rules_modified})? ",&nbsp;$LANG->{static}{rules_modified}:&nbsp;$param{rules_modified}" : ''), qq|</span></td>
    <td width=12% nowrap>|, (($param{rules_modified})? qq|&nbsp;<a href=group_rules.cgi?gid=$gid&id=$param{position}><img src=image/btn_editmod.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{modify}"></a><a href="group_rules.cgi?gid=$gid&id=$param{position}&default=1&ok=1" onclick='return confirm("$reset_msg")'><img src=image/btn_revert.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{resetdef}"></a>|: qq|&nbsp;<a href=group_rules.cgi?gid=$gid&id=$param{position}><img src=image/btn_edit.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{modify}"></a>|), qq|&nbsp;</td>
    </tr>|;
#    <td width=15% nowrap>|, (($param{rules_modified})? qq|&nbsp;<a href=group_rules.cgi?gid=$gid&id=$param{position}><b>$LANG->{static}{modified}</b></a>&nbsp;<br>&nbsp;<a href="group_rules.cgi?gid=$gid&id=$param{position}&default=1&ok=1" onclick='return confirm("$reset_msg")'>$LANG->{static}{resetdef}</a>&nbsp;|: qq|&nbsp;<a href=group_rules.cgi?gid=$gid&id=$param{position}>$LANG->{static}{modify}</a>&nbsp;<br>&nbsp;|), qq|<span class=cookie><br>&nbsp;</span></td>
}


sub GroupNewItem()
{
  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td colspan=2></td>
    <td width=12% nowrap>&nbsp;<a href=group_members.cgi?new=1><img src=image/btn_new.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{create_new_group}"></a>&nbsp;</td>
    </tr>|;
}

sub GroupListItem(%)
{
  my (%param) = @_;
  my ($pos,$gid) = ($param{position}, $param{gid});
  my ($title,$memo) = HTMLString($param{title},$param{memo});
  my $edit_uri = $gid ? "group_members.cgi?gid=$gid" : 'group_actions.cgi?gid=0';

  my $reset_msg = $LANG->{static}{removegroup_confirm};
  $reset_msg =~ s/\$group/$title/g;
  $reset_msg =~ s/(['"\\])/\\$1/g;

  my $pos_str = '&nbsp;';
  if( defined $pos )
  {
    my $pos_prev = ($pos)? $pos-1: 0;

    $pos_str = '&nbsp;'.($pos+1).'.&nbsp;';
    $pos_str .= ( $pos>0 && $gid )? "<a href=?up=$gid#pos_$pos_prev><img src=image/up.gif width=9 height=8 border=0 valign=middle></a>&nbsp;" : '<img src=image/e.gif width=9 height=8 border=0 valign=middle>&nbsp;';
  }

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td align=center width=5% nowrap><a name=pos_$pos></a>$pos_str</td>
    <td><span class=gray_bold><a href=$edit_uri>$title</a></span>|, ((defined $memo)? "<br><span class=comment>$memo</span>": '') , qq|</td>
    <td nowrap>&nbsp;<a href=$edit_uri><img src=image/btn_edit.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{modify}"></a>|;
  print qq|<a href=?delete=$gid onclick='return confirm("$reset_msg")'><img src=image/btn_delete.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{remove}"></a>&nbsp;| if( $gid );
  print qq|&nbsp;</td>
    </tr>|;

#    <td width=15% nowrap>&nbsp;<a href=$edit_uri>$LANG->{static}{modify}</a>&nbsp;|;
#  print qq|<br>&nbsp;<a href=?delete=$gid onclick='return confirm("$reset_msg")'>$LANG->{static}{remove}</a>&nbsp;| if( $gid );
#  print qq|<span class=cookie><br>&nbsp;</span></td>
#    </tr>|;
}


sub EnableGroupMenuItems($)
{
  my ($gid) = @_;
  foreach my $lvl1 (@$MENU)
  {
    if( $lvl1->{pmode} eq 'single' )
    {
      $lvl1->{disabled} = 0 unless($gid == 0 && $lvl1->{page} =~ /_members/);
      $lvl1->{hidden} = 0;
      $lvl1->{page} .= "?gid=$gid" if($lvl1->{page});
    }
    elsif( $lvl1->{pmode} eq 'common' )
    {
      $lvl1->{hidden} = 1;
    }
    if( defined $lvl1->{menu} && scalar(@{$lvl1->{menu}}) )
    {
      foreach my $lvl2 (@{$lvl1->{menu}})
      {
        if( $lvl2->{pmode} eq 'single' )
        {
          $lvl2->{disabled} = 0 unless($gid == 0 && $lvl2->{page} =~ /_members/);
          $lvl2->{hidden} = 0;
          $lvl2->{page} .= "?gid=$gid" if($lvl2->{page});
        }
        elsif( $lvl2->{pmode} eq 'common' )
        {
          $lvl2->{hidden} = 1;
        }
        if( defined $lvl2->{menu} && scalar(@{$lvl2->{menu}}) )
        {
          foreach my $lvl3 (@{$lvl2->{menu}})
          {
            if( $lvl3->{pmode} eq 'single' )
            {
              $lvl3->{disabled} = 0 unless($gid == 0 && $lvl3->{page} =~ /_members/);
              $lvl3->{hidden} = 0;
              $lvl3->{page} .= "?gid=$gid" if($lvl3->{page});
            }
            elsif( $lvl3->{pmode} eq 'common' )
            {
              $lvl3->{hidden} = 1;
            }
            if( defined $lvl3->{menu} && scalar(@{$lvl3->{menu}}) )
            {
              foreach my $lvl4 (@{$lvl3->{menu}})
              {
                if( $lvl4->{pmode} eq 'single' )
                {
                  $lvl4->{disabled} = 0 unless($gid == 0 && $lvl4->{page} =~ /_members/);
                  $lvl4->{hidden} = 0;
                  $lvl4->{page} .= "?gid=$gid" if($lvl4->{page});
                }
                elsif( $lvl4->{pmode} eq 'common' )
                {
                  $lvl4->{hidden} = 1;
                }
              }
            }
          }
        }
      }
    }
  }
}

sub EnableRulesMenuItems($$)
{
  my ($mark, $gid) = @_;
  foreach my $lvl1 (@$MENU)
  {
    unless( $lvl1->{$mark} )
    {
      $lvl1->{hidden} = 1;
    }
    if( $lvl1->{$mark . "_path"} )
    {
      $lvl1->{disabled_path} = 0;
      $lvl1->{page} = "group_rules_list.cgi?gid=$gid" if($mark eq 'grules');
    }
    if( defined $lvl1->{menu} && scalar(@{$lvl1->{menu}}) )
    {
      foreach my $lvl2 (@{$lvl1->{menu}})
      {
        unless( $lvl2->{$mark} )
        {
          $lvl2->{hidden} = 1;
        }
        if( $lvl2->{$mark . "_path"} )
        {
          $lvl2->{disabled_path} = 0;
          $lvl2->{page} = "group_rules_list.cgi?gid=$gid" if($mark eq 'grules');
        }
        if( defined $lvl2->{menu} && scalar(@{$lvl2->{menu}}) )
        {
          foreach my $lvl3 (@{$lvl2->{menu}})
          {
            unless( $lvl3->{$mark} )
            {
              $lvl3->{hidden} = 1;
            }
            if( $lvl3->{$mark . "_path"} )
            {
              $lvl3->{disabled_path} = 0;
              $lvl3->{page} = "group_rules_list.cgi?gid=$gid" if($mark eq 'grules');
            }
            if( defined $lvl3->{menu} && scalar(@{$lvl3->{menu}}) )
            {
              foreach my $lvl4 (@{$lvl3->{menu}})
              {
                unless( $lvl4->{$mark} )
                {
                  $lvl4->{hidden} = 1;
                }
                if( $lvl4->{$mark . "_path"} )
                {
                  $lvl4->{disabled_path} = 0;
                  $lvl4->{page} = "group_rules_list.cgi?gid=$gid" if($mark eq 'grules');
                }
              }
            }
          }
        }
      }
    }
  }
}

sub SetStatisticsMode($)
{
  my ($messages) = @_;
  my $data = '&data=' . ($messages?'m':'s');

  foreach my $lvl1 (@$MENU)
  {
    if( $lvl1->{page} =~ /^stat\.cgi\?/i )
    {
      $lvl1->{page} .= $data;
    }
    if( defined $lvl1->{menu} && scalar(@{$lvl1->{menu}}) )
    {
      foreach my $lvl2 (@{$lvl1->{menu}})
      {
        if( $lvl2->{page} =~ /^stat\.cgi\?/i )
        {
          $lvl2->{page} .= $data;
        }
        if( defined $lvl2->{menu} && scalar(@{$lvl2->{menu}}) )
        {
          foreach my $lvl3 (@{$lvl2->{menu}})
          {
            if( $lvl3->{page} =~ /^stat\.cgi\?/i )
            {
              $lvl3->{page} .= $data;
            }
            if( defined $lvl3->{menu} && scalar(@{$lvl3->{menu}}) )
            {
              foreach my $lvl4 (@{$lvl3->{menu}})
              {
                if( $lvl4->{page} =~ /^stat\.cgi\?/i )
                {
                  $lvl4->{page} .= $data;
                }
              }
            }
          }
        }
      }
    }
  }
}


#######################################################################


sub FormOpen(%)
{
  my (%attr) = @_;

  my $hidden = '';
  if( defined($attr{hidden}) )
  {
    $hidden = $attr{hidden};
    delete $attr{hidden};
  }

  DrawHeader(%attr);

  print qq|
    <form action="$attr{action}" method=post enctype="multipart/form-data">$hidden<table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>|;
}


sub FormClose()
{
  print qq|
    </table></form>|;
}


sub FormEmptyLine(%)
{
  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr><td width=100%>&nbsp;</td></tr>
      </table>
    </td>
    </tr>|;
}


sub FormStaticText(%)
{
  my (%param) = @_;

  my ($text) = HTMLString($param{text});
  my ($memo) = ( defined($param{memo}) && $param{memo} ne '' )? HTMLString($param{memo}): undef;

  $memo = (defined $memo)? "<br><span class=comment>$memo</span>": '';

  if( $param{header} )
  {
#    print qq|
#    <tr bgcolor=#ffffff valign=top>
#    <td>
#      <table width=100% cellpadding=3 border=0>
#      <tr><td width=100%><span class=gray_bold><br>$text</span>$memo<br></td></tr>
#      </table>
#    </td>
#    </tr>|;

    InfoClose();
    InfoOpen(title=>$param{text},memo=>$param{memo});
  }
  else
  {
    my ($text) = HTMLString($param{text});
    print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr><td width=100%>$text$memo</td></tr>
      </table>
    </td>
    </tr>|;
  }
}


sub FormControlRowOpen()
{
  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
    <table width=100% cellpadding=0 border=0>
    <tr>
    |;
}

sub FormControlRowClose()
{
  print qq|
    </tr>
    </table>
    </td>
    </tr>
    |;
}

sub FormSelect(%)
{
  my (%param) = @_;
  my @options = @{$param{options}};

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td colspan=3>
      <table width=100% cellpadding=3 border=0>
      <tr>
      <td width=$NAME_FORM_WIDTH><span class=gray_bold>|, $param{title}, qq|:</span></td>
      <td><select class=width name="$param{id}">|;

  foreach (@options)
  {
    next unless defined;
    my $selected = ( defined($param{selected}) && ($_->{val} eq $param{selected}) )? ' selected': '';
    print qq|<option value="$_->{val}"$selected>|, $_->{text};
  }

  print qq|
      </select></td>
      </tr>|;

  if( defined $param{memo} && $param{memo} ne '' )
  {
    my ($memo) = HTMLString($param{memo});
    print qq|
      <tr><td colspan=2><span class=comment>$memo</span></td></tr>
    |;
  }

  print qq|
      </table>
    </td>
    </tr>|;
}

sub FormEdit(%)
{
  my (%param) = @_;
  my $bgcolor = ($param{flash})? $TABLE_BGCOLOR_ALERT: $TABLE_BGCOLOR_DEFAULT;

  my $maxlen = '';
  $maxlen = " maxlength=\"$param{maxlen}\"" if( defined $param{maxlen} );

  my $disabled = '';
  $disabled = ' disabled' if( defined $param{disable} );

  my $type = 'text';
  $type = 'password' if( defined $param{password} );

  print qq|
    <tr bgcolor=$bgcolor valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr>
      <td width=$NAME_FORM_WIDTH><span class=gray_bold>|, $param{title}, qq|:</span></td>
      <td><input type=$type name="$param{id}" value="|, SafeEscapeHTML($param{value}), qq|" class=width$maxlen$disabled><br></td>
      </tr>|;

  if( defined $param{memo} && $param{memo} ne '' )
  {
    my ($memo) = HTMLString($param{memo});
    print qq|
      <tr><td colspan=2><span class=comment>$memo</span></td></tr>
    |;
  }

  print qq|
      </table>
    </td>
    </tr>|;
}

sub FormFile(%)
{
  my (%param) = @_;

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr>
      <td width=$NAME_FORM_WIDTH><span class=gray_bold>|, $param{title}, qq|:</span></td>
      <td><input type=file class=width size=51 name=uploaded_file value="|, HTMLString($param{value}), qq|"><br></td>
      </tr>|;

  if( defined $param{memo} && $param{memo} ne '' )
  {
    my ($memo) = HTMLString($param{memo});
    print qq|
      <tr><td colspan=2><span class=comment>$memo</span></td></tr>
    |;
  }

  print qq|
      </table>
    </td>
    </tr>|;
}

sub FormTextarea(%)
{
  my (%param) = @_;

  my $rows = ($param{rows})? " rows=$param{rows}": '';
  my $cols = ($param{cols})? " cols=$param{cols}": '';

  my $disabled = '';
  $disabled = ' disabled' if( defined $param{disable} );

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr valign=top>
      <td width=$NAME_FORM_WIDTH><span class=gray_bold>|, $param{title}, qq|:</span></td>
      <td><textarea name="$param{id}" class=width$rows$cols$disabled>|, HTMLString($param{value}), qq|</textarea></td>
      </tr>|;

  if( defined $param{memo} && $param{memo} ne '' )
  {
    my ($memo) = HTMLString($param{memo});
    print qq|
      <tr><td colspan=2><span class=comment>$memo</span></td></tr>
    |;
  }

  print qq|
      </table>
    </td>
    </tr>|;
}

sub FormCheck(%)
{
  my (%param) = @_;

  my $checked = $param{value}? ' checked': '';
  my $in_row = $param{in_row} ? 1 : 0;

  my $one_cell = $param{one_cell} == 1 ? 1 : 0;

  print qq|<tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>| unless($in_row);
  print qq|<td>|;

  print qq|
      <table width=100% cellpadding=3 border=0>
      <tr>
      |;

  if($one_cell)
  {
    print qq|<td>|;
  }
  else
  {
    print qq|<td width=$NAME_FORM_WIDTH>|;
  }

  print qq|<span class=gray_bold>|, $param{title}, qq|:</span>|;
  print qq|&nbsp;&nbsp;| if($one_cell);
  print qq|</td><td>| unless($one_cell);

  print qq|<input type=checkbox name="$param{id}" value=1 $checked>|;
  print qq|<br>| unless($one_cell);
  print qq|</td></tr>|;

  if( defined $param{memo} && $param{memo} ne '' )
  {
    my ($memo) = SafeEscapeHTML($param{memo});
    print qq|
      <tr><td colspan=2><span class=comment>$memo</span></td></tr>
    |;
  }

  print qq|</table>|;
  print qq|</td>|;
  print qq|</tr>| unless($in_row);
}


sub FormCheckRule(%)
{
  my (%param) = @_;

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr>
      <td width=$NAME_FORM_WIDTH><span class=gray_bold>|, $param{title}, qq|:</span></td>
      <td><select class=width name="$param{id}">|;

  foreach (@{$LANG->{control}{check_rule}{options}})
  {
    next unless defined;

    my $selected = '';
    if( defined($_->{val}) )
    {
       $selected = ' selected' if defined($param{value}) && ($_->{val} eq $param{value});
    }
    else
    {
       $selected = ' selected' if !defined($param{value}) || ('' eq $param{value});
    }

    print qq|<option value="$_->{val}"$selected>|, $_->{text};
  }

  print qq|
      </select></td>
      </tr>
      </table>
    </td>
    </tr>|;
}

sub FormActionBlock(%)
{
  my (%param) = @_;
  my @ACTIONS = qw(accept copy forward reject blackhole);
  my $is_normal = ($param{type} =~ /^(not[ _]detected)|(trusted)|(normal)/i);
  my $mode = $param{type};
  my %err_mark;

  %err_mark = %{$param{error_marks}} if defined($param{error_marks});

  my ($name) = HTMLString( $param{name} );
  my $memo = '';
  if( defined($param{memo}) && $param{memo} ne '' )
  {
    ($memo) = HTMLString( $param{memo} );
    $memo = "<br><span class=comment>$memo</span>";
  }

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=$ACTION_TABLE_WIDTH cellpadding=3 border=0>
      <tr><td colspan=2><span class=gray_bold>$LANG->{actions}{name_prefix} $name:</span>$memo</td></tr>
    |;

  if( $is_normal )
  {
    print qq|
      </table><input type=hidden name=$mode\_action value=accept>|;
  }
  else
  {
    my %selected;
    foreach (@ACTIONS)
    {
      $selected{$_} = ($param{action} eq $_)? ' selected': '';
    }

    my $bgcolor = ($err_mark{sendto})? " bgcolor=$TABLE_BGCOLOR_ALERT": '';

    print qq|
      <tr>
      <td width=$ACTION_LEFT_MARGINE nowrap>&nbsp;</td>
      <td width=$ACTION_SELECT_WIDTH>
      <select name=$mode\_action id=$mode\_action style="width: |, $ACTION_SELECT_WIDTH-6, qq|px;" onchange="Show_One('$mode');">
        <option value=accept$selected{accept}>$LANG->{actions}{actions}{accept}</option>
        <option value=copy$selected{copy}>$LANG->{actions}{actions}{copy}</option>
        <option value=forward$selected{forward}>$LANG->{actions}{actions}{forward}</option>
        <option value=reject$selected{reject}>$LANG->{actions}{actions}{reject}</option>
        <option value=blackhole$selected{blackhole}>$LANG->{actions}{actions}{blackhole}</option>
      </select>
      </td>
      </tr></table>

      <table width=$ACTION_TABLE_WIDTH cellpadding=3 cellspacing=0 id=$mode\_sendto_block><tr$bgcolor>
      <td width=$ACTION_LEFT_MARGINE></td>
      <td nowrap width=$ACTION_NAME_WIDTH>&nbsp;&nbsp;$LANG->{actions}{edit}{sendto}:&nbsp;&nbsp;</td>
      <td width=$ACTION_NAME_CTRL align=right><input class=width name=$mode\_sendto type=text value="|, HTMLString($param{sendto}), qq|"></td>
      </tr></table>|;
  }

    my $bgcolor_subject_prefix = ($err_mark{subject_prefix})? " bgcolor=$TABLE_BGCOLOR_ALERT": '';
#    my $bgcolor_subject_suffix = ($err_mark{subject_suffix})? " bgcolor=$TABLE_BGCOLOR_ALERT": '';
#    my $bgcolor_category       = ($err_mark{category})? " bgcolor=$TABLE_BGCOLOR_ALERT": '';
    my $bgcolor_x_header       = ($err_mark{x_header})? " bgcolor=$TABLE_BGCOLOR_ALERT": '';

    print qq|

      <table width=$ACTION_TABLE_WIDTH cellspacing=0 cellpadding=3 id=$mode\_subject_prefix_block><tr$bgcolor_subject_prefix>
      <td width=$ACTION_LEFT_MARGINE></td>
      <td nowrap width=$ACTION_NAME_WIDTH>&nbsp;&nbsp;$LANG->{actions}{edit}{subject_pre}:&nbsp;&nbsp;</td>
      <td width=$ACTION_NAME_CTRL align=right><input name=$mode\_subject_prefix class=width maxlength=$CONST->{etc}{subject_prefix_limit} type=text value="|, SafeEscapeHTML($param{subject_prefix}), qq|"></td>
      </tr></table>

      <table width=$ACTION_TABLE_WIDTH cellpadding=3 cellspacing=0 id=$mode\_x_header_block><tr$bgcolor_x_header>
      <td width=$ACTION_LEFT_MARGINE></td>
      <td nowrap width=$ACTION_NAME_WIDTH>&nbsp;&nbsp;$LANG->{actions}{edit}{x_header}:&nbsp;&nbsp;</td>
      <td width=$ACTION_NAME_CTRL align=right><input name=$mode\_x_header class=width maxlength=$CONST->{etc}{x_header_limit} type=text value="|, HTMLString($param{x_header}), qq|"></td>
      </tr></table>
    </td>
    </tr>|;

#    print qq|
#
#      <table width=$ACTION_TABLE_WIDTH cellspacing=0 cellpadding=3 id=$mode\_subject_prefix_block><tr$bgcolor_subject_prefix>
#      <td width=$ACTION_LEFT_MARGINE></td>
#      <td nowrap width=$ACTION_NAME_WIDTH>&nbsp;&nbsp;$LANG->{actions}{edit}{subject_pre}:&nbsp;&nbsp;</td>
#      <td width=$ACTION_NAME_CTRL align=right><input name=$mode\_subject_prefix class=width type=text value="|, SafeEscapeHTML($param{subject_prefix}), qq|"></td>
#      </tr></table>
#
#      <table width=$ACTION_TABLE_WIDTH cellpadding=3 cellspacing=0 id=$mode\_subject_suffix_block><tr$bgcolor_subject_suffix>
#      <td width=$ACTION_LEFT_MARGINE></td>
#      <td nowrap width=$ACTION_NAME_WIDTH>&nbsp;&nbsp;$LANG->{actions}{edit}{subject_suf}:&nbsp;&nbsp;</td>
#      <td width=$ACTION_NAME_CTRL align=right><input name=$mode\_subject_suffix class=width type=text value="|, SafeEscapeHTML($param{subject_suffix}), qq|"></td>
#      </tr></table>
#
#      <table width=$ACTION_TABLE_WIDTH cellpadding=3 cellspacing=0 id=$mode\_category_block><tr$bgcolor_category>
#      <td width=$ACTION_LEFT_MARGINE></td>
#      <td nowrap width=$ACTION_NAME_WIDTH>&nbsp;&nbsp;$LANG->{actions}{edit}{category}:&nbsp;&nbsp;</td>
#      <td width=$ACTION_NAME_CTRL align=right><input name=$mode\_category class=width type=text value="|, HTMLString($param{category}), qq|"></td>
#      </tr></table>
#
#      <table width=$ACTION_TABLE_WIDTH cellpadding=3 cellspacing=0 id=$mode\_x_header_block><tr$bgcolor_x_header>
#      <td width=$ACTION_LEFT_MARGINE></td>
#      <td nowrap width=$ACTION_NAME_WIDTH>&nbsp;&nbsp;$LANG->{actions}{edit}{x_header}:&nbsp;&nbsp;</td>
#      <td width=$ACTION_NAME_CTRL align=right><input name=$mode\_x_header class=width type=text value="|, HTMLString($param{x_header}), qq|"></td>
#      </tr></table>
#    </td>
#    </tr>|;
}

sub FormSubmitOK(%)
{
  my (%param) = @_;

  my $onclick_apply = (defined($param{onclick_apply}) && $param{onclick_apply} ne '')? qq| onclick="$param{onclick_apply}"| : '';
  my $onclick_reset = (defined($param{onclick_reset}) && $param{onclick_reset} ne '')? qq| onclick="$param{onclick_reset}"| : '';
  my $disabled = '';
  if( $param{disable} )
  {
    $disabled = ' disabled';
    $onclick_apply = $onclick_reset = '';
  }

  my $reset_btn = qq|<input type=reset class=button value="&nbsp;$LANG->{static}{'reset'}&nbsp;"$onclick_reset$disabled>&nbsp;&nbsp;|;
  $reset_btn = '' if( $param{no_reset} );

  my $hidden = (defined $param{hidden})? $param{hidden}: '';
  my $colspan = (defined($param{colspan}) && $param{colspan}>1)? " colspan=$param{colspan}": '';

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td$colspan>
      <table width=100% cellpadding=3 border=0>
      <tr>
      <td align=right nowrap>
      <input type=submit class=button name=ok value="&nbsp;|, $LANG->{static}{ok}, qq|&nbsp;"$onclick_apply$disabled>&nbsp;&nbsp;
      $reset_btn$hidden
      </td>
      </tr>
      </table>
    </td>
    </tr>|;
}


sub FormRulesSubmit(%)
{
  my (%param) = @_;

  my $onclick_apply   = (defined($param{onclick_apply}) && $param{onclick_apply} ne '')? qq| onclick="$param{onclick_apply}"| : '';
  my $onclick_reset   = (defined($param{onclick_reset}) && $param{onclick_reset} ne '')? qq| onclick="$param{onclick_reset}"| : '';
  my $onclick_default = (defined($param{onclick_default}) && $param{onclick_default} ne '')? qq| onclick="$param{onclick_default}"| : '';

  my $ctl = (defined($param{rules_group_id}) && $param{rules_group_id}>=0)? qq|<input type=hidden name="id" value="$param{rules_group_id}">| : '';
  $ctl .= (defined($param{group_id}) && $param{group_id}>=0)? qq|<input type=hidden name="gid" value="$param{group_id}">| : '';

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr>
      <td align=right nowrap>$ctl
      <input type=submit class=button name="ok" value="&nbsp;|, $LANG->{static}{ok}, qq|&nbsp;"$onclick_apply>&nbsp;&nbsp;
      <input type=reset class=button value="&nbsp;|, $LANG->{static}{'reset'}, qq|&nbsp;"$onclick_reset>&nbsp;&nbsp;
      <input type=submit class=button name="default" value="&nbsp;|, $LANG->{static}{default}, qq|&nbsp;"$onclick_default>&nbsp;&nbsp;
      </td>
      </tr>
      </table>
    </td>
    </tr>|;
}


sub FormRebuildButton(%)
{
  my (%param) = @_;
  my $onclick_apply = (defined($param{onclick_apply}) && $param{onclick_apply} ne '')? qq| onclick="$param{onclick_apply}"| : '';

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr>
      <td align=center nowrap>
      <input type=submit class=button name=build value="&nbsp;|, $LANG->{static}{rebuild_policy}, qq|&nbsp;"$onclick_apply>
      </td>
      </tr>
      </table>
    </td>
    </tr>|;
}

sub FormListTypeSelector(%)
{
  my (%param) = @_;
  my $separator = 0;
  my $hidden = '';
  my $colspan = '';

  return unless defined $param{selector};

  $hidden .= qq|<input type=hidden name=type value="$param{type}">| if defined $param{type};
  $colspan = qq| colspan=$param{colspan}| if defined($param{colspan}) && $param{colspan}>1;

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td$colspan>
      <table width=100% cellpadding=0 border=0>
      <tr>
      <td align=right nowrap>|;

  foreach my $item (@{$param{selector}})
  {
    next unless defined($item) && defined($item->{name});
    $item->{url} = '#' unless defined($item->{url});

    print '&nbsp;<span class=gray>|</span>&nbsp;' if $separator++;
    if( $item->{active} )
    {
      print qq|<span class=sel>$item->{name}</span>|;
    }
    else
    {
      print qq|<a href="$item->{uri}">$item->{name}</a>|;
    }
  }

  print qq|$hidden</td>
      </tr>
      </table>
    </td>
    </tr>|;
}

sub FormListCommands(%)
{
  my (%param) = @_;
  my $separator = 0;
  my $hidden = '';

  return unless defined $param{selector};

  $hidden .= qq|<input type=hidden name=type value="$param{type}">| if defined $param{type};

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td colspan=2></td>
    <td>
      <table width=100% cellpadding=0 border=0>
      <tr>
      <td nowrap>|;

  foreach my $item (@{$param{selector}})
  {
    next unless defined($item) && defined($item->{name});
    $item->{url} = '#' unless defined($item->{url});

    print '&nbsp;<span class=gray>|</span>&nbsp;' if $separator++;
    print qq|<a href="$item->{uri}"><b>$item->{name}</b></a>|;
  }

  print qq|$hidden</td>
      </tr>
      </table>
    </td>
    </tr>|;
}


sub FormDNSBLItem(%)
{
  my (%param) = @_;

  my $count_column = ($param{count}>0)? "<td align=center>&nbsp;$param{count}&nbsp;</td>" : '<td>&nbsp;</td>';
  my ($a_open,$a_close) = ('', '');
  if( $param{href} )
  {
    $a_open  = qq|<a href="$param{href}" target=_blank>|;
    $a_close = '</a>';
  }

  my $hostname = (HTMLString($param{hostname}))[0];

  my $delete_msg = $LANG->{static}{removednsbl_confirm};
  $delete_msg =~ s/\$hostname/$hostname/g;
  $delete_msg =~ s/(['"\\])/\\$1/g;


  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
      $count_column
      <td>&nbsp;$a_open$hostname$a_close&nbsp;</td>
      <td><input type=text name="$param{id}" value="|, int($param{value}), qq|" maxlength=3 class=width></td>
      <td align=center>&nbsp;<a href="dnsbl.cgi?delete=$param{hostname}" onclick='return confirm("$delete_msg")'><img src=image/btn_delete.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{remove}"></a>&nbsp;</td>
    </tr>|;
}

sub FormKasDNSBLItem(%)
{
  my (%param) = @_;

  my $count_column = ($param{count}>0)? "<td align=center>&nbsp;$param{count}&nbsp;</td>" : '<td>&nbsp;</td>';
  my ($a_open,$a_close) = ('', '');
  if( $param{href} )
  {
    $a_open  = qq|<a href="$param{href}" target=_blank>|;
    $a_close = '</a>';
  }

  my $hostname = (HTMLString($param{hostname}))[0];

  my $delete_msg = $LANG->{static}{removednsbl_confirm};
  $delete_msg =~ s/\$hostname/$hostname/g;
  $delete_msg =~ s/(['"\\])/\\$1/g;


  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
      $count_column
      <td>&nbsp;$a_open$hostname$a_close&nbsp;</td>
      <td align=center>&nbsp;$param{value}&nbsp;</td>
    </tr>|;
}

sub FormDNSBLTitle()
{
  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
      <td width=5%>&nbsp;</td>
      <td align=center>&nbsp;<span class=gray_bold>$LANG->{static}{dnsbl_hostname}</span>&nbsp;</td>
      <td width=15% align=center>&nbsp;<span class=gray_bold>$LANG->{static}{dnsbl_rate}</span>&nbsp;</td>
      <td width=10%>&nbsp;</td>
    </tr>|;
}

sub FormKasDNSBLTitle()
{
  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
      <td width=5%>&nbsp;</td>
      <td align=center>&nbsp;<span class=gray_bold>$LANG->{static}{dnsbl_hostname}</span>&nbsp;</td>
      <td width=15% align=center>&nbsp;<span class=gray_bold>$LANG->{static}{dnsbl_rate}</span>&nbsp;</td>
    </tr>|;
}

sub FormDNSBLNew(%)
{
  my (%param) = @_;
  my $host = SafeEscapeHTML( defined($param{newhost})? $param{newhost}: '' );
  my $rate = '';

  if( defined($param{newrate}) )
  {
    $rate = int($param{newrate});
    $rate = 1 if( $rate < 1 );
    $rate = 100 if( $rate > 100 );
  }

  print qq|
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
      <td align=center>&nbsp;$LANG->{static}{add_sign}&nbsp;</td>
      <td><input type=text name="newhost" value="$host" maxlength=128 class=width></td>
      <td><input type=text name="newrate" value="$rate" maxlength=3 class=width></td>
      <td>&nbsp;</td>
    </tr>|;
}


sub ErrorBlock(%)
{
  my (%param) = @_;
  my ($msg,$mode,$img);

  foreach (qw(error warning notice info))
  {
    if( defined $param{$_} )
    {
      $msg = $param{$_};
      $mode = $_;
      last;
    }
  }
  return unless defined($mode) && defined($msg);

  my $descs = (defined($param{descriptions}))? ($param{descriptions}): \@ERR;
  my $class = ( $mode =~ /^error$/i )? 'error': ( $mode =~ /^warning$/i )? 'warn' : 'table_title';
  if( $param{top} )
  {
    return if $mode !~ /^(error)|(warning)$/i;

    my $details = ($descs && scalar(@{$descs}))? "<a href=#errdetails>$LANG->{static}{details}</a>": '&nbsp;';

    # Show top block
    print qq|
      <table cellpadding=0 cellspacing=0 width=100% border=0><tr><td><img src=image/e.gif border=0 alt="" width=1 height=12><br></td></tr></table>
      <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>
      <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top><td>
       <table width=100% cellpadding=3 border=0><tr>
        <td width=24><a href=#errdetails><img src="image/$MODES_ICO{$mode}{med}" width=24 height=24 border=0 alt=""></a></td>
        <td><h1 class=$class>$msg</h1></td>
        <td align=right>$details</td>
       </tr></table>
      </td></tr>
      </table>|;
  }
  else
  {
    # Show bottom block

    print qq|
      <a name=errdetails></a><table cellpadding=0 cellspacing=0 width=100% border=0><tr><td><img src=image/e.gif border=0 alt="" width=1 height=18><br></td></tr></table>
      <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>
      <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top><td>
       <table width=100% cellpadding=3 border=0><tr>
        <td width=24><img src="image/$MODES_ICO{$mode}{med}" width=24 height=24 alt=""></td>
        <td><h1 class=$class>$msg</h1></td>
       </tr></table>
      </td></tr>|;

    if( $mode =~ /^(error)|(warning)$/i && defined $descs && scalar(@{$descs}) )
    {
      print qq|
      <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top><td>
       <table width=100% cellpadding=3 border=0><tr><td>|;

      foreach my $msg (@{$descs})
      {
        if( $msg =~ /\n/s )
        {
          ($msg) = HTMLString($msg);
          $msg =~ s/\n/<br>/sg;
          print "<p><tt>$msg</tt>\n";
        }
        else
        {
          print "<p>", HTMLString($msg), "\n";
        }
      }

      print qq|
       </td></tr></table>
      </td></tr>|;
    }

    print "</table>";
  }
}

sub InfoKeyfiles(%)
{
  my %param = @_;
  my @keys   = ($param{'keys'})? @{$param{'keys'}} : ();

  print qq|
    <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>
    |;

  if( @keys )
  {
    print qq|
    <tr valign=top bgcolor=$TABLE_BGCOLOR_DEFAULT>
      <td align=center><span class=gray_bold>$LANG->{static}{file}</span></td>
      <td align=center><span class=gray_bold>$LANG->{static}{serial}</span></td>
      <td align=center><span class=gray_bold>$LANG->{static}{type}</span></td>
      <td align=center><span class=gray_bold>$LANG->{static}{amount}</span></td>
      <td align=center><span class=gray_bold>$LANG->{static}{valid}</span></td>
    </tr>|;

    foreach (@keys)
    {
      print qq|
      <tr valign=top bgcolor=$TABLE_BGCOLOR_DEFAULT>
      <td align=center>|, HTMLString($_->{file}), qq|</td>
      <td align=center>$_->{serial}</td>
      <td align=center>$_->{type} ($_->{version})</td>
      <td align=center>$_->{amount}</td>
      <td align=left>&nbsp;|,
      
      ##strftime($LANG->{timefmt}{date},localtime($_->{valid})),
      FormatDateTime($LANG->{tm_format}{date},localtime($_->{valid})),

      qq|</td>
    </tr>|;
    }
  }
  else
  {
    print qq|
    <tr valign=top bgcolor=$TABLE_BGCOLOR_DEFAULT>
      <td align=center><span class=error>$LANG->{error}{no_keys_found}</span></td>
    </tr>|;
  }

  print qq|
    </table>|;
}


#######################################################################

sub ShowRulesGroup(%)
{
  my (%param) = @_;

  my $group = $param{rules_group};
  my $values = $param{value};
  my $defaults = $param{with_defaults};

  return unless(defined($group) && defined($group->{Control}) && defined($values));

  foreach my $ctl (@{$group->{Control}})
  {
    next unless defined($ctl) && defined($ctl->{Type});
    next unless( $ctl->{Type}<=0 || defined($ctl->{ID}) );

    my $type = $ctl->{Type};
    my $ctl_id = defined($ctl->{ID})? ( '.'.$ctl->{ID} ) : undef;

    if( $type == 1 )
    {
      # Checkbox
      my @options;
      push @options, { val=>'', text=>$LANG->{static}{check_default} } if( $defaults );
      push @options, ( { val=>'0', text=>$LANG->{static}{check_off} }, { val=>'1', text=>$LANG->{static}{check_on} } );

      my $selected = ($defaults)? '': '0';
      $selected = (($values->{$ctl->{ID}})?'1':'0') if defined $values->{$ctl->{ID}};

      FormSelect( title=>$ctl->{Name}, memo=>$ctl->{Memo}, id=>$ctl_id, selected=>$selected, options=>\@options );
    }
    elsif( $type == 2 )
    {
      # Select
      next unless defined($ctl->{Select}) && scalar(@{$ctl->{Select}});

      my @options;
      push @options, { val=>'', text=>$LANG->{static}{check_default} } if( $defaults );

      foreach my $opt (@{$ctl->{Select}})
      {
        next unless defined $opt->{Value};
        push @options, { val=>$opt->{Value}, text=>$opt->{Text} };
      }

      my $selected = '';
      $selected = $values->{$ctl->{ID}} if defined $values->{$ctl->{ID}};
      FormSelect( title=>$ctl->{Name}, memo=>$ctl->{Memo}, id=>$ctl_id, selected=>$selected, options=>\@options );
    }
    elsif( $type == 3 )
    {
      # Edit
      next unless defined($ctl->{Edit});
      my $edit = $ctl->{Edit};
      my $value = (defined $values->{$ctl->{ID}})? $values->{$ctl->{ID}}: '';
      my $flash = 0;

      unless( $defaults && (!defined($value) || $value eq '') )
      {
        if( defined($edit->{ValidateRegex}) && ($edit->{ValidateRegex} !~ /^\s*$/ ) )
        {
          my $rx = $edit->{ValidateRegex};
          eval{ $flash = ( $value !~ /$rx/i ); };
        }
      }

      $edit->{MaxLen} = undef unless( defined($edit->{MaxLen}) && $edit->{MaxLen}>0 );
      $edit->{Width} = undef unless( defined($edit->{Width}) && $edit->{Width}>0 );

      FormEdit( title=>$ctl->{Name}, memo=>$ctl->{Memo}, id=>$ctl_id, value=>$value, maxlen=>$edit->{MaxLen}, width=>$edit->{Width}, flash=>$flash );
    }
    elsif( $type == 4 )
    {
      # Number
      next unless defined($ctl->{Number});
      my $edit = $ctl->{Number};

      $edit->{Min} = -1e19 unless defined $edit->{Min};
      $edit->{Max} = 1e19 unless defined $edit->{Max};

      ($edit->{Min},$edit->{Max}) = ($edit->{Max},$edit->{Min}) if( $edit->{Min} > $edit->{Max} );

      my ($value, $flash);
      if( $defaults )
      {
        $value = (defined($values->{$ctl->{ID}}))? int($values->{$ctl->{ID}}): '';
        $flash = ( $value < $edit->{Min} || $value > $edit->{Max} ) if( defined($value) && $value ne '' );
      }
      else
      {
        if( defined $values->{$ctl->{ID}} )
        {
          $value = int($values->{$ctl->{ID}});
          $flash = ( $value < $edit->{Min} || $value > $edit->{Max} );
        }
        else
        {
          $value = ( $edit->{Min} + $edit->{Max} ) / 2;
        }
      }

      FormEdit( title=>$ctl->{Name}, memo=>$ctl->{Memo}, id=>$ctl_id, value=>$value, maxlen=>$edit->{Width}, width=>$edit->{Width}, flash=>$flash );
    }
    elsif( $type == 5 )
    {
      # Textarea
      next unless defined($ctl->{Textarea});
      my $edit = $ctl->{Textarea};

      my $value = (defined $values->{$ctl->{ID}})? $values->{$ctl->{ID}}: '';

      $edit->{Width} = undef unless( defined($edit->{Width}) && $edit->{Width}>0 );
      $edit->{Height} = undef unless( defined($edit->{Height}) && $edit->{Height}>0 );

      FormTextarea( title=>$ctl->{Name}, memo=>$ctl->{Memo}, id=>$ctl_id, value=>$value, rows=>$edit->{Height}, cols=>$edit->{Width} );
    }
    elsif( $type==-1 || $type==-2 )
    {
      # Title & Static $ctl->{Memo}
      FormStaticText( text=>$ctl->{Name}, memo=>$ctl->{Memo}, header=>(($type==-1)?1:0) );
    }
    else
    {
      # Empty line
      FormEmptyLine();
    }
  }
}


#######################################################################


sub ShowLog(%)
{
  my (%param) = @_;
  DrawHeader(%param);

  my $url = $param{uri};
  $url =~ s/\?.*//;

  print qq|
    <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>
    <tr bgcolor=$TABLE_BGCOLOR_DEFAULT valign=top>
    <td>
      <table width=100% cellpadding=3 border=0>
      <tr valign=middle>
      <td width=$NAME_FORM_WIDTH align=right><span class=gray_bold>$LANG->{control}{log_filter}{title}:&nbsp;</span></td>
      <td><form target=log method=get action="$url"><input type=hidden name=type value="$param{type}"><select class=logtype name=level size=1 onchange="document.getElementById('log').src = '$url?type=$param{type}&level='+this.options[this.selectedIndex].value; return 0;">
        <option value=info>$LANG->{control}{log_filter}{info}
        <option value=notice selected>$LANG->{control}{log_filter}{notice}
        <option value=warning>$LANG->{control}{log_filter}{warning}
        <option value=error>$LANG->{control}{log_filter}{error}
        </select></td>
      <td width=$NAME_FORM_WIDTH align=left><noscript>&nbsp;<input type=submit value="&nbsp;$LANG->{static}{ok}&nbsp;"></noscript></form></td>
      </tr>
      </table>
    </td>
    </tr>
    </table>

    <table width=100% cellpadding=3 border=0>
    <tr valign=top><td><iframe name=log id=log src="$url?type=$param{type}&level=notice" width="100%" height=305 scrolling="auto" frameborder="0" class=frame_b></iframe></td></tr>
    </table>
  |;
}


sub ShowLogFile(%)
{
  my (%param) = @_;

  print qq|<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
  <html>
  <head>
  <title>$LANG->{static}{logo}</title>
  <link rel="stylesheet" type="text/css" href="style.css">
  <link rel="Shortcut Icon" type="image/x-icon" href="image/favicon.ico">
  </head>
  <body bgcolor="#ffffff" link="#005447" alink="#FF0000" vlink="#FF0000" text="#000000" topmargin=0 leftmargin=0 rightmargin=0 marginwidth=0 marginheight=0>
  |;

  if( defined($param{'log'}) && scalar(@{$param{'log'}}) )
  {
    my $count = 0;
    foreach (@{$param{'log'}})
    {
      my ($status,$time,$text) = ($_->{status},$_->{'time'},$_->{text});
      $status = (defined($status) && defined($MODES_ICO{$status}))? $MODES_ICO{$status}{small} : 'e.gif';
      $time = strftime $LANG->{timefmt}{datetime_ylong}, localtime($time);
      $text = SafeEscapeHTML($text);

      my $class = (($count++)%2)? 'logrow_odd': 'logrow_even';
      print qq|
      <div class=$class><nobr><img src=image/$status border=0 width=16 height=16 align=absmiddle vspace=4 hspace=5>&nbsp;$time&nbsp;&nbsp;$text</nobr></div>|;
    }
  }
  else
  {
      print qq|
      <p>$LANG->{static}{log_no_events}|;
  }

  print qq|
  </body>
  </html>|;
}

1;
__END__
