#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use vars qw($LANG);

my $product_name = 'Kaspersky Anti-Spam';
my $title_suffix = "- $product_name";

$LANG = {
          locale  => 'C',
          timefmt => {
                        date       => '%Y-%m-%d',
                        time_short => '%H:%M',
                        time_long  => '%H:%M:%S',
                        datetime_short => '%Y-%m-%d %H:%M',
                        datetime_long  => '%Y-%m-%d %H:%M:%S',
                        datetime_ylong => '%m-%d %H:%M:%S',
                        timespan_long  => '%d days',
                        timespan_middle=> '%d days %02d:%02d:%02d',
                        timespan_short => '%02d:%02d:%02d',
                     },
          
          tm_format =>  {
                            date => '%mon %d %Y',
                        },

          month_names_short =>  {
                        1 => 'Jan',
                        2 => 'Feb',
                        3 => 'Mar',
                        4 => 'Apr',
                        5 => 'May',
                        6 => 'Jun',
                        7 => 'Jul',
                        8 => 'Aug',
                        9 => 'Sep',
                        10 => 'Oct',
                        11 => 'Nov',
                        12 => 'Dec'
                     },
          menu => {
                    items => {
                                tab_monitoring  => 'Monitoring',
                                tab_license     => 'License',
                                tab_statistics  => 'Statistics',
                                tab_policy      => 'Policies',
                                tab_settings    => 'Settings',

                                mon_header      => 'Monitoring',
                                status      => 'General Status',
                                antispam    => 'Anti-Spam Engine',
                                updates     => 'Updates',
                                license     => 'License',

                                lic_info    => 'License Keys',

                                stat_header => 'Statistics',
                                daily       => 'Last Day',
                                weekly      => 'Last Week',
                                monthly     => 'Last Month',
                                annually    => 'Last Year',

                                pcommon     => 'Common',
                                rules       => 'Default Rules',
                                dnsbl       => 'DNS Black Lists',
                                blocked     => 'Black List',
                                allowed     => 'White List',

                                rules_item  => '$rules_group_name',

                                pgroup      => 'Groups',
                                grlist      => 'Group List',
                                group_new   => 'New Group',

                                single_group => 'Group Policy',
                                members      => 'General',
                                actions      => 'Actions',
                                group_rules  => 'Rules',

                                build       => 'Build',
                                rebuild_policy => 'Rebuild All Policies',

                                settings_antispam => 'Anti-Spam Engine',
                                settings_common   => 'Common',
                                settings_server   => 'Process Server',
                                settings_filter   => 'Filtration Process',
                                settings_checks   => 'Check Options',
                                settings_clients  => 'MTA Clients',
                                settings_messages => 'Reject Messages',

                                settings_maintenence => 'Maintenance',
                                settings_updater  => 'Updater',

                                cc          => 'Control Center',

                                protected_domains => 'Protected Domains',
                             },

                    title => {
                                default         => $product_name,

                                'index'         => "General Status $title_suffix",
                                as_status       => "Monitoring : Anti-Spam Engine $title_suffix",
                                asup_status     => "Monitoring : Updates $title_suffix",
                                aslic_status    => "Monitoring : License $title_suffix",

                                lic_info        => "License Keys $title_suffix",

                                stat_d          => "Daily Statistics $title_suffix",
                                stat_w          => "Weekly Statistics $title_suffix",
                                stat_m          => "Monthly Statistics $title_suffix",
                                stat_y          => "Annual Statistics $title_suffix",

                                common_rules    => "Default Rules $title_suffix",
                                common_rules_item => "Default Rules : \$rules_group_name $title_suffix",
                                dnsbl           => "DNS Black Lists $title_suffix",
                                common_blocked  => "Common Black List $title_suffix",
                                common_allowed  => "Common White List $title_suffix",

                                group_list      => "Group List $title_suffix",
                                group_new       => "New Group $title_suffix",

                                group_members     => "Group Policy : General Settings $title_suffix",
                                group_actions     => "Group Policy : Actions $title_suffix",
                                group_rules       => "Group Policy : Rules $title_suffix",
                                group_rules_item  => "Group Policy : Rules : \$rules_group_name $title_suffix",
                                group_blocked     => "Group Policy : Black List $title_suffix",
                                group_allowed     => "Group Policy : White List $title_suffix",

                                ##group_rules       => "Group '\$group_name': \$rules_group_name $title_suffix",

                                rebuild_policy  => "Rebuild All Policies",

                                settings_common   => "Common Settings $title_suffix", 
                                settings_server   => "Process Server Settings $title_suffix",
                                settings_filter   => "Filtration Process Settings $title_suffix",
                                settings_checks   => "Check Options $title_suffix",
                                settings_clients  => "MTA Clients Settings $title_suffix",
                                settings_messages => "Reject Messages $title_suffix",
                                settings_updater  => "Updater Settings $title_suffix",

                                cc              => "Control Center Settings $title_suffix",

                                protected_domains => "Protected Domains $title_suffix",
                             },
                  },
          group  => {
                      system_status     => { title=>'System Information', memo=>undef },
                      as_system         => { title=>$product_name, memo=>undef },

                      as_status         => { title=>'Monitoring: Anti-Spam Engine', memo=>undef },
                      as_log            => { title=>'Last Anti-Spam Engine Events', memo=>undef },

                      asup_status       => { title=>'Monitoring: Updates', memo=>undef },
                      asup_log          => { title=>'Last Updater Events', memo=>undef },

                      aslic_status      => { title=>'Monitoring: License', memo=>undef },
                      aslic_log         => { title=>'Last License Daemon Events', memo=>undef },

                      license           => { title=>'Active License Information', memo=>undef },
                      license_keys      => { title=>'License Key Files', memo=>undef },
                      license_update    => { title=>'Install a New License Key', memo=>undef },

                      stat_d            => { title=>'Daily Statistics', memo=>'Last 24 hours data' },
                      stat_w            => { title=>'Weekly Statistics', memo=>'Last 7 days data' },
                      stat_m            => { title=>'Monthly Statistics', memo=>'Last 30 days data' },
                      stat_y            => { title=>'Annual Statistics', memo=>'Last 365 days data' },

                      common_rules_list => { title=>'Default Rules',
                                      memo=>'Attention! Settings below can be redefined in Group Policy.' },
                      common_rules      => { title=>'Default Rules: $rules_group_name',
                                      memo=>undef },
                      dnsbl             => { title=>'User Defined DNS Black Lists',
                                      memo=>'List of user defined DNS-based Black List services' },
                      kas_dnsbl         => { title=>'Default DNS Black Lists',
                                      memo=>'List of default DNS-based Black List services' },
                      common_blocked    => { title=>'Common Black List',
                                      memo=>'Common black list of e-mail and ip addresses' },
                      common_allowed    => { title=>'Common White List',
                                      memo=>'Common list of trusted e-mail and ip addresses' },

                      group_list        => { title=>'Group List',  memo=>'List of recipient groups' },
                      group_new         => { title=>'New Group',   memo=>undef },

                      group_members     => { title=>"General Settings&nbsp;-&nbsp;Group: \$group_name",
                                      memo=>"Group name, description and member list" },
                      group_actions     => { title=>"Actions&nbsp;-&nbsp;Group: \$group_name",
                                      memo=>"Actions to be performed on incoming messages" },
                      group_rules_list  => { title=>"Rules&nbsp;-&nbsp;Group: \$group_name",
                                      memo=>undef },
                      group_rules       => { title=>"Rules&nbsp;-&nbsp;\$rules_group_name&nbsp;-&nbsp;Group: \$group_name",
                                      memo=>undef },
                      group_blocked     => { title=>"Black List&nbsp;-&nbsp;Group: \$group_name",
                                      memo=>'Black list of e-mail and ip addresses' },
                      group_allowed     => { title=>"White List&nbsp;-&nbsp;Group: \$group_name",
                                      memo=>'List of trusted e-mail and ip addresses' },

                      rebuild_policy    => { title=>'Rebuild All Policies',
                                      memo=>'Check and compile the settings for all group policies' },

                      settings_common   => { title=>'Common Settings',  memo=>'Common settings of all Anti-Spam components' },
                      settings_server   => { title=>'Process Server Settings',  memo=>'Settings of process server (ap-process-server)' },
                      settings_filter   => { title=>'Filtration Process Settings',  memo=>'Single/individual filtration process settings (ap-mailfilter)' },
                      settings_checks   => { title=>'Check Options',  memo=>'Settings of different check options' },
                      settings_clients  => { title=>'MTA Clients Settings',  memo=>'Common settings of MTA clients' },
                      settings_messages => { title=>'Reject Messages',  memo=>'Text of reject and bounce messages' },

                      settings_updates  => { title=>'Updater Settings',  memo=>'Updater schedule and settings' },
                      cc                => { title=>'Control Center Settings',  memo=>undef },

                      protected_domains => { title=>'Protected Domains',
                                      memo=>'List of domains to be protected by Anti-Spam engine' },
                    },
          static => {
                      product_name      => $product_name,
                      logo              => $product_name,
                      help              => 'Help',
                      ok                => 'Apply',
                      'reset'           => 'Reset',
                      default           => 'Default',
                      add               => 'Add',
                      details           => 'details',

                      status_ok         => 'OK',
                      status_errors     => "Errors...",
                      status_warnings   => "Warnings...",

                      status_proc_running  => 'processes',

                      log_no_events     => '&nbsp;No events',

                      stattype_messages => 'Messages',
                      stattype_bytes    => 'Bytes',
                      stattype_messages1=> 'messages',
                      stattype_bytes1   => 'bytes',
                      stat_export       => 'Export data',
                      stat_export_csv   => 'CSV',
                      stat_export_excel => 'Excel',
                      stat_export_html  => 'Html',
                      stat_export_htime => 'Created at',
                      stat_export_htz   => 'Timezone',
                      stat_export_time  => 'Date & Time',
                      stat_export_normal_m => $CONST->{detected_classes}{normal},
                      stat_export_normal_s => "$CONST->{detected_classes}{normal}, bytes",
                      stat_export_spam_m   => $CONST->{detected_classes}{spam},
                      stat_export_spam_s   => "$CONST->{detected_classes}{spam}, bytes",
                      stat_export_probable_m => $CONST->{detected_classes}{probable},
                      stat_export_probable_s => "$CONST->{detected_classes}{probable}, bytes",
                      stat_export_formal_m => $CONST->{detected_classes}{formal},
                      stat_export_formal_s => "$CONST->{detected_classes}{formal}, bytes",
                      stat_export_trust_m  => $CONST->{detected_classes}{trust},
                      stat_export_trust_s  => "$CONST->{detected_classes}{trust}, bytes",
                      stat_export_black_m  => $CONST->{detected_classes}{black},
                      stat_export_black_s  => "$CONST->{detected_classes}{black}, bytes",
                      stat_view         => 'View by',
                      stat_total        => 'Total',

                      copyright         => 'Copyright &copy;&nbsp;2002-2006&nbsp;<a href=http://www.kaspersky.com target=_blank>Kaspersky Lab</a><br>All rights reserved.',
                      copyright_text    => "Copyright (C) 2002-2006, Kaspersky Lab. All rights reserved.\nhttp://www.kaspersky.com",
                      rules             => 'Rules',
                      rules_modified    => 'Modified',
                      modified          => 'Modified',
                      modify            => 'Modify',
                      resetdef          => 'Reset to defaults',
                      remove            => 'Delete',
                      resetdef_confirm  => 'Do you want to reset "$group" to default settings?',
                      rebuild_policy    => 'Rebuild',
                      check_default     => '-- by default --',
                      check_on          => 'Enabled',
                      check_off         => 'Disabled',
                      listtype_email    => 'e-mails',
                      listtype_ip       => 'ip addresses',
                      group_default     => { name=>'All',  memo=>'All recipients not included in any other group' },
                      create_new_group  => 'New group',
                      removegroup_confirm => 'Do you want to delete group "$group"?',

                      lickeys_active    => 'Active',
                      lickeys_all       => 'All',
                      file              => 'File',
                      serial            => 'Serial',
                      type              => 'Type',
                      amount            => 'Volume',
                      valid             => 'Valid till',
                      more_info         => '&gt;&gt;',
                      dnsbl_hostname    => 'Hostname',
                      dnsbl_rate        => 'Rate',
                      removednsbl_confirm => 'Do you want to remove "$hostname" from DNS Blacklists?',
                      add_sign          => '+',

                      lic_expires_in    => '(expires in %d days)',
                    },
          control =>{
                      hostname              => 'Host Name',
                      osystem               => 'System',
                      status_loadaverage    => 'Load Average',

                      status_product        => 'Product',
                      status_filter_version => 'Version',
                      as_server             => 'Anti-Spam Engine',
                      as_updater            => 'Updates',
                      as_license            => 'License',

                      status_process_server => 'ap-process-server',
                      status_filter         => 'ap-mailfilter',
                      status_spfd           => 'ap-spfd',
                      status_thttpd         => 'kas-thttpd',
                      status_milter         => 'kas-milter',
                      status_monitoring     => 'Monitoring & Statistics',

                      status_updater        => 'Automatic Updates',
                      status_update_info    => 'Anti-Spam Database Id',
                      status_last_update    => 'Last Update',

                      status_license        => 'License',
                      status_licvalid       => 'Valid till',
                      status_licdaemon      => 'License Daemon',

                      log_filter  => {
                                        title   => 'View',
                                        info    => 'All messages',
                                        notice  => 'Notifications, Warnings and Errors',
                                        warning => 'Warnings and Errors',
                                        error   => 'Errors only',
                                     },

                      license_path=> 'License key file',

                      lists       => {
                                        allowed => {
                                                     email => 'Trusted e-mail addresses',
                                                     ip    => 'Trusted ip addresses',
                                                   },
                                        blocked => {
                                                     email => 'Blacklisted e-mail addresses',
                                                     ip    => 'Blacklisted ip addresses',
                                                   },
                                  },
                      check_rule  => {
                                    title => undef,
                                    options => [
                                                { val=>undef,  text=>'-- by default --' },
                                                { val=>'1', text=>'enable' },
                                                { val=>'0', text=>'disable' },
                                              ],
                                     },
                      group_name  => 'Group Name',
                      group_memo  => 'Description',
                      group_members => 'E-mail addresses of group members',
                      group_id => 'Group Id',

                      facility => {
                                    title => 'Syslog facility',
                                    options => [
                                                { val=>'mail',   text=>'mail' },
                                                { val=>'user',   text=>'user' },
                                                { val=>'local0', text=>'local0' },
                                                { val=>'local1', text=>'local1' },
                                                { val=>'local2', text=>'local2' },
                                                { val=>'local3', text=>'local3' },
                                                { val=>'local4', text=>'local4' },
                                                { val=>'local5', text=>'local5' },
                                                { val=>'local6', text=>'local6' },
                                                { val=>'local7', text=>'local7' }
                                              ],
                                  },
                      log_level => {
                                    title => 'Log level',
                                    options => [
                                                { val=>'0', text=>'minimum' },
                                                { val=>'1', text=>'low' },
                                                { val=>'2', text=>'normal' },
                                                { val=>'3', text=>'high' },
                                                { val=>'4', text=>'debug' },
                                                { val=>'5', text=>'more debug' }
                                              ],
                                  },

                      maxfilters   => 'Max. number of filtration processes',
                      startfilters => 'Number of filtration processes at server start-up',
                      sparefilters => 'Number of spare filtration processes',

                      maxmessages  => 'Max. number of mail messages to be processed',
                      randmsg      => 'Max. number of mail messages randomization',
                      maxidle      => 'Max. idle time (in&nbsp;seconds)',
                      delayedexit  => 'Exit delay (in&nbsp;seconds)',

                      received_headers_limit => 
                            'Number of \'Received\' headers to be parsed while retrieving ip address (for use in DNSBL checks)',
                      dnsbl_timeout          =>
                            'Overall timeout of all DNS requests (in&nbsp;seconds)',
                      extract_msoffice       =>
                            'Check MS Word and RTF files',
                      extract_rtf            =>
                            'Check RTF files',
                      uds_enabled            =>
                            'UDS enabled',
                      uds_timeout          =>
                            'Timeout for receiving response from UDS server (in&nbsp;seconds)',

                      client_filtering_size_limit => 'Filtering size limit&nbsp;(KB)',
                      client_domain               => 'Default domain',
                      client_connect_timeout=> 'Connection timeout (in&nbsp;seconds)',
                      client_rw_timeout     => 'Data exchange timeout (in&nbsp;seconds)',
                      client_onerror => {
                                          title => 'On filtering error',
                                          options => [
                                                      { val=>'accept', text=>'accept message' },
                                                      { val=>'reject', text=>'reject message' },
                                                      { val=>'tempfail', text=>'generate temporary error' },
                                                    ],
                                        },

                      reject_msg  => 'Reject message',
                      bounce_msg  => 'Bounce message',

                      schedule => {
                                    title => 'Run updater automatically',
                                    options => [
                                                { val=>'3',  text=>'every 3 minutes' },
                                                { val=>'10',  text=>'every 10 minutes' },
                                                { val=>'20',  text=>'every 20 minutes' },
                                                { val=>'30',  text=>'every 30 minutes' },
                                                { val=>'60',  text=>'every hour' },
                                                { val=>'120', text=>'every 2 hours' },
                                                { val=>'180', text=>'every 3 hours' },
                                                { val=>'0',   text=>'disabled (not recommended)' },
                                              ],
                                  },

                      connect_timeout => 'Network timeout (in&nbsp;seconds)',
                      passive_ftp => 'Use passive FTP mode',

                      updatesserver_settings  => 'Updates Server',
                      region                  => 'Region',
                      updateserverurl => 'Updates server URL',
                      use_updateserverurl => 'Use updates server URL',
                      use_updateserverurl_only => 'Use updates server URL only',

                      proxy_settings => 'Proxy Server',
                      proxy          => 'Proxy address',
                      proxy_user     => 'User',
                      proxy_pass     => 'Password',
                      use_proxy   => 'Use proxy',

                      updater_loglevel => {
                                    title => 'Updater log level',
                                    options => [
                                                { val=>'0', text=>'fatal' },
                                                { val=>'1', text=>'error' },
                                                { val=>'2', text=>'warning' },
                                                { val=>'3', text=>'info' },
                                                { val=>'4', text=>'activity' },
                                                { val=>'9', text=>'debug' }
                                              ],
                                  },
                      alerts_email=> 'Send alerts to',
                      cc_lang     => 'Language',
                      monitor_thttpd=> 'Monitoring of kas-thttpd daemon',
                      monitor_milter=> 'Monitoring of kas-milter daemon',

                      protected_domains => 'Protected domains',

                      use_kas_dnsbl_list => {
                                    title => 'Default list',
                                    options => [
                                                { val=>'yes', text=>'Enabled'},
                                                { val=>'no', text=>'Disabled'}
                                              ],
                                  },
                    },
          error =>  {
                      load_average_too_high   => 'Too high',
                      proc_not_running        => 'Process not found',
                      exec_not_found          => 'Executable file not found',

                      scripts_not_found       => 'Script(s) not found',
                      scripts_not_in_crontab  => 'Script(s) not in crontab',

                      mon_updater_not_found       => 'Updater script not found',
                      mon_updater_enabled         => 'Enabled',
                      mon_updater_not_in_crontab  => 'DISABLED',

                      base_too_old            => 'Anti-Spam database is too old',

                      licvalid_outdated       => 'EXPIRED',

                      no_information          => 'No information',

                      cant_open_licfile       => "Cannot save license file",
                      licfile_already_exists  => 'License key file $file already exists',
                      zero_size_upload        => 'Zero size file has been uploaded',
                      error_install_key       => 'Installing new key: Error',

                      no_keys_found           => 'No installed keys found',

                      cant_restart_licdaemon  => "Cannot restart license daemon",
                      cant_save_domains_list  => "Cannot save the list of protected domains",
                      cant_read_domains_list  => "Cannot read the list of protected domains",

                      not_valid_domains       => 'Invalid domain address(es)',

                      cant_save_config         => "Cannot save configuration",
                      cant_restart_server      => "Cannot restart Anti-Spam daemons",
                      cant_read_kasupdaterini  => "Cannot read keepup2date.conf",
                      cant_write_kasupdaterini => "Cannot write keepup2date.conf",

                      STSConfig_release_error => 'Error while calling STSConfig::release',
                      STSConfig_init_error    => 'Error while calling STSConfig::init',
                      STSConfig_init_warning  => 'Warnings while calling STSConfig::init',

                      cant_access_policy_data => 'Cannot access policy data',
                      warn_access_policy_data => 'Warnings while loading policy data',
                      cant_access_messages    => 'Cannot access settings.conf',
                      warn_access_messages    => 'Warnings while loading settings.conf',
                      cant_save_messages      => 'Cannot save settings.conf',
                      warn_save_messages      => 'Warnings while saving settings.conf',
                      wrong_group_id          => 'Invalid group identifier',
                      cant_build_profiles     => 'Cannot build policies',
                      warn_build_profiles     => 'Warnings while building policies',
                      validation_fault        => 'Incorrect value(s)',
                      verify_highlighted      => 'Verify highlighted field(s)',
                      cant_save_list          => 'Cannot save list',
                      warn_save_list          => 'Warnings while saving list',
                      cant_remove_group       => 'Cannot delete group',
                      warn_remove_group       => 'Warnings while deleting group',
                      cant_reorder_group      => 'Cannot reorder groups',
                      warn_reorder_group      => 'Warnings while reordering group',
                      cant_create_group       => 'Cannot create new group',
                      warn_create_group       => 'Warning while creating new group',
                      cant_update_group_info  => 'Cannot update group',
                      warn_update_group_info  => 'Warning while updating group',
                      not_defined_mandatory   => 'Group name and address list cannot be empty',
                      cant_save_policies      => 'Cannot save policy',
                      warn_save_policies      => 'Warning while saving policy',
                      crontab_get_fault       => "Cannot access crontab",
                      crontab_set_fault       => "Cannot modify crontab",
                      not_found               => 'Not found',
                      wrong_new_dnsbl_hostname=> "Invalid hostname",
                      exists_new_dnsbl_hostname=>"Repeated hostname",
                      not_valid_mime_header   => 'Non-ASCII symbols in header',
                      not_valid_ip            => 'Invalid IP address(es)',
                      alertto_is_empty        => 'Missed e-mail address',
                      sendto_is_empty         => 'Missed e-mail address(es)',
                      group_name_already_exist=> 'A group with the same name already exists',
                      group_descr_too_long    => 'Group description is too long (longer then $field symbols)',
                      invalid_value           => 'Invalid value of "$field"',
                      invalid_value_limits    => 'Possible values are from $min to $max.',
                      empty_proxy_pass        => 'Enter proxy password!',
                      invalid_value_proxy     => 'Enter proxy address or clear \'Use proxy\' setting!',
                      invalid_value_serverurl => 'Enter updates server URL or clear \'Use updates server URL\' and \'Use updates server URL only\' settings!',
                      invalid_value_keepup2date => 'Invalid value of \'$field\' in keepup2date.conf. You need to fix it manually.',
                      updater_not_in_crontab  => 'Warning: Automatic updates are disabled',
                    },

          scriptnames => {
                            dologs      => 'dologs.sh',
                            dograph     => 'dograph.sh',
                            logrotate   => 'logrotate.sh',
                            monitoring  => 'sfmonitoring',                    
                            udsrtts     => 'uds-rtts.sh',                    
                          },
          message=> {
                      keyfile_ok            => "New license key file installed successfully",
                      applied               => "Settings have been saved and applied successfully",
                      no_statistics         => 'Diagram is not built yet',
                      configapi_ready       => "STS Config API initiated successfully",
                      policies_compiled_ok  => "Profiles compiled successfully",
                    },
          actions=> {
                      name_prefix => 'If a message is recognized as',
                      actions =>{
                                  'accept'    => 'Accept this message',
                                  'copy'      => 'Send a copy of this message to other recipient(s)',
                                  'forward'   => 'Redirect this message to other recipient(s)',
                                  'reject'    => 'Reject this message',
                                  'blackhole' => 'Delete this message'
                                },
                      edit   => {
                                  sendto      => 'Send message to',
                                  subject_pre => 'Prepend to the Subject',
                                  subject_suf => 'Append to the Subject',
                                  category    => 'Append message Category',
                                  x_header    => 'Set X-SpamTest-Header'
                                },
                    },
          stsconfig => {
                      undef_err               => 'STSConfig API: Unknown error (%d)',
                      undef_warn              => 'STSConfig API: Unknown warning (%d)',
                      ok                      => 'STSConfig API: No errors',
                      action_config_notfound  => 'STSConfig API: Not found action-config[_xx].conf file',
                      action_config_notparsed => 'STSConfig API: Cannot parse action-config[-xx].conf file',
                      api_not_inited          => 'STSConfig API: API is not initiated',
                      cant_create_xml_file    => 'STSConfig API: Cannot create XML file',
                      cant_get_file_size      => 'STSConfig API: Cannot get file size',
                      cant_load_groups_file   => 'STSConfig API: Cannot load group file',
                      cant_run_compiler       => 'STSConfig API: Cannot run compiler',
                      cant_run_preprocessor   => 'STSConfig API: Cannot run preprocessor',
                      compiler                => 'STSConfig API: Compiler error',
                      convert_arguments       => 'STSConfig API: Cannot convert arguments',
                      create_tmp_file         => 'STSConfig API: Cannot create temporary file',
                      exec_kill_all_script    => 'STSConfig API: Cannot restart server',
                      file_not_open           => 'STSConfig API: Cannot open file',
                      memory_allocation       => 'STSConfig API: Memory allocation error',
                      preprocessor            => 'STSConfig API: Precompiler error',
                      rule_config_notfound    => 'STSConfig API: rule-config[_xx].conf file was not found',
                      rule_config_notparsed   => 'STSConfig API: Cannot parse rule-config[_xx].conf file',
                      wrong_argument          => 'STSConfig API: Invalid argument',
                      wrong_define_type       => 'STSConfig API: Invalid defined type',
                      wrong_group_id          => 'STSConfig API: Invalid group ID',
                      wrong_group_operation   => 'STSConfig API: Invalid group operation',
                      warn_cant_append_log_file => 'STSConfig API: Cannot append messages to log file',
                      warn_lock_file_exist_cant_compile => 'STSConfig API: Lock file already exist. Cannot run compiler.',
                    },
          monitoring => {
                      report_title            => 'Kaspersky Anti-Spam Monitoring Report',
                      alert_title             => 'Kaspersky Anti-Spam Monitoring Alert',
                      generate_time           => 'Created at',
                      stat_fmt_err_warn       => '%d error(s) and %d warning(s) detected',
                      stat_fmt_err            => '%d error(s) detected',
                      stat_fmt_warn           => '%d warning(s) detected',
                      stat_no_err_warn        => 'No errors detected',
                      error                   => 'ERROR',
                      warning                 => 'WARNING',
                      started_at              => 'This status was entered at ',
                      has_been_in_this_state  => 'Duration: ',
                      system_group            => 'System parameters',
                      as_group                => 'Anti-Spam engine',
                      asup_group              => 'Anti-Spam updates',
                      lic1_group               => 'License',
                      lic2_group               => 'License daemon',
                      system_group_length     => 0,
                      as_group_length         => 0,
                      asup_group_length       => 0,
                      lic1_group_length        => 0,
                      lic2_group_length        => 0,
                    },
        };

1;
__END__
