#!/usr/local/ap-mailfilter3/bin/perl -w
#######################################################################
# Default paths and constants

use strict;
use FindBin qw($Bin);

use vars qw($CONST $CONFIG $LANG $VERBOSE $QUIET @ERR);

BEGIN {
  require 'config.pl';

  if( $] >= 5.005 )
  {
#
# XXX ALK: Static perl linkage since 22.11.05
#
#    my $int64;
#    eval {
#      local $^W = 0;
#      $int64 = ( int(0xFFFFFFFF) ^ int(-1) )? '_int64': undef;
#    };
#
#    $int64 = '' unless $int64;
#    unshift @INC, ("$CONFIG->{CCLibPath}/blib_5.005$int64/arch", "$CONFIG->{CCLibPath}/blib_5.005$int64/lib");
#
  }
  else
  {
    die "Perl version too old: $]\n";
  }
};

use STSConfig;

my %STS_ERRORS = (
                    &STSConfig::STSCFG_OK                            => $LANG->{stsconfig}{ok},
                    &STSConfig::STSCFG_ERROR_ACTION_CONFIG_NOTFOUND  => $LANG->{stsconfig}{action_config_notfound},
                    &STSConfig::STSCFG_ERROR_ACTION_CONFIG_NOTPARSED => $LANG->{stsconfig}{action_config_notparsed},
                    &STSConfig::STSCFG_ERROR_API_NOT_INITED          => $LANG->{stsconfig}{api_not_inited},
                    &STSConfig::STSCFG_ERROR_CANT_CREATE_XML_FILE    => $LANG->{stsconfig}{cant_create_xml_file},
                    &STSConfig::STSCFG_ERROR_CANT_GET_FILE_SIZE      => $LANG->{stsconfig}{cant_get_file_size},
                    &STSConfig::STSCFG_ERROR_CANT_LOAD_GROUPS_FILE   => $LANG->{stsconfig}{cant_load_groups_file},
                    &STSConfig::STSCFG_ERROR_CANT_RUN_COMPILER       => $LANG->{stsconfig}{cant_run_compiler},
                    &STSConfig::STSCFG_ERROR_CANT_RUN_PREPROCESSOR   => $LANG->{stsconfig}{cant_run_preprocessor},
                    &STSConfig::STSCFG_ERROR_COMPILER                => $LANG->{stsconfig}{compiler},
                    &STSConfig::STSCFG_ERROR_PREPROCESSOR            => $LANG->{stsconfig}{preprocessor},
                    &STSConfig::STSCFG_ERROR_CONVERT_ARGUMENTS       => $LANG->{stsconfig}{convert_arguments},
                    &STSConfig::STSCFG_ERROR_CREATE_TMP_FILE         => $LANG->{stsconfig}{create_tmp_file},
                    &STSConfig::STSCFG_ERROR_EXEC_KILL_ALL_SCRIPT    => $LANG->{stsconfig}{exec_kill_all_script},
                    &STSConfig::STSCFG_ERROR_FILE_NOT_OPEN           => $LANG->{stsconfig}{file_not_open},
                    &STSConfig::STSCFG_ERROR_MEMORY_ALLOCATION       => $LANG->{stsconfig}{memory_allocation},
                    &STSConfig::STSCFG_ERROR_RULE_CONFIG_NOTFOUND    => $LANG->{stsconfig}{rule_config_notfound},
                    &STSConfig::STSCFG_ERROR_RULE_CONFIG_NOTPARSED   => $LANG->{stsconfig}{rule_config_notparsed},
                    &STSConfig::STSCFG_ERROR_WRONG_ARGUMENT          => $LANG->{stsconfig}{wrong_argument},
                    &STSConfig::STSCFG_ERROR_WRONG_DEFINE_TYPE       => $LANG->{stsconfig}{wrong_define_type},
                    &STSConfig::STSCFG_ERROR_WRONG_GROUP_ID          => $LANG->{stsconfig}{wrong_group_id},
                    &STSConfig::STSCFG_ERROR_WRONG_GROUP_OPERATION   => $LANG->{stsconfig}{wrong_group_operation},
                    &STSConfig::STSCFG_WARN_CANT_APPEND_LOG_FILE     => $LANG->{stsconfig}{warn_cant_append_log_file},
                    &STSConfig::STSCFG_WARN_LOCK_FILE_EXIST_CANT_COMPILE => $LANG->{stsconfig}{warn_lock_file_exist_cant_compile},
                 );

my $initdata = {
                  DefPath            => $CONFIG->{FilterDefConfigPath},
                  XMLPath            => $CONFIG->{FilterSrcConfigPath},
                  BinDataPath        => $CONFIG->{FilterDataConfigPath},
                  CFDataMainsetPath  => $CONFIG->{FilterCFDataMainsetPath},
                  BinPath            => $CONFIG->{BinRootPath},
                  FilterConfPath     => $CONFIG->{ConfigPath},
                  TempPath           => $CONFIG->{FilterTmpConfigPath},
                  Lang               => $CONFIG->{Lang},
                  MessagesFilePath   => $CONFIG->{CompilerLog},
                  RestartScriptPath  => $CONFIG->{RestartScript},
                  PrecompilerPath    => $CONFIG->{PrecompilerPath},
                  CompilerPath       => $CONFIG->{CompilerPath},
                  CompileIncremental => 1,
                  LockFilePath       => $CONFIG->{MkProfilesPIDFile},
               };

my $IsReady;

$IsReady = 1 if( STSConfig::init($initdata) );

{
  my $err = STSConfig::get_last_error();
  if( $err != &STSConfig::STSCFG_OK )
  {
    my $err = STSCFG_GetErrorMessage($err);
    my $msg = $LANG->{error}{ ($IsReady)? 'STSConfig_init_warning' :'STSConfig_init_error' };
    push @ERR, $msg;
    push @ERR, $err;
    warn "$msg\n";
    warn "$err\n";
  }
}

if( $IsReady )
{
  warn "Init STSConfig API\n" if($VERBOSE);
  if( open(LOG,'>'.$CONFIG->{CompilerLog} ) )
  {
#    print LOG "Open log at " . scalar(localtime) . "\n\n";
    close LOG;
  }
}

END {
  if( $IsReady )
  {
    STSConfig::release();

    my $err = STSConfig::get_last_error();
    if( $err != &STSConfig::STSCFG_OK )
    {
      my $err = STSCFG_GetErrorMessage();
      my $msg = $LANG->{error}{STSConfig_init_error};
      push @ERR, $msg;
      push @ERR, $err;
      warn "$msg\n";
      warn "$err\n";
    }

    unlink $CONFIG->{CompilerLog} if -e $CONFIG->{CompilerLog};
    warn "Release STSConfig API\n" if($VERBOSE);
  }
};

###############################################################################

sub STSCFG_IsReady
{
  return $IsReady;
}


sub STSCFG_GetErrorMessage(;$)
{
  my ($err) = @_;
  $err = STSConfig::get_last_error() unless defined $err;
  my $res = (defined $STS_ERRORS{$err})? $STS_ERRORS{$err} : sprintf( $LANG->{stsconfig}{($err<&STSConfig::STSCFG_OK)? 'undef_err':'undef_warn'}, $err );
  return $res;
}


sub STSCFG_Succeeded($$$;$)
{
  my($errors,$call,$err_msg,$warn_msg) = @_;

  my $err = STSConfig::get_last_error();
  if( $err != &STSConfig::STSCFG_OK )
  {
    if( $err==&STSConfig::STSCFG_ERROR_CANT_RUN_COMPILER
     || $err==&STSConfig::STSCFG_ERROR_CANT_RUN_PREPROCESSOR
     || $err==&STSConfig::STSCFG_ERROR_COMPILER
     || $err==&STSConfig::STSCFG_ERROR_PREPROCESSOR )
    {
      $err_msg = $LANG->{error}{cant_build_profiles};
      $warn_msg = $LANG->{error}{warn_build_profiles};
    }

    push @ERR, STSCFG_GetErrorMessage($err) . (defined($call)? " - $call" : '');
    if( STSConfig::HasErrors($err) )
    {
      $errors->{error} = $err_msg;
      return 0;
    }
    else
    {
      $errors->{warning} = $warn_msg if defined $warn_msg;
    }
  }

  return 1;
}


sub STSCFG_PushErrorLog()
{
  if( -r $CONFIG->{CompilerLog} && -s _ && open(LOG,$CONFIG->{CompilerLog}) )
  {
    if( open(LOG,'<'.$CONFIG->{CompilerLog} ) )
    {
      local $/;
      $/ = undef;
      my $log = <LOG>;
      close LOG;

      if( $QUIET || !$VERBOSE )
      {
        $log =~ s/^\[[^\]]+\]\s+Run\s+\(command\s+line\)\:.*?2\>\&1\s*$//gm;
        $log =~ s/^\s*$//gm;
      }

      push @ERR, $log if( defined($log) && $log ne '' );
    }
  }
}


sub STSCFG_ErrorLogHelper($)
{
  my ($error) = @_;
  STSCFG_PushErrorLog() if( defined($error->{error}) || defined($error->{warning}) );
}


sub STSCFG_ValuesHash($)
{
  my ($arr) = @_;

  my %hash;
  foreach (@{$arr}) { $hash{$_->{ID}} = $_->{Value}; }
  return \%hash;
}


sub STSCFG_JoinValuesDefaults($$)
{
  my ($valules, $defines) = @_;

  my ($key, $val);
  while( ($key, $val) = each(%{$defines}) )
  {
    $valules->{$key} = $val unless exists $valules->{$key};
  }
}


sub STSCFG_ValidateRules(%)
{
  my (%param) = @_;
  my ($group, $values, $defaults) = ($param{rules_group}, $param{value}, $param{with_defaults});

  return 1 unless(defined($group) && defined($group->{Control}) && defined($values));

  foreach my $ctl (@{$group->{Control}})
  {
    next unless defined($ctl) && defined($ctl->{Type});
    next unless( $ctl->{Type}==&STSConfig::STSCFG_EDIT || $ctl->{Type}==&STSConfig::STSCFG_NUMBER );
    next unless defined($ctl->{ID});

    if( $ctl->{Type}==&STSConfig::STSCFG_EDIT )
    {
      # Edit
      next unless defined($ctl->{Edit});

      my $value = (defined $values->{$ctl->{ID}})? $values->{$ctl->{ID}}: '';
      next if( $defaults && (!defined($value) || $value eq '') );

      my $edit = $ctl->{Edit};
      my $error = 0;

      if( defined($edit->{ValidateRegex}) && ($edit->{ValidateRegex} !~ /^\s*$/ ) )
      {
        my $rx = $edit->{ValidateRegex};
        eval{ $error = ( $value !~ /$rx/i ); };
      }

      return 0 if $error;
    }
    else
    {
      # Number
      next unless defined($ctl->{Number});

      my $edit = $ctl->{Number};
      $edit->{Min} = -1e19 unless defined $edit->{Min};
      $edit->{Max} = 1e19 unless defined $edit->{Max};
      ($edit->{Min},$edit->{Max}) = ($edit->{Max},$edit->{Min}) if( $edit->{Min} > $edit->{Max} );

      my $value;
      if( $defaults )
      {
        $value = $values->{$ctl->{ID}};
        next unless( defined($value) && $value ne '' );
        $value = int($value);
      }
      else
      {
        $value = (defined $values->{$ctl->{ID}})? int($values->{$ctl->{ID}}): ($edit->{Min}+$edit->{Max})/2;
      }

      return 0 if $value < $edit->{Min} || $value > $edit->{Max};
    }
  }

  return 1;
}


sub STSCFG_PrepareCommonRulesGroupInfo($$$)
{
  my ($group,$basedef,$defines) = @_;

  my @controls = grep {defined($_->{ID}) && $_->{ID} ne '' } @{$group->{Control}};
  $group->{RulesCount} = scalar(@controls);
  warn "Group '$group->{Name}' contains no rules!" unless $group->{RulesCount};

  $group->{RulesModified} = 0;
  foreach my $ctl (@controls)
  {
    next unless defined $defines->{$ctl->{ID}};
    $group->{RulesModified}++ unless( defined($basedef->{$ctl->{ID}}) && $defines->{$ctl->{ID}} eq $basedef->{$ctl->{ID}} );
  }
}


sub STSCFG_PrepareGroupRulesGroupInfo($$)
{
  my ($group,$defines) = @_;

  my @controls = grep {defined($_->{ID}) && $_->{ID} ne '' } @{$group->{Control}};
  $group->{RulesCount} = scalar(@controls);
  warn "Group '$group->{Name}' contains no rules!" unless $group->{RulesCount};

  $group->{RulesModified} = 0;
  foreach my $ctl (@controls)
  {
    $group->{RulesModified}++ if defined $defines->{$ctl->{ID}};
  }
}


sub STSCFG_ValuesList($$)
{
  my ($basedef,$defines) = @_;
  my @list;

  my ($key, $val);
  while( ($key, $val) = each(%{$defines}) )
  {
    next unless defined($val);
    next if( (defined($basedef->{$key}) && $basedef->{$key} eq $val) );
    push @list, { ID=>$key, Value=>$val };
  }

  return \@list;
}


sub STSCFG_PackList($)
{
  my ($list) = @_;

  my @list;
  foreach (@{$list})
  {
    push @list, $_->{Entry} if( defined $_->{Entry} && $_->{Entry} !~ /^\s*$/ );
  }
  return \@list;
}


sub STSCFG_UnpackList($)
{
  my ($list) = @_;
  my @list;
  if( defined $list )
  {
    foreach (@{$list}) { push @list, { Entry => $_ }; }
  }

  return \@list;
}

sub STSCFG_FilterRBLList($)
{
  my ($list) = @_;
  my @list;
  if( defined $list )
  {
    my %list;
    foreach (@{$list})
    {
      next unless( defined($_->{Entry}) );

      $_->{Entry} =~ s/[^-a-z0-9\.]+//ig;
      next if( $_->{Entry} eq '' );

      my $rate = int($_->{Rate});
      if( $rate < 1 ) { $rate = 1 }
      elsif( $rate > 100 ) { $rate = 100 }

      $list{lc($_->{Entry})} += $rate;
    }

    while( my ($key,$val) = each(%list) )
    {
      push @list, { Entry=>$key, Rate=>$val };
    }
  }

  return sort { _by_domain($a->{Entry},$b->{Entry}) } @list;
}

sub STSCFG_get_local_rbl_workaround($)
{
  my ($file_name) = @_;
  my @list;
  my $xml;

  open(FILE, $file_name) || return @list;

  while(<FILE>){
        $xml.=$_;
  }

  while ($xml =~ s/<\/?\s*([^<>]+)\s*\/?\s*>//mio)
  {
        my $tag = $1;
        my $zone = undef;

        while ($tag =~ s/\s*([a-z]+)\s*=\s*(["'])([a-z0-9-\.]+)\2\s*//mio)
        {
                $zone->{'Entry'} = lc($3) if lc($1) eq 'zone';
                $zone->{'Rate'} = lc($3) if lc($1) eq 'weight';
        }

        push @list, $zone if $zone;
  }

  close(FILE);

  return @list;
}

sub STSCFG_CheckGroupList($)
{
  my ($list) = @_;

  push @{$list}, { ID=>0 } unless( scalar(@{$list}) && ${$list}[-1]{ID}==0 );
  ${$list}[-1]{Name} = $LANG->{static}{group_default}{name};
  ${$list}[-1]{Memo} = $LANG->{static}{group_default}{memo};

  foreach (@{$list})
  {
    $_->{Name} = '?' unless( defined($_->{Name}) && $_->{Name} !~ /^\s*$/ );
  }
}

sub STSCFG_VerifyGroupID($$)
{
  my ($list,$gid) = @_;
  my @g = grep { $_->{ID} == $gid } @{$list};
  return scalar(@g);
}

sub STSCFG_GetGroupByID($$)
{
  my ($list,$gid) = @_;
  my @g = grep { $_->{ID} == $gid } @{$list};
  return shift @g;
}


sub STSCFG_DeleteGroupStat($)
{
  my ($gid) = @_;
  return unless $gid;

  my $fid = sprintf( '%08X', $gid );
  my @files = (
                "$CONFIG->{CCStatPath}/$fid",
                "$CONFIG->{CCStatPath}/$fid.old",
                "$CONFIG->{CCWwwStatPath}/d$fid\_1mM.png",
                "$CONFIG->{CCWwwStatPath}/d$fid\_1mS.png",
                "$CONFIG->{CCWwwStatPath}/d$fid\_1yM.png",
                "$CONFIG->{CCWwwStatPath}/d$fid\_1yS.png",
                "$CONFIG->{CCWwwStatPath}/d$fid\_24hM.png",
                "$CONFIG->{CCWwwStatPath}/d$fid\_24hS.png",
                "$CONFIG->{CCWwwStatPath}/d$fid\_7dM.png",
                "$CONFIG->{CCWwwStatPath}/d$fid\_7dS.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_1mM.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_1mS.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_1yM.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_1yS.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_24hM.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_24hS.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_7dM.png",
                "$CONFIG->{CCWwwStatPath}/g$fid\_7dS.png"
              );

  foreach (@files)
  {
    unlink($_);
  }
}


1;
__END__

################################
# DEBUG

print Dumper(\%STS_ERRORS), "\n";
print Dumper($initdata), "\n";
print ($IsReady)? "Ready\n": "Not ready\n";
