#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

#use FindBin qw($Bin);
use CGI qw(:standard -private_tempfiles);
$CGI::POST_MAX=1024 * 100; # 100k max post
#use POSIX qw(strftime);
use Data::Dumper;


use vars qw($CONST $CONFIG $LANG $MENU $PARAM @ERR);

BEGIN {
  require 'root.pl';
  require 'const.pl';
  require 'config.pl';
};

sub FullHostname ()
{
        my(@tmp,$hostname1);
        if (open(P1,"hostname -f 2>&1 |") && (@tmp = <P1>) && close(P1)){
                chop($hostname1 = $tmp[$#tmp]);
        }
        elsif (open(P1,"hostname 2>&1 |") && (@tmp = <P1>) && close(P1)){
                chop($hostname1 = $tmp[$#tmp]);
        }
        elsif (open(P1,"uname -n 2>&1 |") && (@tmp = <P1>) && close(P1)){
                chop($hostname1 = $tmp[$#tmp]);
        }else{
                die "$0: Cannot get hostname from 'hostname' or 'uname -n'\n";
        }
        @tmp = ();
        close P1; # Just in case we failed in an odd spot....
        $hostname1;
}


sub GetParams()
{
  my $query = new CGI;

  my %param;
  foreach ($query->param)
  {
    unless( /^uploaded_file$/ )
    {
      $param{$_} = $query->param($_);
    }
    else
    {
      my $filename = $query->param($_);
      next unless( defined($filename) && $filename !~ /^\s*$/ );

      local $/ = undef;
      $param{uploaded_file} = <$filename>;
      my $disposition = $query->uploadInfo($filename)->{'Content-Disposition'};
      if( $disposition =~ /filename\s*\=\s*(("([^"]+)")|('([^']+)')|(\S+))/i )
      {
        $param{uploaded_filename} = $3 || $5 || $6;
        my @items = grep { defined($_) && $_ ne '' } split /[\\\/:]/, $param{uploaded_filename};
        $param{uploaded_filename} = $items[-1] if @items;
        $param{uploaded_filename} =~ s#[^a-z0-9_.,@-]##ig;
      }
    }
  }

  $param{this} = $ENV{REQUEST_URI};
  $param{this} =~ s/\#.*// if defined $param{this} && $param{this} ne '';
  $param{this} =~ s/\?.*// if defined $param{this} && $param{this} ne '';
  $PARAM = \%param;

  return %param;
}

sub InstallKey($)
{
  my ($key_file) = @_;

  push @ERR, `$CONFIG->{InstallKey} -V 3 -i -x $key_file 2>&1`;
  my $ret = $?>>8;
  return !$ret;
}

sub RestartServer(;%)
{
  my (%param) = @_;
  $param{filter} = 1 unless scalar(@_);

  my $RestartParam = '';
  $RestartParam .= ' -f' if( $param{filter} );
  $RestartParam .= ' -p' if( $param{process_server} );
  $RestartParam .= ' -s' if( $param{license} );

  push @ERR, `$CONFIG->{RestartScript} -V 3 $RestartParam 2>&1`;
  my $ret = $?>>8;
  ##push @ERR, "$CONFIG->{RestartScript} $RestartParam exit with code: $ret";
  return !$ret;
}

sub RestartHelper($)
{
  my ($error) = @_;

  unless( defined $error->{error} )
  {
    $error->{warning} = $LANG->{error}{cant_restart_server} unless( RestartServer() );
    return undef;
  }
  else
  {
    $error->{notice} = $LANG->{message}{applied} unless defined $error->{notice};
  }

  return 1;
}

sub by_ips
{
  my ($ip_a, $ip_b) = ($a, $b);
  $ip_a = sprintf( '%03d.%03d.%03d.%03d/%03d', $1, $2, $3, $4, ((defined $6)? $6: 255) ) if( $ip_a =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)(\/(\d+))?$/ );
  $ip_b = sprintf( '%03d.%03d.%03d.%03d/%03d', $1, $2, $3, $4, ((defined $6)? $6: 255) ) if( $ip_b =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)(\/(\d+))?$/ );

  return $ip_a cmp $ip_b;
}


sub _by_domain($$)
{
  my ($domain_a,$domain_b) = @_;

  $domain_a =~ s/^@/@\./;
  $domain_b =~ s/^@/@\./;

  my $cmp_a = join( '.', reverse( split(/\./,$domain_a) ) );
  my $cmp_b = join( '.', reverse( split(/\./,$domain_b) ) );

  return $cmp_a cmp $cmp_b;
}

sub by_domains
{
  return _by_domain($a,$b);
}

sub by_emails
{
  my ($email_a, $domain_a) = ($a =~ /^([^@]*)(@?.*)$/);
  my ($email_b, $domain_b) = ($b =~ /^([^@]*)(@?.*)$/);

  unless( defined($domain_a) && $domain_a ne '' )
  {
    $domain_a = $email_a;
    $email_a = undef;
  }

  unless( defined $domain_b && $domain_b ne '' )
  {
    $domain_b = $email_b;
    $email_b = undef;
  }

  if( $domain_a eq $domain_b )
  {
    return $email_a cmp $email_b;
  }
  else
  {
    return _by_domain($domain_a,$domain_b);
  }
}

sub load_stat($$$)
{
  my ($data_path,$step,$count) = @_;

  unless( -r $data_path  )
  {
      warn "Not found data file $data_path\n";
      return undef;
  }

  unless( open( DATA, "<$data_path" ) )
  {
      warn "Can't open file $data_path\n";
      return undef;
  }

  my @data;
  my $line;
  my $i = 0;
  my $current_time = time();
  my $end_time = int( ($current_time+$step-1)/$step ) * $step - $step;
  my $time = $end_time;
  my $item = { 'time'=>$time, normal_m=>0, normal_s=>0, spam_m=>0, spam_s=>0, probable_m=>0, probable_s=>0, formal_m=>0, formal_s=>0, trust_m=>0, trust_s=>0, black_m=>0, black_s=>0 };

  while( $i<$count && defined( $line=<DATA> ) )
  {
    chomp($line);
    my ($ltime,$spam_m,$spam_s,$probable_m,$probable_s,$formal_m,$formal_s,$normal_m,$normal_s,$black_m,$black_s,$trust_m,$trust_s) = ( $line =~ /(\d+)/gi );

    while( $i<$count && $time>$ltime )
    {
      # Save result cell and shift frame
      push @data, $item;
      $time -= $step; $i++;
      $item = { 'time'=>$time, normal_m=>0, normal_s=>0, spam_m=>0, spam_s=>0, probable_m=>0, probable_s=>0, formal_m=>0, formal_s=>0, trust_m=>0, trust_s=>0, black_m=>0, black_s=>0 };
    }

    if( $i<$count )
    {
      # Current cell
      $item->{normal_m}   += $normal_m;
      $item->{normal_s}   += $normal_s;
      $item->{spam_m}     += $spam_m;
      $item->{spam_s}     += $spam_s;
      $item->{probable_m} += $probable_m;
      $item->{probable_s} += $probable_s;
      $item->{formal_m}   += $formal_m;
      $item->{formal_s}   += $formal_s;
      $item->{trust_m}    += $trust_m;
      $item->{trust_s}    += $trust_s;
      $item->{black_m}    += $black_m;
      $item->{black_s}    += $black_s;
    }
  }

  close(DATA);
  push @data, $item if( $i < $count );

  return \@data;
}

sub HumanValue($;$$)
{
  my ($value,$space,$base) = @_;

  if($base == 1000)
  {
    if($value >= 1000000)
    {
      return sprintf('%u,%03u,%03u', int($value/1000000), int(($value%1000000)/1000), $value%1000);
    }
    elsif($value >= 1000)
    {
      return sprintf('%u,%03u', int($value/1000), $value%1000);
    }
    else
    {
      return sprintf('%u', $value);
    }
  }
  else
  {
    $base = 1024;
    my $k = 0;
    while($value >= $base && $k < 3)
    {
      $value /= $base;
      $k++;
    }
    my @exts;
    @exts = ("", " Kb", " Mb", "Gb");

    return sprintf('%.2f',$value).$exts[$k]  if( $k && $value < 10 );
    return sprintf('%.1f',$value).$exts[$k]  if( $k && $value < 100 );
    return sprintf('%.0f',$value).$exts[$k];
  }

  # old variant

  $base = 1024 unless defined $base;
  $space = ' ' unless defined $space;

  return $value if( $value < 100*$base );

  my $ext;
  foreach $ext ('', $space.'Kb', $space.'Mb', $space.'Gb')
  {
    if( $value < $base )
    {
      return sprintf('%.2f',$value).$ext  if( $value < 10 );
      return sprintf('%.1f',$value).$ext  if( $value < 100 );
      return sprintf('%.0f',$value).$ext;
    }
    $value /= $base;
  }

  $ext = $space.'T';
  return sprintf('%.2f',$value).$ext  if( $value < 10 );
  return sprintf('%.1f',$value).$ext  if( $value < 100 );
  return sprintf('%.0f',$value).$ext;
}

sub SeparateValue($;$$)
{
  my ($v,$space,$count) = @_;
  $count = 3   unless defined $count;
  $space = ' ' unless defined $space;

  $space = reverse $space;

  $v =~ s/\s//g;
  $v = reverse $v;
  $v =~ s/(\d{$count})(?=\d)(?!\d\.)/$1$space/g;
  return scalar reverse $v;
}


sub CalcTZOffset(;$)
{
  my ($time) = @_;
  $time = time() unless( $time && $time>0 );

  my (@l) = localtime($time);
  my (@g) = gmtime($time);

  my $off = $l[0] - $g[0] + ($l[1] - $g[1]) * 60 + ($l[2] - $g[2]) * 3600;

  if ($l[7] == $g[7]) {
    # done
  } elsif ($l[7] == $g[7] + 1) {
    $off += 86400;
  } elsif ($l[7] == $g[7] - 1) {
    $off -= 86400;
  } elsif ($l[7] < $g[7]) {
    #    !
    $off += 86400;
  } else {
    $off -= 86400;
  }

  return $off;
}


sub IsMailfltUser()
{
  return ($CONST->{user}{mailflt_uid} == $<);
}


sub IsRootUser()
{
  return ($CONST->{user}{root_uid} == $<);
}


my %CRONTAB_SERVICES = (
                          updater    => $CONFIG->{UpdaterScriptPath},
                          dologs     => $CONFIG->{DoLogsPath},
                          dograph    => $CONFIG->{DoGraphPath},
                          logrotate  => $CONFIG->{LogRotatePath},
                          monitoring => $CONFIG->{MonitoringScriptPath},
                          udsrtts    => $CONFIG->{UdsRttsPath},
                       );

my %CRONTAB_CHECKS = reverse %CRONTAB_SERVICES;
my $CRONTAB_RX;
{
  my @keys = keys(%CRONTAB_CHECKS);
  foreach (@keys) { $_ = quotemeta; }
  $CRONTAB_RX = '(' . join( ')|(', @keys ) . ')';
}

sub ReadCrontab()
{
  my %res;
  my @crontab = grep { chomp; $_ !~ /(^\s*\#)|(^\s*$)/ } `$CONST->{bin}{crontab_get}`;
  my $ret = $?>>8;
  if( $ret )
  {
#    push @ERR, "\"$CONST->{bin}{crontab_get}\" return $ret";
    return \%res;
  }

  foreach (@crontab)
  {
    if( /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*?)\s*$/ )
    {
      my ($min,$hour,$day,$month,$weekday,$cmd) = ($1,$2,$3,$4,$5,$6);
      if( $cmd =~ /^($CRONTAB_RX)(\s|$)/o )
      {
        my $command = $CRONTAB_CHECKS{$1};
        if( $command && $command eq 'updater' )
        {
          if( $day ne '*' || $month ne '*' || $weekday ne '*' )
          {
            $res{updater} = 180;
          }
          elsif( $hour eq '*' )
          {
            # 3, 10, 20, 30, 60 min
            if( $min =~ /^\*\/3$/ )
            {
              $res{updater} = 3;
            }
            elsif( $min =~ /^\d+$/ )
            {
              $res{updater} = 60;
            }
            elsif( $min =~ /^((\d+,\d+)|(\*\/30))$/ )
            {
              $res{updater} = 30;
            }
            elsif( $min =~ /^((\d+,\d+,\d+)|(\*\/20))$/ )
            {
              $res{updater} = 20;
            }
            else
            {
              $res{updater} = 10;
            }
          }
          else
          {
            # 2,3 hour
            $res{updater} = ( $hour =~ /^((\*\/3)|(\d+(,\d+){7}))$/ )? 180: 120;
          }
        }
        else
        {
          $res{$command} = 1 if $command;
        }
      }
    }
    elsif( /^\s*MAILTO\s*\=\s*((\S+)|(\"[^\"]+\"))$/ )
    {
      my $mailto = $1;
      $mailto =~ s/^\"([^\"]+)\"$/$1/;
      $res{MAILTO} = $mailto;
    }
  }

  return \%res;
}


sub WriteCrontab(%)
{
  my (%param) = @_;

  if( $CONFIG->{CCDontModifyCrontab} )
  {
    warn "Crontab not modified by user settings in $CONFIG->{ConfigPath}\n";
    return 1;
  }

  my @crontab = `$CONST->{bin}{crontab_get}`;
  my $ret = $?>>8;
  if( $ret )
  {
#    push @ERR, "\"$CONST->{bin}{crontab_get}\" return $ret";
#    return undef;
    @crontab = ();
  }
  foreach (@crontab) { chomp; }

  my @newcrontab;
  if( $param{MAILTO} )
  {
    my $addr = $param{MAILTO};
    $addr =~ s/[\"\';,<>]/ /g;
    $addr =~ s/^\s+//;
    $addr =~ s/\s+$//;
    $addr =~ s/\s+/ /g;

    $addr = "\"$addr\"" if( $addr =~ /\s/ );

    push @newcrontab, "MAILTO=$addr";
  }

  my $nopath = 1;
  my $nohome = 1;
  foreach (@crontab)
  {
    next if( /(^\s*\#)|(^\s*$)/ );
    $nopath = undef if( /^\s*PATH\s*\=/ );
    $nohome = undef if( /^\s*HOME\s*\=/ );
  }

  push @newcrontab, 'PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin' if( $nopath );
  push @newcrontab, "HOME=$CONFIG->{RunRootPath}" if( $nohome );

  foreach my $line (@crontab)
  {
    if( $line =~ /(^\s*\#)|(^\s*$)/ )
    {
      push @newcrontab, $line;
    }
    elsif( exists($param{MAILTO}) && $line =~ /^\s*MAILTO\s*\=/ )
    {
      next;
    }
    elsif( $line =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*?)\s*$/ )
    {
      my ($min,$hour,$day,$month,$weekday,$cmd) = ($1,$2,$3,$4,$5,$6);
      if( $cmd =~ /^($CRONTAB_RX)(\s|$)/o )
      {
        my $command = $CRONTAB_CHECKS{$1};
        push @newcrontab, $line unless( $command && exists($param{$command}) );
      }
      else
      {
        push @newcrontab, $line;
      }
    }
    else
    {
      push @newcrontab, $line;
    }
  }

  if( $param{updater} )
  {
    my $period = $param{updater};
    my $line;

    if( $period < 90 )
    {
      if( $period < 5 )
      {
        # 3 min
        $line = sprintf("*/3 * * * *");
      }
      elsif( $period < 15 )
      {
        # 10 min
        my $offset = int(rand()*10);
        $line = sprintf("%-d,%-d,%-d,%-d,%-d,%-d * * * *", $offset, $offset+10, $offset+20, $offset+30, $offset+40, $offset+50 );
      }
      elsif( $period < 25 )
      {
        # 20 min
        my $offset = int(rand()*20);
        $line = sprintf("%-d,%-d,%-d * * * *", $offset, $offset+20, $offset+40 );
      }
      elsif( $period < 45 )
      {
        # 30 min
        my $offset = int(rand()*30);
        $line = sprintf("%-d,%-d * * * *", $offset, $offset+30 );
      }
      else
      {
        # 60 min
        my $offset = int(rand()*60);
        $line = sprintf("%-d * * * *", $offset );
      }
    }
    else
    {
      if( $period < 150 )
      {
        # 2 hour
        my $offset = int(rand()*120);
        my $offset_h = int($offset/60);
        my $offset_m = $offset - $offset_h*60;
        $line = sprintf("%-d %-d,%-d,%-d,%-d,%-d,%-d,%-d,%-d,%-d,%-d,%-d,%-d * * *", $offset_m, $offset_h, $offset_h+2, $offset_h+4, $offset_h+6, $offset_h+8, $offset_h+10, $offset_h+12, $offset_h+14, $offset_h+16, $offset_h+18, $offset_h+20, $offset_h+22 );
      }
      else
      {
        # 3 hour
        my $offset = int(rand()*180);
        my $offset_h = int($offset/60);
        my $offset_m = $offset - $offset_h*60;
        $line = sprintf("%-d %-d,%-d,%-d,%-d,%-d,%-d,%-d,%-d * * *", $offset_m, $offset_h, $offset_h+3, $offset_h+6, $offset_h+9, $offset_h+12, $offset_h+15, $offset_h+18, $offset_h+21 );
      }
    }

    if( $period < 5 )
    {
        $line .= " $CRONTAB_SERVICES{updater} -q -M";
    }
    else
    {
        $line .= " $CRONTAB_SERVICES{updater} -q";
    }
    push @newcrontab, $line;
  }

  if( $param{udsrtts} )
  {
    my $offset = int(rand()*5);
    my $line = sprintf("%-d,%-d,%-d,%-d,%-d,%-d * * * *", $offset, $offset+11, $offset+22, $offset+33, $offset+44, $offset+55);
    $line .= " $CRONTAB_SERVICES{udsrtts} -q";
    push @newcrontab, $line;
  }

  foreach (qw(monitoring dologs dograph logrotate))
  {
    push @newcrontab, "$CONST->{crontab}{$_} $CRONTAB_SERVICES{$_} -q" if( $param{$_} );
  }

  unless( open( CRONTAB, "|$CONST->{bin}{crontab_set}" ) )
  {
    push @ERR, "Can't execute $CONST->{bin}{crontab_set}: $!";
    return undef;
  }
  print CRONTAB join("\n",@newcrontab),"\n";
  close CRONTAB;
  return 1;
}


sub GetCrontabScriptInfo()
{
  my $crontab = ReadCrontab();

  my ($key,$path);
  my %res;
  while( ($key,$path) = each(%CRONTAB_SERVICES) )
  {
    if( -x $path )
    {
      $res{$key} = ($crontab->{$key})? 1: 0;
    }
    else
    {
      $res{$key} = -1;
    }
  }

  return \%res;
}

sub print_timespan($)
{
  my ($time) = @_;

  my ($d, $h, $m, $s);
  $s = $time;
  $m = int( $s / 60 );
  $s -= $m * 60;
  $h = int( $m / 60 );
  $m -= $h * 60;
  $d = int( $h / 24 );
  $h -= $d * 24;

  if( $d <= 0 ) {
    return sprintf( $LANG->{timefmt}{timespan_short}, $h, $m, $s ); }
  elsif( $d < $CONST->{etc}{timespan_long_limits} ) {
    return sprintf( $LANG->{timefmt}{timespan_middle}, $d, $h, $m, $s ); }
  else {
    return sprintf( $LANG->{timefmt}{timespan_long}, $d ); }
}

sub alltrim($)
{
  my $str = $_[0];

  # remove repeated spaces and all blank chars
  $str =~ s/\s+/ /g;
  $str =~ s/^ //;
  $str =~ s/ $//;

  return $str;
}

sub VerifyString(%)
{
  my (%param) = @_;
  my $res = 1;

  while( my ($key,$val) = each(%param) )
  {
    next if $key eq 'strict';
    if( $key eq 'multiline' )
    {
      $$val =~ s/\x0D//g;
      $$val =~ s/[\x00-\x09\x0B-\x1F]/ /g
    }
    else
    {
      $$val =~ s/[\x00-\x1F]/ /g
    }
  }

  if( defined($param{email}) )
  {
    # e-mail
    if( $param{'strict'} )
    {
      $res = ( ${$param{email}} !~ /(\s|[\"\'\`\<\>\x7F-\xFF\/\\\|\*\?])/ );
      if( $res && ${$param{email}} =~ /^(.*?)(@(.+))?$/ )
      {
        my ($local,$domain) = ( $1, $3 );
        $res = ( defined($local) && $local ne '' && $local !~ /^.*?@+$/ );
        $res = ( $domain =~ /^([-a-z0-9]+\.)*[a-z]{2,}$/i ) if($res && defined($domain) && $domain ne '');
      }
      else
      {
        undef $res;
      }
    }
    else
    {
      $res = ( ${$param{email}} !~ /(\s|[\"\'\`\<\>\x7F-\xFF\/\\\|])/ );
    }
  }
  elsif( defined($param{ip}) )
  {
    # IP
    if( ${$param{ip}} =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)(\/(\d+))?$/ )
    {
      my ($a,$b,$c,$d,$m) = ( int($1), int($2), int($3), int($4), $6 );
      $m = (defined $m)? int($m): 32;
      $res = ( $a>0 && $a<224 && $b>=0 && $b<=255 && $c>=0 && $c<=255 && $d>=0 && $d<=255 && $m>0 && $m<=32 );
      ##if( $res && $m<32 )
      ##{
      ##  my %MASKS = ( 0 => 0,
      ##                1 => 0x80000000,   9 => 0xFF800000,  17 => 0xFFFF8000,  25 => 0xFFFFFF80,
      ##                2 => 0xC0000000,  10 => 0xFFC00000,  18 => 0xFFFFC000,  26 => 0xFFFFFFC0,
      ##                3 => 0xE0000000,  11 => 0xFFE00000,  19 => 0xFFFFE000,  27 => 0xFFFFFFE0,
      ##                4 => 0xF0000000,  12 => 0xFFF00000,  20 => 0xFFFFF000,  28 => 0xFFFFFFF0,
      ##                5 => 0xF8000000,  13 => 0xFFF80000,  21 => 0xFFFFF800,  29 => 0xFFFFFFF8,
      ##                6 => 0xFC000000,  14 => 0xFFFC0000,  22 => 0xFFFFFC00,  30 => 0xFFFFFFFC,
      ##                7 => 0xFE000000,  15 => 0xFFFE0000,  23 => 0xFFFFFE00,  31 => 0xFFFFFFFE,
      ##                8 => 0xFF000000,  16 => 0xFFFF0000,  24 => 0xFFFFFF00,  32 => 0xFFFFFFFF,
      ##              );
      ##  my $net = ($a<<24 & 0xFF000000) | ($b<<16 & 0xFF0000) | ($c<<8 & 0xFF00) | ($d & 0xFF);
      ##  my $subnet = $net & $MASKS{$m};
      ##  $res = ( $net == $subnet );
      ##}
    }
    else
    {
      $res = undef;
    }
  }
  elsif( defined($param{hostname}) )
  {
    # Hostname
    $res = ( ${$param{hostname}} =~ /^([-a-z0-9]+\.)+[a-z]{2,}$/i );
    ${$param{hostname}} = lc(${$param{hostname}}) if( $res );
  }
  elsif( defined($param{protected_domain}) )
  {
    $res = ( ${$param{protected_domain}} =~ /^([-a-z0-9\?]+\.)+[a-z\?]{2,}$/i );
    if(! $res && ${$param{protected_domain}} =~ /\*/)
    {
      $res = ( ${$param{protected_domain}} =~ /^[-a-z0-9\.\?\*]+$/i );
    }
  }
  elsif( defined($param{multiline}) )
  {
    # Multiline string
    ${$param{multiline}} =~ s/[ \t]+\n/\n/sg;
#    ${$param{multiline}} =~ s/\n[ \t]+/\n/sg;
#    ${$param{multiline}} =~ s/[ \t]+/ /mg;
    ${$param{multiline}} =~ s/(\n\s*)+$//sg;
  }
  elsif( defined($param{string}) )
  {
    # Single-line string
    ${$param{string}} =~ s/[\x00-\x20]/ /g;
    ${$param{string}} =~ s/^\s+//;
    ${$param{string}} =~ s/\s+$//;
    ${$param{string}} =~ s/\s+/ /g;
  }
  elsif( defined($param{mime_header}) )
  {
    $res = ( ${$param{mime_header}} !~ /[\x7F-\xFF]/ );
    ${$param{mime_header}} =~ s/\s+/ /g if( $res );
  }

  return $res;
}

sub FormatDateTime
{
  my ($frm, $sec, $min, $hours, $day, $month, $year) = @_;
  $year += 1900;
  $month = $month+1;

  # month
  $frm =~ s/%mon/$LANG->{month_names_short}{$month}/;

  $month = sprintf("%02d", $month);
  $day = sprintf("%02d", $day);
  $hours = sprintf("%02d", $hours);
  $min = sprintf("%02d", $min);
  $sec = sprintf("%02d", $sec);

  $frm =~ s/%y/$year/i;
  $frm =~ s/%m/$month/;
  $frm =~ s/%d/$day/;
  $frm =~ s/%H/$hours/;
  $frm =~ s/%M/$min/;
  $frm =~ s/%S/$sec/;
   
  return $frm;
}

sub CheckConfigLimits($$)
{
  #returns error message or undef
  my ($param, $control) = @_;

  if( $CONFIG->{$param} < $CONST->{defaults}{$param . "_min"}
              || $CONFIG->{$param} > $CONST->{defaults}{$param . "_max"} )
  {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{$control}/g;

      my $msg_limits = $LANG->{error}{invalid_value_limits};
      $msg_limits =~ s/\$min/$CONST->{defaults}{$param . "_min"}/g;
      $msg_limits =~ s/\$max/$CONST->{defaults}{$param . "_max"}/g;
      push @ERR, $msg_limits;
      return $msg;
  }
  else
  {
      return undef;
  }
}

1;
__END__

my $e = '@postmaster';
print STDERR VerifyString(email=>\$e,'strict'=>1);
