#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $LANGS $MENU);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
  require 'logs.pl';
};

my %param = GetParams();
my %error;
my $wrong_data = undef;

# Save settings
if( defined $param{ok} )
{
  my $save_map = {
                    Lang     => $CONST->{config_map}{Lang},
                    AlertsTo => $CONST->{config_map}{AlertsTo},
                    MonitoringHttpd => $CONST->{config_map}{MonitoringHttpd},
                    MonitoringKasMilter => $CONST->{config_map}{MonitoringKasMilter},
                 };

  $CONFIG->{MonitoringHttpd} = ($param{monitor_thttpd})? 'yes' : 'no';
  $CONFIG->{MonitoringKasMilter} = ($param{monitor_milter})? 'yes' : 'no';

  my $alert_emails = $param{alert};
  $alert_emails =~ s/([;,]|\s)+/;/g;
  my @emails = grep { defined($_) && $_ !~ /^\s*$/ } split( ';', $alert_emails );
  my @wrong_emails = grep { !VerifyString(email=>\$_,'strict'=>1) } @emails;
  if( scalar(@wrong_emails) )
  {
    $error{error} = $LANG->{error}{not_valid_email};
    $wrong_data = 1;
    push @ERR, @wrong_emails;
  }
  elsif( !scalar(@emails) )
  {
    $error{error} = $LANG->{error}{alertto_is_empty};
    $wrong_data = 1;
  }
  else
  {
    $CONFIG->{AlertsTo} = join(', ', @emails);
  }

  $error{notice} = $LANG->{message}{applied};
  unless( $error{error} )
  {
    if( defined $param{lang} && defined($LANGS->{$param{lang}}) && $LANGS->{$param{lang}}{charset} )
    {
      $CONFIG->{Lang}    = $param{lang};
      $CONFIG->{charset} = $LANGS->{$CONFIG->{Lang}}{charset};
      unless( WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $save_map ) )
      {
        $error{error} = $LANG->{error}{cant_save_config};
      }

      unless( $error{error} || $error{warning} )
      {
        PageOpen('cc.cgi', redirect=>"$param{this}?msg=ok");
      }
    }
    else
    {
      unless( WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $save_map ) )
      {
        $error{error} = $LANG->{error}{cant_save_config};
      }
    }
  }

  unless( $error{error} )
  {
    $error{error} = $LANG->{error}{crontab_set_fault} unless( WriteCrontab( MAILTO=>$CONFIG->{AlertsTo} ) );
  }
}
elsif( defined($param{msg}) && $param{msg} eq 'ok' )
{
  $error{notice} = $LANG->{message}{applied};
}


# Build page
my @lang_options;
foreach my $langid (keys(%{$LANGS}))
{
  push @lang_options, { val=>$langid,  text=>$LANGS->{$langid}{name}.' / '.$LANGS->{$langid}{charset} };
}
@lang_options = sort { $a->{text} cmp $b->{text} } @lang_options;

my $alert_value = ($wrong_data && defined($param{alert}))? $param{alert}: $CONFIG->{AlertsTo};

PageOpen('cc.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( action=>$param{this}, title=>$LANG->{group}{cc}{title}, memo=>$LANG->{group}{cc}{memo}, 'time'=>1 );

FormSelect( title=>$LANG->{control}{cc_lang}, id=>'lang', selected=>$CONFIG->{Lang}, options=>\@lang_options )
                                                                            if( scalar(keys(%{$LANGS})) > 1 );

FormEdit( title=>$LANG->{control}{alerts_email}, id=>'alert', value=>$alert_value, flash=>$wrong_data );

FormCheck( title=>$LANG->{control}{monitor_thttpd}, id=>'monitor_thttpd',
           value=>($CONFIG->{MonitoringHttpd}!~/no/i), one_cell=>0 );

FormCheck( title=>$LANG->{control}{monitor_milter}, id=>'monitor_milter',
           value=>($CONFIG->{MonitoringKasMilter}!~/no/i), one_cell=>0 );


FormSubmitOK();
FormClose();
ErrorBlock( %error );
PageClose();
