#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;
my $wrong_data;

$param{type} = 'email' unless defined $param{type};
$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );

my ($type,$title);

if( $param{type} eq 'ip' )
{
  $type = &STSConfig::STSCFG_ALLOW_IP;
  $title = $LANG->{control}{lists}{allowed}{ip};
}
else
{
  $type = &STSConfig::STSCFG_ALLOW_EMAIL;
  $title = $LANG->{control}{lists}{allowed}{email};
}
my @selector = ( { name=>$LANG->{static}{listtype_email}, uri=>$param{this}.'?type=email', active=>($param{type} ne 'ip') },
                 { name=>$LANG->{static}{listtype_ip},    uri=>$param{this}.'?type=ip',    active=>($param{type} eq 'ip') }  );

my $list = ();
unless( defined $error{error} )
{
  $list = STSConfig::get_local_list($type);
  STSCFG_Succeeded(\%error,'get_local_list('.(($type==&STSConfig::STSCFG_ALLOW_IP)?'STSCFG_ALLOW_IP':'STSCFG_ALLOW_EMAIL').')',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  $list = () unless defined $list;
  $list = STSCFG_PackList($list);
}

# Save settings
if( defined($param{ok}) )
{
  $error{notice} = $LANG->{message}{applied};

  $param{list} = '' unless defined $param{list};
  my $par_list = $param{list};
  $par_list =~ s/([;,]|\s)+/;/gs;
  my @list = split /;/, $par_list;
  {
    my %list;
    foreach (@list) { $list{$_}++ }
    @list = keys %list;

    if( $type == &STSConfig::STSCFG_ALLOW_IP )
    {
      my @err_ips = sort by_ips grep { !VerifyString(ip=>\$_) } @list;
      if( @err_ips )
      {
        $error{error} = $LANG->{error}{not_valid_ip};
        $wrong_data = 1;
        push @ERR, @err_ips;
      }
    }
    else
    {
      my @err_emails = sort by_emails grep { !VerifyString(email=>\$_) } @list;
      if( @err_emails )
      {
        $error{error} = $LANG->{error}{not_valid_email};
        $wrong_data = 1;
        push @ERR, @err_emails;
      }
    }
  }
  $list = \@list;

  unless( $error{error} )
  {
    my $vallist = STSCFG_UnpackList($list);
    STSConfig::set_local_list($type,$vallist);
    STSCFG_Succeeded(\%error,'set_local_list('.(($type==&STSConfig::STSCFG_ALLOW_IP)?'STSCFG_ALLOW_IP':'STSCFG_ALLOW_EMAIL').')',$LANG->{error}{cant_save_policies},$LANG->{error}{warn_save_policies});
  }
}

my $value;
{
  if( $wrong_data && defined($param{list}) )
  {
    $value = $param{list};
  }
  else
  {
    my @sorted;
    if( $type == &STSConfig::STSCFG_ALLOW_IP )
    {
      @sorted = sort by_ips @{$list};
    }
    else
    {
      @sorted = sort by_emails @{$list};
    }

    $value = join("\n", @sorted);
  }
}

# Build page
PageOpen('common_allowed.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( action=>$param{this}, title=>$LANG->{group}{common_allowed}{title}, memo=>$LANG->{group}{common_allowed}{memo}, 'time'=>1 );
FormListTypeSelector( selector=>\@selector, type=>$param{type} );
FormTextarea( title=>$title, id=>'list', rows=>15, value=>$value );
FormSubmitOK();
FormClose();
ErrorBlock( %error );
PageClose();
