#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;
my $redirect = undef;

my ($checks,$defines,$basedef);

$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );

$checks  = STSConfig::get_checks() unless defined $error{error};
$basedef = STSConfig::get_base_defines(&STSConfig::STSCFG_RULE) if( !defined($error{error}) && STSCFG_Succeeded(\%error,'get_checks',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data}) );
$defines = STSConfig::get_local_defines(&STSConfig::STSCFG_RULE) if( !defined($error{error}) && STSCFG_Succeeded(\%error,'get_base_defines(STSCFG_RULE)',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data}) );

STSCFG_Succeeded(\%error,'get_local_defines(STSCFG_RULE)',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data}) unless defined $error{error};
$error{error} = $LANG->{error}{cant_access_policy_data} unless( defined($error{error}) || ($defines && $basedef && $defines) );

$basedef = STSCFG_ValuesHash($basedef) if $basedef;
$defines = STSCFG_ValuesHash($defines) if $defines;

my $group;
my $id = $param{id};
if( defined($id) && $id>=0 && $id<scalar(@{$checks}) )
{
  $group = ${$checks}[$id];
}
else
{
#  $error{error} = $LANG->{error}{wrong_group_id};
#  push @ERR, "Incorrect rules group id = $param{id}";
  PageOpen('common_rules.cgi', redirect=>'common_rules_list.cgi');
  die "Redirected to common_rules_list.cgi due invalid rules group id=$id\n";
}

my ($rules_group_name, $rules_group_memo) = ( defined $group )? ($group->{Name}, $group->{Memo}): ('?', undef);

# "Default" command processing
if( defined $param{default} && !defined($error{error}) )
{
  if( defined($group) && defined($group->{Control}) )
  {
    foreach my $item (@{$group->{Control}})
    {
      next unless defined($item) && defined($item->{Type}) && defined($item->{ID});
      next if( $item->{Type} <= 0 );
      delete $defines->{$item->{ID}} if exists $defines->{$item->{ID}};
    }
  }
}

STSCFG_JoinValuesDefaults($defines,$basedef);

# Save settings
if( defined $param{ok} && !defined($error{error}) )
{
  $error{notice} = $LANG->{message}{applied};

  if( defined($group) && defined($group->{Control}) )
  {
    foreach my $item (@{$group->{Control}})
    {
      next unless defined($item) && defined($item->{Type}) && defined($item->{ID});
      next if( $item->{Type} <= 0 );

      my $ctl_id = '.'.$item->{ID};

      if( defined $param{$ctl_id} && $param{$ctl_id} ne '' )
      {
        $defines->{$item->{ID}} = $param{$ctl_id};
      }
      else
      {
        delete $defines->{$item->{ID}};
      }
    }
  }

  unless( STSCFG_ValidateRules( rules_group=>$group, value=>$defines ) )
  {
    $error{error} = $LANG->{error}{validation_fault};
    push @ERR, $LANG->{error}{verify_highlighted};
  }

  unless( $error{error} )
  {
    my $vallist = STSCFG_ValuesList( $basedef, $defines );
    STSConfig::set_local_defines(&STSConfig::STSCFG_RULE,$vallist);
    STSCFG_Succeeded(\%error,'set_local_defines(STSCFG_RULE,...)', $LANG->{error}{cant_build_profiles}, $LANG->{error}{warn_build_profiles} );
  }

  RestartHelper(\%error);
  STSCFG_ErrorLogHelper(\%error);

  $redirect = "common_rules.cgi?id=$id&message=ok" unless( $error{warning} || $error{error} );
}
elsif( defined($param{message}) && $param{message} eq 'ok' )
{
  $error{notice} = $LANG->{message}{applied};
}

my $memo = ($rules_group_memo)? $rules_group_memo: $LANG->{group}{common_rules}{memo};

# Build page
EnableRulesMenuItems('drules', 0);

PageOpen('common_rules.cgi', rules_group_name=>$rules_group_name, redirect=>$redirect );
ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( action=>$param{this}, title=>$LANG->{group}{common_rules}{title}, memo=>$memo, rules_group_name=>$rules_group_name, 'time'=>1 );
ShowRulesGroup( rules_group=>$group, value=>$defines, with_defaults=>0 );
FormRulesSubmit( rules_group_id=>$id );
FormClose();
ErrorBlock( %error );
PageClose();
