#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;

my ($checks,$defines,$basedef);

$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );

$checks  = STSConfig::get_checks() unless defined $error{error};
$basedef = STSConfig::get_base_defines(&STSConfig::STSCFG_RULE) if( !defined($error{error}) && STSCFG_Succeeded(\%error,'get_checks',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data}) );
$defines = STSConfig::get_local_defines(&STSConfig::STSCFG_RULE) if( !defined($error{error}) && STSCFG_Succeeded(\%error,'get_base_defines(STSCFG_RULE)',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data}) );
STSCFG_Succeeded(\%error,'get_local_defines(STSCFG_RULE)',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data}) unless defined $error{error};
$error{error} = $LANG->{error}{cant_access_policy_data} unless( defined($error{error}) || ($defines && $basedef && $defines) );

$basedef = STSCFG_ValuesHash($basedef) if $basedef;
$defines = STSCFG_ValuesHash($defines) if $defines;

my $error_load = defined($error{error})? 1: 0;

if( defined($param{def}) && $param{def}>=0 && $param{def}<scalar(@{$checks}) )
{
  my $group = $checks->[$param{def}];
  $error{notice} = $LANG->{message}{applied};

  if( defined($group) && defined($group->{Control}) )
  {
    foreach my $item (@{$group->{Control}})
    {
      next unless defined($item) && defined($item->{Type}) && defined($item->{ID});
      next if( $item->{Type} <= 0 );

      delete $defines->{$item->{ID}} if exists $defines->{$item->{ID}};
    }

    my $vallist = STSCFG_ValuesList( $basedef, $defines );

    STSConfig::set_local_defines(&STSConfig::STSCFG_RULE,$vallist);
    STSCFG_Succeeded(\%error,'set_local_defines(STSCFG_RULE,...)', $LANG->{error}{cant_build_profiles}, $LANG->{error}{warn_build_profiles} );
  }
  else
  {
    $error{error} = $LANG->{error}{wrong_group_id};
    push @ERR, "Group ID = $param{def}";
  }

  RestartHelper(\%error);
  STSCFG_ErrorLogHelper(\%error);
}
elsif( defined($param{message}) && $param{message} =~ /^ok$/i )
{
  $error{notice} = $LANG->{message}{applied};
}


# Build page
PageOpen('common_rules_list.cgi');

if( $error_load )
{
  ListOpen( title=>$LANG->{group}{common_rules_list}{title}, memo=>$LANG->{group}{common_rules_list}{memo}, 'time'=>1 );
  ListClose();
  ErrorBlock( %error );
}
else
{
  ErrorBlock( top=>1, %error ) if( $error{warning} || $error{error} );
  ListOpen( title=>$LANG->{group}{common_rules_list}{title}, memo=>$LANG->{group}{common_rules_list}{memo}, 'time'=>1 );

  my $position = 0;
  foreach my $group (@{$checks})
  {
    STSCFG_PrepareCommonRulesGroupInfo($group,$basedef,$defines);
    RulesCommonListItem( title=>$group->{Name}, memo=>$group->{Memo}, position=>$position++, rules_total=>$group->{RulesCount}, rules_modified=>$group->{RulesModified} );
  }

  ListClose();
  ErrorBlock( %error );
}

PageClose();
