#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;
my $wrong_data;

$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );

my @list;
my @kas_list;
unless( defined $error{error} )
{
  my $list = STSConfig::get_local_list(&STSConfig::STSCFG_RBL);
  STSCFG_Succeeded(\%error,'get_local_list(STSCFG_RBL)',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  @list = STSCFG_FilterRBLList($list) if( defined($list) && scalar(@{$list})>0 );

  @kas_list = STSCFG_get_local_rbl_workaround($CONFIG->{KasDNSBlackList});
}

# Save settings
if( defined $param{ok} )
{
  $error{notice} = $LANG->{message}{applied};

  my $save_map = {UseKasDNSBL => $CONST->{config_map}{UseKasDNSBL}};
  $CONFIG->{UseKasDNSBL} = $param{kasdnsbl} if defined $param{kasdnsbl};

  $error{error} = $LANG->{error}{cant_save_config}
                unless( WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $save_map ) );

  my %list;
  foreach (@list)
  {
    $list{lc($_->{Entry})} += $_->{Rate};
  }

  while( my($key,$val) = each %param )
  {
    if( $key =~ /^DNSBL:(.*)$/ )
    {
      my $hostname = lc($1);
      $list{$hostname} = int($val);
    }
  }

  if( defined($param{newhost}) && $param{newhost} !~ /^\s*$/ )
  {
    my $newhost = alltrim($param{newhost});
    if( defined($newhost) && VerifyString(hostname=>\$newhost) )
    {
      unless( $list{$newhost} )
      {
        $list{$newhost} = int($param{newrate});
      }
      else
      {
        $error{error} = $LANG->{error}{exists_new_dnsbl_hostname};
        push @ERR, $param{newhost};
        $wrong_data = 1;
      }
    }
    else
    {
      $error{error} = $LANG->{error}{wrong_new_dnsbl_hostname};
      $wrong_data = 1;
      push @ERR, $param{newhost};
    }
  }

  unless( $error{error} )
  {
    @list = ();
    while( my($key,$val) = each %list )
    {
      push @list, { Entry=>$key, Rate=>$val };
    }
    @list = STSCFG_FilterRBLList(\@list);

    STSConfig::set_local_list(&STSConfig::STSCFG_RBL,\@list);
    STSCFG_Succeeded(\%error,'set_local_list(STSCFG_RBL)',$LANG->{error}{cant_save_policies},$LANG->{error}{warn_save_policies});
  }
}
elsif( $param{'delete'} )
{
  $error{notice} = $LANG->{message}{applied};

  my $delhost = lc($param{'delete'});
  $delhost =~ s/[^-a-z0-9\.]+//ig;
  if( defined($delhost) && $delhost =~ /[-a-z0-9]+$/i )
  {
    @list = grep{ lc($_->{Entry}) ne $delhost } @list;
    STSConfig::set_local_list(&STSConfig::STSCFG_RBL,\@list);
    STSCFG_Succeeded(\%error,'set_local_list(STSCFG_RBL)',$LANG->{error}{cant_save_policies},$LANG->{error}{warn_save_policies});

    unless( defined($error{error}) )
    {
      PageOpen('dnsbl.cgi', redirect=>'dnsbl.cgi?message=ok');
      die;
    }
  }
}
elsif( defined($param{message}) && $param{message} eq 'ok' )
{
  $error{notice} = $LANG->{message}{applied};
}

# Build page
PageOpen('dnsbl.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( action=>$param{this}, %{$LANG->{group}{kas_dnsbl}}, 'time'=>1 );
FormKasDNSBLTitle();
my $count = 1;
foreach (@kas_list)
{
  my $href = $_->{Entry};
  $href =~ s!^.*?([^\.]+\.[^\.]+)$!http://www.$1!;
  FormKasDNSBLItem( count=>$count++, id=>"DNSBL:$_->{Entry}", hostname=>$_->{Entry}, value=>$_->{Rate}, href=>$href );
}

FormSelect( %{$LANG->{control}{use_kas_dnsbl_list}}, id=>'kasdnsbl', selected=>$CONFIG->{UseKasDNSBL} );

#some hack
print qq|
    </table>|;
DrawHeader( action=>$param{this}, %{$LANG->{group}{dnsbl}}, 'time'=>0 );
print qq|
    <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>|;

FormDNSBLTitle();
my $count = 1;
foreach (@list)
{
  my $href = $_->{Entry};
  $href =~ s!^.*?([^\.]+\.[^\.]+)$!http://www.$1!;
  FormDNSBLItem( count=>$count++, id=>"DNSBL:$_->{Entry}", hostname=>$_->{Entry}, value=>$_->{Rate}, href=>$href );
}
FormDNSBLNew( newhost=>(($wrong_data && defined($param{newhost}))? alltrim($param{newhost}): undef), newrate=>(($wrong_data && $param{newrate})? int($param{newrate}): undef) );

FormSubmitOK(colspan=>4);
FormClose();
ErrorBlock( %error );
PageClose();

__END__
