#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU);

use CGI qw(:standard);
use CGI::Carp;
use POSIX qw(strftime tzset tzname);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'utils.pl';
};

my %modes = (
              'd'=>{ title=>$LANG->{group}{stat_d}{title}, step=>  300, count=>288, time_fmt=>'%F %H:%M', fprefix=>'daily-' },
              'w'=>{ title=>$LANG->{group}{stat_w}{title}, step=> 3600, count=>168, time_fmt=>'%F %H:%M', fprefix=>'weekly-' },
              'm'=>{ title=>$LANG->{group}{stat_m}{title}, step=> 3600, count=>720, time_fmt=>'%F %H:%M', fprefix=>'monthly-' },
              'y'=>{ title=>$LANG->{group}{stat_y}{title}, step=>86400, count=>366, time_fmt=>'%F',       fprefix=>'annually-' },
            );

my %formats = (
                'csv'  => { content_type=>'text/plain',               ext=>'txt', },
                'xl'   => { content_type=>'application/vnd.ms-excel', ext=>'xls', },
                'html' => { content_type=>'text/html',                ext=>'html', },
              );

my %param = GetParams();

my $period = $param{period};
$period = 'd' unless( defined($period) && defined($modes{$period}) );

my $format = $param{'format'};
$format = 'csv' unless( defined($format) && defined($formats{$format}) );

my ($step, $count, $time_fmt) = ( $modes{$period}->{step}, $modes{$period}->{count}, $modes{$period}->{time_fmt} );
my $data_path = $CONFIG->{CCStatBasePath};

my $current_time = time();
my ($tzname, $tzoffset);
{
  tzset();
  $tzname = (tzname())[ ((localtime($current_time))[8])? 1 : 0 ];

  my $off = int(CalcTZOffset()/60+0.5);
  my $off_s;
  if( $off >=0 )
  {
    $off_s = '+';
  }
  else
  {
    $off_s = '-';
    $off = -$off;
  }

  my $off_h = int($off/60);
  my $off_m = $off - $off_h * 60;

  $tzoffset = sprintf( "GMT$off_s%d:%02d", $off_h, $off_m);
}

my @data;
{
  my $data = load_stat($data_path,$step,$count);
  unless( $data )
  {
      print redirect( -uri => "stat.cgi?period=$period&data=$param{data}" );
      return 0;
  }

  @data = @{$data};
  foreach my $item (@data)
  {
    $item->{human_time} = strftime($time_fmt,localtime($item->{'time'})),
  }
}

my $exfname = $modes{$period}->{fprefix} . strftime('%y%m%d-%H%M',localtime($current_time)) . '.' . $formats{$format}->{ext};
my $title   = $modes{$period}->{title} . ' ' . strftime('%F %R',localtime($current_time));


print header ( -TYPE => $formats{$format}->{content_type}.'; charset='.$CONFIG->{charset},
               -EXPIRES => '+'.int($step/4).'s',
#               -EXPIRES => '-1s',
               -CONTENT_DISPOSITION => qq|attachment; filename="$exfname"|
             );

open_export($format);
header_export($format, $modes{$period}->{title}, $tzname, $tzoffset);

foreach my $item (@data)
{
  my @line = [
                { text=>$item->{human_time}, nowrap=>1 },
                { text=>$item->{normal_m} },   { text=>$item->{normal_s} },
                { text=>$item->{spam_m} },     { text=>$item->{spam_s} },
                { text=>$item->{probable_m} }, { text=>$item->{probable_s} },
                { text=>$item->{formal_m} },   { text=>$item->{formal_s} },
                { text=>$item->{trust_m} },    { text=>$item->{trust_s} },
                { text=>$item->{black_m} },    { text=>$item->{black_s} },
             ];
  exporl_line($format,@line);
}

close_export($format);


sub open_export($)
{
  my ($format) = @_;
  if( $format eq 'xl' )
  {
    print qq|<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=$CONFIG->{charset}">
<title>$title - $LANG->{static}{product_name}</title>
</head>
<body>
<table border=1 cellpadding=5 cellspacing=0>
|;
  }
  elsif( $format eq 'html' )
  {
    print qq|<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=$CONFIG->{charset}">
<title>$title - $LANG->{static}{product_name}</title>
<style type="text/css"><!--
 body { font-family: Verdana, Arial, sans-serif;
        font-size: 8pt; }
 td { font-family: Verdana, Arial, sans-serif;
        font-size: 8pt; }
--></style>
</head>
<body>
<table border=1 cellpadding=5 cellspacing=0>
|;
  }
}


sub close_export($)
{
  my ($format) = @_;
  if( $format =~ /^(xl)|(html)$/i )
  {
    print qq|</table>
<!--
  Copyright (C) 2002-2006 Kaspersky Lab.
  All rights reserved.
-->
</body>
</html>
|;
  }
}

sub header_export($$$$)
{
  my ($format,$title,$tzname,$tzoff) = @_;

  my @lines = (
                [ { text=>"$LANG->{static}{product_name}: $title", colspan=>13, bold=>1, big=>1 } ],
                [ { text=>$LANG->{static}{copyright_text}, colspan=>13, it=>1, small=>1 } ],
                [ { text=>' ', colspan=>13 } ],
                [ { text=>"$LANG->{static}{stat_export_htime}:", colspan=>2, bold=>1 }, { text=>strftime('%F %R',localtime($current_time)), colspan=>2 } ],
                [ { text=>"$LANG->{static}{stat_export_htz}:", colspan=>2, bold=>1 }, { text=>$tzoff . " " . $tzname, colspan=>2 } ],
                [ { text=>' ', colspan=>13 } ],
                [
                  { text=>$LANG->{static}{stat_export_time}, bold=>1 },
                  { text=>$LANG->{static}{stat_export_normal_m}, bold=>1 }, { text=>$LANG->{static}{stat_export_normal_s}, bold=>1 },
                  { text=>$LANG->{static}{stat_export_spam_m}, bold=>1 }, { text=>$LANG->{static}{stat_export_spam_s}, bold=>1 },
                  { text=>$LANG->{static}{stat_export_probable_m}, bold=>1 }, { text=>$LANG->{static}{stat_export_probable_s}, bold=>1 },
                  { text=>$LANG->{static}{stat_export_formal_m}, bold=>1 }, { text=>$LANG->{static}{stat_export_formal_s}, bold=>1 },
                  { text=>$LANG->{static}{stat_export_trust_m}, bold=>1 }, { text=>$LANG->{static}{stat_export_trust_s}, bold=>1 },
                  { text=>$LANG->{static}{stat_export_black_m}, bold=>1 }, { text=>$LANG->{static}{stat_export_black_s}, bold=>1 },
                ],
              );
  foreach (@lines)
  {
    exporl_line($format,$_);
  }
}



sub exporl_line($$)
{
  my ($format,$cols) = @_;
  my $cols_count = 13;

  if( $format =~ /^(xl)|(html)$/i )
  {
    print '<tr>';
    foreach my $item (@{$cols})
    {
      $item = {} unless defined $item;
      $item->{text} = ' ' unless( defined $item->{text} );

      my $out = SafeEscapeHTML($item->{'text'});
      $out =~ s/\n/<br>/gm;
      $out = '' unless defined $out;
      $out = "<i>$out</i>" if $item->{it};
      $out = "<b>$out</b>" if $item->{bold};
      $out = "<big>$out</big>" if $item->{big};
      $out = "<small>$out</small>" if $item->{small};

      my $colspan = $item->{colspan};
      if( $colspan && $colspan!=1 )
      {
        $cols_count -= $colspan;
        $colspan = " colspan=$colspan";
      }
      else
      {
        --$cols_count;
        $colspan = '';
      }

      my $nowrap = ($item->{nowrap})? ' nowrap': '';

      print "<td$colspan$nowrap>$out</td>";
    }
    for( ; $cols_count>0; --$cols_count )
    {
      print "<td>&nbsp;</td>";
    }
    print "</tr>\n";
  }
  else
  {
    my $coma = 0;
    foreach my $item (@{$cols})
    {
      my $out = $item->{text};
      $out = '' unless defined $out;
      $out =~ s/\"/\\\"/;
      $out =~ s/\n/\r\n/gm;
      if( $coma )
      {
        print ',';
      }
      else
      {
        $coma = 1;
      }
      print qq|"$out"|;
      --$cols_count;
    }
    for( ; $cols_count>0; --$cols_count )
    {
      if( $coma )
      {
        print ',';
      }
      else
      {
        $coma = 1;
      }
      print '""';
    }
    print "\r\n";
  }
}

__END__;
