#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};


my %param = GetParams();
my %error;
my $wrong_params;
my %error_mark;
my %ACTION_VALUE_TO_FORM = (
                              &STSConfig::STSCFG_ACCEPT    => 'accept',
                              &STSConfig::STSCFG_ADDRCPT   => 'copy',
                              &STSConfig::STSCFG_NEWRCPT   => 'forward',
                              &STSConfig::STSCFG_REJECT    => 'reject',
                              &STSConfig::STSCFG_BOUNCE    => 'reject',
                              &STSConfig::STSCFG_BLACKHOLE => 'blackhole',
                           );
my %ACTION_FORM_TO_VALUE = (
                              'accept'    => &STSConfig::STSCFG_ACCEPT,
                              'copy'      => &STSConfig::STSCFG_ADDRCPT,
                              'forward'   => &STSConfig::STSCFG_NEWRCPT,
                              'reject'    => &STSConfig::STSCFG_REJECT,
                              'blackhole' => &STSConfig::STSCFG_BLACKHOLE,
                           );


sub verify_email($$)
{
  my ($emails,$refflag) = @_;
  unless( defined($emails) && $emails ne '' )
  {
    if( $refflag )
    {
      $wrong_params = $$refflag = 1;
      $error{error} = $LANG->{error}{sendto_is_empty};
    }
    return undef;
  }

  $emails =~ s/[;,]/ /g;
  my @emails = grep { defined($_) && !/^\s*$/ } split( /\s+/, $emails );
  my @wrong_emails = sort by_emails grep { !VerifyString( email=>\$_, 'strict'=>1 ) } @emails;

  if( @wrong_emails )
  {
    if( $refflag )
    {
      $error{error} = $LANG->{error}{validation_fault};
      $wrong_params = $$refflag = 1;
      foreach (@wrong_emails)
      {
        push @ERR, $LANG->{error}{not_valid_email} . ': ' . $_;
      }
    }
    return undef;
  }
  else
  {
    if(@emails)
    {
      return join('; ', @emails);
    }
    else
    {
      if( $refflag )
      {
        $wrong_params = $$refflag = 1;
        $error{error} = $LANG->{error}{sendto_is_empty};
      }
      return undef;
    }
  }
}

sub make_rx($)
{
  my ($str) = @_;
  return '.' unless( defined($str) && $str !~ /^\s*$/ );

  $str =~ s/^\s+//;
  $str =~ s/\s+$//;
  return ( defined($str) && $str ne '' )? quotemeta($str): '.';
}


sub verify_prefix($$)
{
  my ($prefix,$refflag) = @_;
  return ('','.') unless( defined($prefix) && $prefix ne '' );

  unless( VerifyString(mime_header=>\$prefix) )
  {
    if( $refflag )
    {
      $error{error} = $LANG->{error}{validation_fault};
      $wrong_params = $$refflag = 1;
      push @ERR, $LANG->{error}{not_valid_mime_header} . ": $prefix";
    }
    return ('','.');
  }

  $prefix =~ s/^\s+//;
  return ($prefix,make_rx($prefix));
}

sub verify_suffix($$)
{
  my ($suffix,$refflag) = @_;
  return ('','.') unless( defined($suffix) && $suffix ne '' );

  unless( VerifyString(mime_header=>\$suffix) )
  {
    if( $refflag )
    {
      $error{error} = $LANG->{error}{validation_fault};
      $wrong_params = $$refflag = 1;
      push @ERR, $LANG->{error}{not_valid_mime_header} . ": $suffix";
    }
    return ('','.');
  }

  $suffix =~ s/\s+$//;
  return ($suffix,make_rx($suffix));
}

sub verify_header($$)
{
  my ($header,$refflag) = @_;
  return undef unless( defined($header) && $header ne '' );

  unless( VerifyString(mime_header=>\$header) )
  {
    if( $refflag )
    {
      $error{error} = $LANG->{error}{validation_fault};
      $wrong_params = $$refflag = 1;
      push @ERR, $LANG->{error}{not_valid_mime_header} . ": $header";
    }
    return undef;
  }

  $header =~ s/^\s+//;
  $header =~ s/\s+$//;
  return $header;
}


$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );
my $error_load = defined($error{error});

my $groups = ();
unless( $error{error} )
{
  $groups = STSConfig::get_groups();
  STSCFG_Succeeded(\%error,'get_groups()',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  STSCFG_CheckGroupList($groups);
  $groups = () unless defined $groups;
}

my $gid = int($param{gid});
if( defined $param{gid} && STSCFG_VerifyGroupID($groups,$gid) )
{
  EnableGroupMenuItems($gid);
}
else
{
  PageOpen('group_members.cgi', redirect=>'group_list.cgi' );
  die "group_action.cgi: Group width id=$gid not exist\n";
}

my $hidden = "<input type=hidden name=gid value=$gid>";
my $group_name;
{
  my $g = STSCFG_GetGroupByID($groups,$gid);
  $group_name = (defined($g) && defined($g->{Name}))? $g->{Name} : '?';
}

my $statuses = ();
unless( defined $error{error} )
{
  $statuses = STSConfig::get_statuses();
  STSCFG_Succeeded(\%error,'get_statuses()',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  $statuses = () unless defined $statuses;
}

my ($basedef, $defines);
unless( defined $error{error} )
{
  $basedef = STSConfig::get_base_defines(&STSConfig::STSCFG_ACTION);
  STSCFG_Succeeded(\%error,'get_base_defines(STSCFG_ACTION)',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  $basedef = $basedef ? STSCFG_ValuesHash($basedef) : {};
  STSCFG_JoinValuesDefaults($basedef,$CONST->{actions_defaults});

  unless( defined $error{error} )
  {
    $defines = STSConfig::get_local_defines(&STSConfig::STSCFG_ACTION);
    STSCFG_Succeeded(\%error,'get_local_defines(STSCFG_ACTION)',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
    $defines = $defines? STSCFG_ValuesHash($defines) : $basedef;
  }

  unless( defined $error{error} )
  {
    STSCFG_JoinValuesDefaults($defines,$basedef);
    $basedef = $defines;
  }
}

unless( defined $error{error} )
{
  $defines = STSConfig::get_group_defines($gid,&STSConfig::STSCFG_ACTION);
  STSCFG_Succeeded(\%error,"get_group_defines($gid,STSCFG_ACTION)",$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  $defines = $defines? STSCFG_ValuesHash($defines) : {};
}

STSCFG_JoinValuesDefaults($defines,$basedef) unless( defined $error{error} );

# Save settings
if( defined $param{ok} )
{
  $error{notice} = $LANG->{message}{applied};

  foreach my $category (@{$statuses})
  {
    next unless( defined($category) && defined($category->{Name}) && defined($category->{Status}) && defined($category->{ID}) && $category->{Name} ne '' && $category->{Status} ne '' && $category->{ID} ne '' );

    my $action_name = 'ACTION_'.$category->{ID};
    $error_mark{$category->{ID}} = { sendto=>0, subject_prefix=>0, subject_suffix=>0, category=>0, x_header=>0 };

    my $mode = $defines->{$action_name.'_MODE'} = ( exists $ACTION_FORM_TO_VALUE{$param{$category->{Status}.'_action'}} )? $ACTION_FORM_TO_VALUE{$param{$category->{Status}.'_action'}} : &STSConfig::STSCFG_ACCEPT;
    $defines->{$action_name.'_EMAIL'} = verify_email( $param{$category->{Status}.'_sendto'}, ($mode==&STSConfig::STSCFG_ADDRCPT || $mode==&STSConfig::STSCFG_NEWRCPT)? \$error_mark{$category->{ID}}{sendto}: undef );
    ($defines->{$action_name.'_SUBJECT_PREFIX'}, $defines->{$action_name.'_SUBJECT_PREFIX_RX'}) = verify_prefix( $param{$category->{Status}.'_subject_prefix'}, ($mode==&STSConfig::STSCFG_REJECT || $mode==&STSConfig::STSCFG_BLACKHOLE)? undef: \$error_mark{$category->{ID}}{subject_prefix} );
#    ($defines->{$action_name.'_SUBJECT_SUFFIX'}, $defines->{$action_name.'_SUBJECT_SUFFIX_RX'}) = verify_suffix( $param{$category->{Status}.'_subject_suffix'}, ($mode==&STSConfig::STSCFG_REJECT || $mode==&STSConfig::STSCFG_BLACKHOLE)? undef: \$error_mark{$category->{ID}}{subject_suffix} );
#    $defines->{$action_name.'_KEYWORDS'}       = verify_header( $param{$category->{Status}.'_category'}, ($mode==&STSConfig::STSCFG_REJECT || $mode==&STSConfig::STSCFG_BLACKHOLE)? undef: \$error_mark{$category->{ID}}{category} );
    $defines->{$action_name.'_USERINFO'}       = verify_header( $param{$category->{Status}.'_x_header'}, ($mode==&STSConfig::STSCFG_REJECT || $mode==&STSConfig::STSCFG_BLACKHOLE)? undef: \$error_mark{$category->{ID}}{x_header} );
  }

  unless( defined($error{error}) )
  {
    my $vallist = STSCFG_ValuesList( {}, $defines );
    STSConfig::set_group_defines($gid,&STSConfig::STSCFG_ACTION,$vallist);
    STSCFG_Succeeded(\%error,"set_group_defines($gid,STSCFG_ACTION)",$LANG->{error}{cant_save_policies},$LANG->{error}{warn_save_policies});
    RestartHelper(\%error);
    STSCFG_ErrorLogHelper(\%error);
  }
}

# Build page
PageOpen("group_actions.cgi?gid=$gid", group_name=>$group_name, script=>'actions.js.cgi', onload=>'Show_Groups();' );
ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( action=>$param{this}, title=>$LANG->{group}{group_actions}{title},
        memo=>$LANG->{group}{group_actions}{memo}, group_name=>$group_name, 'time'=>1 );

foreach my $category (@{$statuses})
{
  next unless( defined($category) && defined($category->{Name}) && defined($category->{Status}) && defined($category->{ID}) && $category->{Name} ne '' && $category->{Status} ne '' && $category->{ID} ne '' );

  my ($action,$sendto,$subject_prefix,$subject_suffix,$keywords,$x_header);

  if( $wrong_params )
  {
    $action         = $param{$category->{Status}.'_action'};
    $sendto         = $param{$category->{Status}.'_sendto'};
    $subject_prefix = $param{$category->{Status}.'_subject_prefix'};
#    $subject_suffix = $param{$category->{Status}.'_subject_suffix'};
#    $keywords       = $param{$category->{Status}.'_category'};
    $x_header       = $param{$category->{Status}.'_x_header'};
  }

  $action         = ( exists $ACTION_VALUE_TO_FORM{$defines->{'ACTION_'.$category->{ID}.'_MODE'}} )? $ACTION_VALUE_TO_FORM{$defines->{'ACTION_'.$category->{ID}.'_MODE'}} : $ACTION_VALUE_TO_FORM{&STSConfig::STSCFG_ACCEPT} unless( defined $action );
  $sendto         = $defines->{'ACTION_'.$category->{ID}.'_EMAIL'} unless( defined $sendto );
  $subject_prefix = $defines->{'ACTION_'.$category->{ID}.'_SUBJECT_PREFIX'} unless( defined $subject_prefix );
#  $subject_suffix = $defines->{'ACTION_'.$category->{ID}.'_SUBJECT_SUFFIX'} unless( defined $subject_suffix );
#  $keywords       = $defines->{'ACTION_'.$category->{ID}.'_KEYWORDS'} unless( defined $keywords );
  $x_header       = $defines->{'ACTION_'.$category->{ID}.'_USERINFO'} unless( defined $x_header );

#  FormActionBlock( type=>$category->{Status}, name=>$category->{Name}, memo=>$category->{Memo}, action=>$action, sendto=>$sendto, subject_prefix=>$subject_prefix, subject_suffix=>$subject_suffix, category=>$keywords, x_header=>$x_header, error_marks=>$error_mark{$category->{ID}} );
  FormActionBlock( type=>$category->{Status}, name=>$category->{Name}, memo=>$category->{Memo}, action=>$action, sendto=>$sendto, subject_prefix=>$subject_prefix, x_header=>$x_header, error_marks=>$error_mark{$category->{ID}} );
}

FormSubmitOK( onclick_reset=>'Show_Groups();', hidden=>$hidden );
FormClose();
ErrorBlock( %error );
PageClose();
