#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;
my $wrong_data;

$param{type} = 'email' unless defined $param{type};
$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );

my ($type,$title);

if( $param{type} eq 'ip' )
{
  $type = &STSConfig::STSCFG_DENY_IP;
  $title = $LANG->{control}{lists}{blocked}{ip};
}
else
{
  $type = &STSConfig::STSCFG_DENY_EMAIL;
  $title = $LANG->{control}{lists}{blocked}{email};
}

my $groups = ();
unless( $error{error} )
{
  $groups = STSConfig::get_groups();
  STSCFG_Succeeded(\%error,'get_groups()',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  STSCFG_CheckGroupList($groups);
  $groups = () unless defined $groups;
}

my $gid = int($param{gid});
if( defined $param{gid} && STSCFG_VerifyGroupID($groups,$gid) )
{
  EnableGroupMenuItems($gid);
}
else
{
  PageOpen('group_members.cgi', redirect=>'group_list.cgi' );
  die "group_action.cgi: Group width id=$gid not exist\n";
}

my $hidden = "<input type=hidden name=gid value=$gid>";
my @selector = ( { name=>$LANG->{static}{listtype_email}, uri=>$param{this}."?gid=$gid&type=email", active=>($param{type} ne 'ip') },
                 { name=>$LANG->{static}{listtype_ip},    uri=>$param{this}."?gid=$gid&type=ip",    active=>($param{type} eq 'ip') }  );

my $group_name;
{
  my $g = STSCFG_GetGroupByID($groups,$gid);
  $group_name = (defined($g) && defined($g->{Name}))? $g->{Name} : '?';
}

my $list = ();
unless( defined $error{error} )
{
  $list = STSConfig::get_group_list($gid,$type);
  STSCFG_Succeeded(\%error,"get_group_list($gid,".(($type==&STSConfig::STSCFG_ALLOW_IP)?'STSCFG_DENY_IP':'STSCFG_DENY_EMAIL').')',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  $list = () unless defined $list;
  $list = STSCFG_PackList($list);
}


# Save settings
if( defined $param{ok} )
{
  $error{notice} = $LANG->{message}{applied};

  $param{list} = '' unless defined $param{list};
  my $par_list = $param{list};
  $par_list =~ s/([;,]|\s)+/;/gs;
  my @list = split /;/, $par_list;
  {
    my %list;
    foreach (@list) { $list{$_}++ }
    @list = keys %list;

    if( $type == &STSConfig::STSCFG_DENY_IP )
    {
      my @err_ips = sort by_ips grep { !VerifyString(ip=>\$_) } @list;
      if( @err_ips )
      {
        $error{error} = $LANG->{error}{not_valid_ip};
        $wrong_data = 1;
        push @ERR, @err_ips;
      }
    }
    else
    {
      my @err_emails = sort by_emails grep { !VerifyString(email=>\$_) } @list;
      if( @err_emails )
      {
        $error{error} = $LANG->{error}{not_valid_email};
        $wrong_data = 1;
        push @ERR, @err_emails;
      }
    }
  }
  $list = \@list;

  unless( $error{error} )
  {
    my $vallist = STSCFG_UnpackList($list);
    STSConfig::set_group_list($gid,$type,$vallist);
    STSCFG_Succeeded(\%error,"set_group_list($gid,".(($type==&STSConfig::STSCFG_ALLOW_IP)?'STSCFG_DENY_IP':'STSCFG_DENY_EMAIL').')',$LANG->{error}{cant_save_policy_data},$LANG->{error}{warn_save_policy_data});
  }
}

my $value;
{
  if( $wrong_data && defined($param{list}) )
  {
    $value = $param{list};
  }
  else
  {
    my @sorted;
    if( $type == &STSConfig::STSCFG_DENY_IP )
    {
      @sorted = sort by_ips @{$list};
    }
    else
    {
      @sorted = sort by_emails @{$list};
    }

    $value = join("\n", @sorted);
  }
}

# Build page
PageOpen("group_blocked.cgi?gid=$gid", group_name=>$group_name );
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( action=>$param{this}, title=>$LANG->{group}{group_blocked}{title}, memo=>$LANG->{group}{group_blocked}{memo}, group_name=>$group_name, 'time'=>1 );
FormListTypeSelector( selector=>\@selector, type=>$param{type} );
FormTextarea( title=>$title, id=>'list', rows=>15, value=>$value );
FormSubmitOK( hidden=>$hidden );
FormClose();
ErrorBlock( %error );
PageClose();
