#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;

$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );
my $error_load = defined($error{error});

my $groups;
unless( $error{error} )
{
  $groups = STSConfig::get_groups();
  STSCFG_Succeeded(\%error,'get_groups()',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  STSCFG_CheckGroupList($groups);
}

# Save settings
unless( defined($error{error}) )
{
  if( defined $param{ok} )
  {
    $error{notice} = $LANG->{message}{applied};
  }
  elsif( $param{up} )
  {
    $error{notice} = $LANG->{message}{applied};

    my @group_order;
    my $pos = undef;
    my $count = 0;
    foreach (@{$groups})
    {
      next unless( defined($_) && $_->{ID} );
      push @group_order, $_->{ID};
      $pos = $count if( $_->{ID} == $param{up} );
      $count++;
    }

    @group_order[$pos-1,$pos] = @group_order[$pos,$pos-1] if $pos;
    push @group_order, 0;

    STSConfig::reorder_groups(\@group_order);
    STSCFG_Succeeded(\%error,"reorder_groups()",$LANG->{error}{cant_reorder_group},$LANG->{error}{warn_reorder_group});

    RestartHelper(\%error);
    STSCFG_ErrorLogHelper(\%error);

    unless( defined $error{error} )
    {
      PageOpen('group_list.cgi', redirect=>'group_list.cgi?ok=1');
      die;
    }
    else
    {
      $groups = STSConfig::get_groups();
    }
  }
  elsif( $param{'delete'} )
  {
    $error{notice} = $LANG->{message}{applied};

    my $gid = $param{'delete'};
    STSConfig::remove_group($gid);
    STSCFG_Succeeded(\%error,"remove_group($gid)",$LANG->{error}{cant_remove_group},$LANG->{error}{warn_remove_group});
    RestartHelper(\%error);
    STSCFG_ErrorLogHelper(\%error);
    STSCFG_DeleteGroupStat($gid) if( $gid );

    unless( defined $error{error} )
    {
      PageOpen('group_list.cgi', redirect=>'group_list.cgi?ok=1');
      die;
    }
    else
    {
      $groups = STSConfig::get_groups();
    }
  }
}

# Build page
PageOpen('group_list.cgi');

if( $error_load )
{
  ListOpen( title=>$LANG->{group}{common_rules_list}{title}, memo=>$LANG->{group}{common_rules_list}{memo}, 'time'=>1 );
  ListClose();
  ErrorBlock( %error );
}
else
{
#  my @selector = ( { name=>qq|<img src=image/btn_editmod.gif width=22 height=22 vspace=0 hspace=2 border=0 alt="$LANG->{static}{create_new_group}">|, uri=>'group_members.cgi?new=1', active=>0 } );
#  FormListCommands( selector=>\@selector );

  ErrorBlock( top=>1, %error ) if( $error{warning} || $error{error} );
  ListOpen( title=>$LANG->{group}{group_list}{title}, memo=>$LANG->{group}{group_list}{memo}, 'time'=>1 );
  GroupNewItem();

  my $position = 0;
  use utf8;
  foreach my $group (@{$groups})
  {
    my $memo = $group->{Memo};
    my $memo_view_len = 50;
    $memo =~ s/^((\&(amp|quot|lt|gt|#\d+);|.){$memo_view_len})(.*)$/"$1".($4?" ...":"")/ei;

    GroupListItem( gid=>$group->{ID}, title=>$group->{Name}, memo=>$memo, position=>$position++ );
  }

  ListClose();
  ErrorBlock( %error );
}

PageClose();
