#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;
my $wrong_data;

$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );
my $error_load = defined($error{error});

my $groups;
unless( $error{error} )
{
  $groups = STSConfig::get_groups();
  STSCFG_Succeeded(\%error,'get_groups()',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  STSCFG_CheckGroupList($groups);
}

my %group_name;
if( $groups )
{
  foreach (@$groups)
  {
    my $name = defined($_->{Name})? $_->{Name}: '';
    $name =~ s/^\s+//;
    $name =~ s/\s+$//;
    $name =~ s/\s+/ /g;
    $group_name{$name} = $_->{ID};
  }
}

my $gid = undef;
my $new = defined $param{'new'};
if( defined $param{gid} )
{
  delete $param{'new'} if defined $param{'new'};
  $new = undef;

  $gid = int($param{gid});
  if( STSCFG_VerifyGroupID($groups,$gid) )
  {
    EnableGroupMenuItems($gid);
  }
  else
  {
    PageOpen('group_members.cgi', redirect=>'group_list.cgi' );
    die "group_members.cgi: Group width id=$gid not exist\n";
  }
}

# Save settings
if( defined($param{msg}) )
{
    $error{notice} = $LANG->{message}{applied};
}

if( defined $param{ok} && !defined($error{error}) )
{
    $error{notice} = $LANG->{message}{applied};

    unless( defined($param{name}) && $param{name} !~ /^\s*$/ )
    {
      $error{error} = $LANG->{error}{not_defined_mandatory};
    }
    else
    {
      VerifyString(string=>\$param{name});

      my $test_name = $param{name};
      $test_name =~ s/^\s+//;
      $test_name =~ s/\s+$//;
      $test_name =~ s/\s+/ /g;

      if( defined($gid) && exists($group_name{$test_name}) && $group_name{$test_name}!=$gid || !defined($gid) && exists($group_name{$test_name}) )
      {
        $error{error} = $LANG->{error}{group_name_already_exist};
        $wrong_data = 1;
      }
    }

    VerifyString(multiline=>\$param{memo}) if defined($param{memo});

    if(defined($param{memo}) && length($param{memo}) > $CONST->{etc}{max_group_desc_length})
    {
      my $msg = $LANG->{error}{group_descr_too_long};
      $msg =~ s/\$field/$CONST->{etc}{max_group_desc_length}/g;
      $error{error} = $msg;

      $wrong_data = 1;
    }

    my $members = (defined($param{members}))? $param{members}: '';
    $members =~ s/([;,]|\s)+/;/gs;
    my @members = grep { defined($_) && $_ !~ /^\s*$/ } split /;/, $members;
    {
      my %members;
      foreach (@members) { $members{$_}++ }
      @members = keys %members;

      my @err_emails = sort by_emails grep { !VerifyString(email=>\$_) } @members;
      if( @err_emails )
      {
        $error{error} = $LANG->{error}{not_valid_email};
        $wrong_data = 1;
        push @ERR, @err_emails;
      }
    }
    unless( @members )
    {
      $error{error} = $LANG->{error}{not_defined_mandatory};
      $wrong_data = 1;
    }

    if( defined $param{'new'} )
    {
      my $newgid = undef;
      unless( defined($error{error}) )
      {
        $newgid = STSConfig::add_group( { Name=>$param{name}, Memo=>$param{memo} } );
        STSCFG_Succeeded(\%error,'add_group',$LANG->{error}{cant_create_group},$LANG->{error}{warn_create_group});
        $error{error} = $LANG->{error}{cant_create_group} unless defined($error{error}) || $newgid;
        STSCFG_DeleteGroupStat($newgid) if( $newgid );
      }

      if( $newgid && !$wrong_data && scalar(@members) )
      {
        my $members = \@members;
        my $vallist = STSCFG_UnpackList($members);
        STSConfig::set_group_list($newgid,&STSConfig::STSCFG_MEMBERS,$vallist);
        STSCFG_Succeeded(\%error,"set_group_list($newgid,STSCFG_MEMBERS)",$LANG->{error}{cant_save_list},$LANG->{error}{warn_save_list});
      }

      RestartHelper(\%error);
      STSCFG_ErrorLogHelper(\%error);

      unless( $error{error} || $error{warning} )
      {
        PageOpen('group_members.cgi', redirect=>"group_members.cgi?gid=$newgid&msg=1" );
        die;
      }
    }
    elsif( defined $param{gid} )
    {
      if( !$wrong_data && scalar(@members) )
      {
        my $members = \@members;
        my $vallist = STSCFG_UnpackList($members);
        STSConfig::set_group_list($gid,&STSConfig::STSCFG_MEMBERS,$vallist);
        STSCFG_Succeeded(\%error,"set_group_list($gid,STSCFG_MEMBERS)",$LANG->{error}{cant_save_list},$LANG->{error}{warn_save_list});
      }

      unless( defined($error{error}) )
      {
        STSConfig::set_group($gid, { Name=>$param{name}, Memo=>$param{memo} });
        STSCFG_Succeeded(\%error,"set_group($gid,...)",$LANG->{error}{cant_update_group_info},$LANG->{error}{warn_update_group_info});
      }

      RestartHelper(\%error);
      STSCFG_ErrorLogHelper(\%error);

      unless( $error{error} || $error{warning} )
      {
        PageOpen('group_members.cgi', redirect=>"group_members.cgi?gid=$gid&msg=1" );
        die;
      }
    }
}

# Build page
my ($page,$title,$memo,$group_name,$group_memo,$group_members,$disable,$hidden);
if( $new )
{
  $page = 'group_members.cgi?new';
  $title = $LANG->{group}{group_new}{title};
  $memo = $LANG->{group}{group_new}{memo};
  $group_name = defined($param{name})? $param{name}: '';
  $group_memo = defined($param{memo})? $param{memo}: '';
  $group_members = defined($param{members})? $param{members}: '';
  $hidden = '<input type=hidden name=new value=1>';
}
else
{
  $page = "group_members.cgi?gid=$gid";
  $title = $LANG->{group}{group_members}{title};
  $memo = $LANG->{group}{group_members}{memo};

  my $g = STSCFG_GetGroupByID($groups,$gid);

  $group_name = $param{name};
  $group_memo = $param{memo};

  $group_name = (defined($g) && defined($g->{Name}))? $g->{Name} : '' unless defined($param{name});
  $group_memo = (defined($g) && defined($g->{Memo}))? $g->{Memo} : '' unless defined($param{memo});
  if( $wrong_data && defined($param{members}) )
  {
    $group_members = $param{members};
  }
  elsif( $gid )
  {
    my $members = STSConfig::get_group_list($gid,&STSConfig::STSCFG_MEMBERS);
    STSCFG_Succeeded(\%error,"get_group_list($gid,STSCFG_MEMBERS)",$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
    $members = () unless defined $members;
    $members = STSCFG_PackList($members);
    $group_members = join("\n", (sort by_emails @{$members}) );
  }
  else
  {
    $disable = 1;
    $group_members = '';
  }
  $hidden = "<input type=hidden name=gid value=$gid>";
}

PageOpen($page, group_name=>$group_name );
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( action=>$param{this}, title=>$title, memo=>$memo, group_name=>$group_name, gid=>$gid, 'time'=>1 );
FormEdit( title=>$LANG->{control}{group_name}, id=>'name', value=>$group_name, disable=>$disable, maxlen=>$CONST->{etc}{groupname_limit} );
unless($new)
{
  FormEdit( title=>$LANG->{control}{group_id}, id=>'gr_id', value=>$gid, disable=>1);
}
FormTextarea( title=>$LANG->{control}{group_memo}, id=>'memo', rows=>3, value=>$group_memo, disable=>$disable );
FormTextarea( title=>$LANG->{control}{group_members}, id=>'members', rows=>15, value=>$group_members, disable=>$disable );
FormSubmitOK( disable=>$disable, hidden=>$hidden );
FormClose();
ErrorBlock( %error );
PageClose();
