#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;

$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );
my $error_load = defined($error{error});

my $groups = ();
unless( $error{error} )
{
  $groups = STSConfig::get_groups();
  STSCFG_Succeeded(\%error,'get_groups()',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  STSCFG_CheckGroupList($groups);
  $groups = () unless defined $groups;
}

my $gid = int($param{gid});
if( defined($gid) && STSCFG_VerifyGroupID($groups,$gid) )
{
  EnableGroupMenuItems($gid);
}
else
{
  PageOpen('group_members.cgi', redirect=>'group_list.cgi' );
  die "group_action.cgi: Group width id=$gid not exist\n";
}

my $group_name;
{
  my $g = STSCFG_GetGroupByID($groups,$gid);
  $group_name = (defined($g) && defined($g->{Name}))? $g->{Name} : '?';
}

my $checks;
unless( defined $error{error} )
{
  $checks  = STSConfig::get_checks() unless defined $error{error};
  STSCFG_Succeeded(\%error,"get_checks()",$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
}

my $rules_group;
my $id = $param{id};
if( defined($id) && $id>=0 && $id<scalar(@{$checks}) )
{
  $rules_group = ${$checks}[$id];
}
else
{
  PageOpen('group_rules.cgi', redirect=>"group_rules_list.cgi?gid=$gid");
  die "Redirected to group_rules_list.cgi?gid=$gid due invalid rules group id=$id\n";
}
my ($rules_group_name, $rules_group_memo) = ( defined $rules_group )? ($rules_group->{Name}, $rules_group->{Memo}): ('?', undef);

my $defines;
unless( defined $error{error} )
{
  $defines = STSConfig::get_group_defines($gid,&STSConfig::STSCFG_RULE);
  STSCFG_Succeeded(\%error,"get_group_defines($gid,STSCFG_RULE)",$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
}
$defines = $defines? STSCFG_ValuesHash($defines) : {};

# "Default" command processing
if( defined $param{default} && !defined($error{error}) )
{
  if( defined($rules_group) && defined($rules_group->{Control}) )
  {
    foreach my $item (@{$rules_group->{Control}})
    {
      next unless defined($item) && defined($item->{Type}) && defined($item->{ID});
      next if( $item->{Type} <= 0 );
      delete $defines->{$item->{ID}} if exists $defines->{$item->{ID}};
    }
  }
}

# Save settings
if( defined $param{ok} && !defined($error{error}) )
{
  $error{notice} = $LANG->{message}{applied};

  if( defined($rules_group) && defined($rules_group->{Control}) )
  {
    foreach my $item (@{$rules_group->{Control}})
    {
      next unless defined($item) && defined($item->{Type}) && defined($item->{ID});
      next if( $item->{Type} <= 0 );

      my $ctl_id = '.'.$item->{ID};

      if( defined $param{$ctl_id} && $param{$ctl_id} ne '' )
      {
        $defines->{$item->{ID}} = $param{$ctl_id};
      }
      else
      {
        delete $defines->{$item->{ID}};
      }
    }
  }

  unless( STSCFG_ValidateRules( rules_group=>$rules_group, value=>$defines, with_defaults=>1 ) )
  {
    $error{error} = $LANG->{error}{validation_fault};
    push @ERR, $LANG->{error}{verify_highlighted};
  }

  unless( $error{error} )
  {
    my $vallist = STSCFG_ValuesList( {}, $defines );
    STSConfig::set_group_defines($gid,&STSConfig::STSCFG_RULE,$vallist);
    STSCFG_Succeeded(\%error,"set_group_defines($gid,STSCFG_RULE)", $LANG->{error}{cant_build_profiles}, $LANG->{error}{warn_build_profiles} );
  }

  RestartHelper(\%error);
  STSCFG_ErrorLogHelper(\%error);
  PageOpen("group_rules.cgi?gid=$gid", group_name=>$group_name, rules_group_name=>$rules_group_name, redirect=>"group_rules.cgi?gid=$gid&id=$id&message=ok" ) unless( $error{warning} || $error{error} );
}
elsif( defined($param{message}) && $param{message} eq 'ok' )
{
  $error{notice} = $LANG->{message}{applied};
}

my $memo = ($rules_group_memo)? $rules_group_memo: $LANG->{group}{group_rules}{memo};

# Build page
EnableRulesMenuItems('grules', $gid);

PageOpen("group_rules.cgi?gid=$gid", group_name=>$group_name, rules_group_name=>$rules_group_name );
ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );
FormOpen( uri=>$param{this}, title=>$LANG->{group}{group_rules}{title}, memo=>$memo, group_name=>$group_name, rules_group_name=>$rules_group_name, 'time'=>1 );
ShowRulesGroup( rules_group=>$rules_group, value=>$defines, with_defaults=>1 );
FormRulesSubmit( rules_group_id=>$id, group_id=>$gid );
FormClose();
ErrorBlock( %error );
PageClose();
