#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'stsconfig.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;

$error{error} = $LANG->{error}{cant_access_policy_data} unless( STSCFG_IsReady() );
my $error_load = defined($error{error});

my $groups = ();
unless( $error{error} )
{
  $groups = STSConfig::get_groups();
  STSCFG_Succeeded(\%error,'get_groups()',$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
  STSCFG_CheckGroupList($groups);
  $groups = () unless defined $groups;
}

my $gid = int($param{gid});
if( defined $param{gid} && STSCFG_VerifyGroupID($groups,$gid) )
{
  EnableGroupMenuItems($gid);
}
else
{
  PageOpen('group_members.cgi', redirect=>'group_list.cgi' );
  die "group_action.cgi: Group width id=$gid not exist\n";
}

my $group_name;
{
  my $g = STSCFG_GetGroupByID($groups,$gid);
  $group_name = (defined($g) && defined($g->{Name}))? $g->{Name} : '?';
}

my $checks;
unless( defined $error{error} )
{
  $checks  = STSConfig::get_checks() unless defined $error{error};
  STSCFG_Succeeded(\%error,"get_checks()",$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
}

my $defines;
unless( defined $error{error} )
{
  $defines = STSConfig::get_group_defines($gid,&STSConfig::STSCFG_RULE);
  STSCFG_Succeeded(\%error,"get_group_defines($gid,STSCFG_RULE)",$LANG->{error}{cant_access_policy_data},$LANG->{error}{warn_access_policy_data});
}
$defines = $defines? STSCFG_ValuesHash($defines) : {};


# Save settings
if( defined($param{message}) && $param{message} eq 'ok' )
{
  $error{notice} = $LANG->{message}{applied};
}


# Build page
PageOpen("group_rules_list.cgi?gid=$gid", group_name=>$group_name );

if( $error_load )
{
  ListOpen( title=>$LANG->{group}{group_rules_list}{title}, memo=>$LANG->{group}{group_rules_list}{memo}, group_name=>$group_name, 'time'=>1 );
  ListClose();
  ErrorBlock( %error );
}
else
{
  ErrorBlock( top=>1, %error ) if( $error{warning} || $error{error} );
  ListOpen( title=>$LANG->{group}{group_rules_list}{title}, memo=>$LANG->{group}{group_rules_list}{memo}, group_name=>$group_name, 'time'=>1 );

  my $position = 0;
  foreach my $group (@{$checks})
  {
    STSCFG_PrepareGroupRulesGroupInfo($group,$defines);
    RulesGroupListItem( gid=>$gid, title=>$group->{Name}, memo=>$group->{Memo}, position=>$position++, rules_total=>$group->{RulesCount}, rules_modified=>$group->{RulesModified} );
  }

  ListClose();
  ErrorBlock( %error );
}

PageClose();
