#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
  require 'events.pl';
};

my $info = $CONST->{etc}{info_label};

my $events = GetEvents(system_info=>1, apfilters=>1, approc_server=>1, crontab=>1, spfd=>1,
                       httpdaemon=>1, kasmilter=>1,
                       licdaemon=>1, license=>1, update=>1, nocache=>1);

my %hostname      = %{$events->{hostname}{message}};
my %os            = %{$events->{os}{message}};
my %load_average  = %{$events->{load_average}{message}};

my %product       = %{$events->{lic_product}{message}};
my %filter_id     = %{$events->{filter_id}{message}};

my %as_status = ( alert=>'notice', text=>$LANG->{static}{status_ok} );
my %asup_status  = %as_status;
my %aslic_status = %as_status;

# Server total status
my ($item,$warn,$err,$warn_msg,$err_msg);
$warn = $err = 0;
$warn_msg = $err_msg = undef;
foreach $item (qw(apfilters approc_server crontab_status filter_id spfd_exist httpdaemon kasmilter))
{
  if( defined($events->{$item}) && defined($events->{$item}{message}) )
  {
    if( $events->{$item}{message}{alert} eq 'error' )
    {
      $err++;
      $err_msg = $events->{$item}{message}{text};
    }
    elsif( $events->{$item}{message}{alert} eq 'warning' )
    {
      $warn++;
      $warn_msg = $events->{$item}{message}{text};
    }
  }
}

if( $err ) {
  %as_status = ( alert=>'error', text=>$LANG->{static}{status_errors} );
} elsif( $warn ) {
  %as_status = ( alert=>'warning', text=>$LANG->{static}{status_warnings} );
}

# Updates total status
$warn = $err = 0;
$warn_msg = $err_msg = undef;
foreach $item (qw(crontab_updater update_id update_time))
{
  if( defined($events->{$item}) && defined($events->{$item}{message}) )
  {
    if( $events->{$item}{message}{alert} eq 'error' )
    {
      $err++;
      $err_msg = $events->{$item}{message}{text};
    }
    elsif( $events->{$item}{message}{alert} eq 'warning' )
    {
      $warn++;
      $warn_msg = $events->{$item}{message}{text};
    }
  }
}

if( $err ) {
  %asup_status = ( alert=>'error', text=>$LANG->{static}{status_errors} );
} elsif( $warn ) {
  %asup_status = ( alert=>'warning', text=>$LANG->{static}{status_warnings} );
}

# License total status
$warn = $err = 0;
$warn_msg = $err_msg = undef;
foreach $item (qw(lic_valid lic_type lic_product licdaemon))
{
  if( defined($events->{$item}) && defined($events->{$item}{message}) )
  {
    if( $events->{$item}{message}{alert} eq 'error' )
    {
      $err++;
      $err_msg = $events->{$item}{message}{text};
    }
    elsif( $events->{$item}{message}{alert} eq 'warning' )
    {
      $warn++;
      $warn_msg = $events->{$item}{message}{text};
    }
  }
}

if( $err ) {
  %aslic_status = ( alert=>'error', text=>$LANG->{static}{status_errors} );
} elsif( $warn ) {
  %aslic_status = ( alert=>'warning', text=>$LANG->{static}{status_warnings} );
}


PageOpen('index.cgi', reload=>$CONST->{cgi}{refresh_time});

InfoOpen( %{$LANG->{group}{system_status}}, 'time'=>1 );
InfoBlock( title=>$LANG->{control}{hostname}, %hostname );
InfoBlock( title=>$LANG->{control}{osystem}, %os );
InfoBlock( title=>$LANG->{control}{status_loadaverage}, %load_average ) if( $load_average{text} );
InfoClose();

InfoOpen( %{$LANG->{group}{as_system}} );
InfoBlock( title=>$LANG->{control}{status_product}, %product );
InfoBlock( title=>$LANG->{control}{status_filter_version}, %filter_id );
InfoBlock( title=>$LANG->{control}{as_server}, %as_status, more_url=>'as_status.cgi' );
InfoBlock( title=>$LANG->{control}{as_updater}, %asup_status, more_url=>'asup_status.cgi' );
InfoBlock( title=>$LANG->{control}{as_license}, %aslic_status, more_url=>'aslic_status.cgi' );
InfoClose();

PageClose();

__END__

my %filter        = %{$events->{apfilters}{message}};
my %proc_server   = %{$events->{approc_server}{message}};
my %licdaemon     = %{$events->{licdaemon}{message}};
my %updater       = %{$events->{crontab_updater}{message}};
my %monitoring    = %{$events->{crontab_status}{message}};
my %update_info   = %{$events->{update_id}{message}};
my %update_time   = %{$events->{update_time}{message}};
my %valid         = %{$events->{lic_valid}{message}};
my %license       = %{$events->{lic_type}{message}};

InfoOpen( %{$LANG->{group}{system_status}}, 'time'=>1 );
InfoBlock( title=>$LANG->{control}{hostname}, %hostname );
InfoBlock( title=>$LANG->{control}{osystem}, %os );
InfoBlock( title=>$LANG->{control}{status_product}, %product );
InfoBlock( title=>$LANG->{control}{status_filter_version}, %filter_id );
InfoBlock( title=>$LANG->{control}{status_loadaverage}, %load_average ) if( $load_average{text} );
InfoClose();

InfoOpen( %{$LANG->{group}{as_status}} );
InfoBlock( title=>$LANG->{control}{status_process_server}, %proc_server );
InfoBlock( title=>$LANG->{control}{status_filter}, %filter );
InfoBlock( title=>$LANG->{control}{status_monitoring}, %monitoring );
InfoClose();

InfoOpen( %{$LANG->{group}{asup_status}} );
InfoBlock( title=>$LANG->{control}{status_updater}, %updater );
InfoBlock( title=>$LANG->{control}{status_update_info}, %update_info );
InfoBlock( title=>$LANG->{control}{status_last_update}, %update_time );
InfoClose();

InfoOpen( %{$LANG->{group}{aslic_status}} );
InfoBlock( title=>$LANG->{control}{status_license}, %license );
InfoBlock( title=>$LANG->{control}{status_licvalid}, %valid );
InfoBlock( title=>$LANG->{control}{status_licdaemon}, %licdaemon );
InfoClose();
