#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
  require 'events.pl';
};

my %param = GetParams();
my %error;
my $wrong_data = undef;

if( defined( $param{domains} ))
{
  $error{notice} = $LANG->{message}{applied};

  $param{domains} = '' unless defined $param{domains};

  my $list = alltrim($param{domains});
  $list =~ s/([;,]|\s)+/;/gs;

  my @domains = split /;/, $list;
  {
    my %list;
    foreach (@domains) { $list{$_}++ }
    @domains = keys %list;
  }

  my @err_domains = sort by_domains grep { !VerifyString(protected_domain=>\$_) } @domains;
  if( @err_domains )
  {
    $error{error} = $LANG->{error}{not_valid_domains};
    $wrong_data = 1;
    push @ERR, @err_domains;
  }

  unless( defined($error{error}) )
  {
    unless( SetProtectedDomains(@domains) )
    {
      $error{error} = $LANG->{error}{cant_save_domains_list};
    }
  }

  unless( defined($error{error}) )
  {
    $error{warning} = $LANG->{error}{cant_restart_server} unless( RestartServer() );
  }
}

my $value;

if( $wrong_data && defined($param{domains}) )
{
  $value = $param{domains};
}
else
{
  my $domains = GetProtectedDomains();
  if( $domains )
  {
    my @domains = sort by_domains @{$domains};
    $value = join("\n", @domains);
  }
  else
  {
    $error{warning} = $LANG->{error}{cant_read_domains_list};
  }
}

# Build page

PageOpen('protected_domains.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} || $error{notice} );
FormOpen( action=>$param{this}, %{$LANG->{group}{protected_domains}}, 'time'=>1 );
FormTextarea( title=>$LANG->{control}{protected_domains}, id=>'domains', rows=>20, value=>$value );
FormSubmitOK();
FormClose();
ErrorBlock( %error );
PageClose();

sub GetProtectedDomains()
{
  my $path = $CONFIG->{ProtectedDomainsPath};

  unless( open(FILE,"<$path") )
  {
    push @ERR, "Can't open $path: $!";
    return undef;
  }

  my @domains;
  while( <FILE> )
  {
    chomp;
    next if /^\s*$/;

    my @d = grep { /^\S+$/ } split /\s+/m;
    push @domains, @d if( @d );
  }
  close FILE;

  return \@domains;
}

sub SetProtectedDomains(@)
{
  my @domains = grep { /^\S+$/ } @_;
  my $path = $CONFIG->{ProtectedDomainsPath};
  my $tmppath = $path . ".$$.tmp";

  {
    my %domains;
    foreach (@domains) { $domains{$_}++; }
    @domains = sort by_domains (keys(%domains));
  }

  if( open( FILE, ">$tmppath" ) )
  {
    foreach (@domains) { print FILE "$_\n" }
    close FILE;

    if( move $tmppath, $path )
    {
      return 1;
    }
    else
    {
      push @ERR, "Can't move $tmppath to $path: $!";
    }
  }
  else
  {
    push @ERR, "Can't open file $tmppath: $!";
  }

  return undef;
}
