#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;

# Save settings
if( defined $param{ok} )
{
  $error{notice} = $LANG->{message}{applied};
  my $save_map = {
                    FilterDNSTimeout            => $CONST->{config_map}{FilterDNSTimeout},
                    FilterReceivedHeadersLimit  => $CONST->{config_map}{FilterReceivedHeadersLimit},
                    FilterParseMSOffice         => $CONST->{config_map}{FilterParseMSOffice},
##                    FilterParseRTF              => $CONST->{config_map}{FilterParseRTF}
                    FilterUDSEnabled            => $CONST->{config_map}{FilterUDSEnabled},
                    FilterUDSTimeout            => $CONST->{config_map}{FilterUDSTimeout},
                 };

  $CONFIG->{FilterDNSTimeout}           = int($param{dnstimeout}) if defined ($param{dnstimeout});
  $CONFIG->{FilterReceivedHeadersLimit} = int($param{headers})    if defined ($param{headers});
  $CONFIG->{FilterParseMSOffice}        = ($param{msoffice})? 'yes' : 'no';
##  $CONFIG->{FilterParseRTF}             = ($param{rtf})? 'yes' : 'no';
  $CONFIG->{FilterUDSEnabled}           = ($param{uds_enabled})? 'yes' : 'no';
  $CONFIG->{FilterUDSTimeout}           = int($param{udstimeout}) if defined ($param{udstimeout});

  $error{error} = CheckConfigLimits('FilterDNSTimeout', 'dnsbl_timeout') unless(defined($error{error}));
  $error{error} = CheckConfigLimits('FilterReceivedHeadersLimit', 'received_headers_limit') unless(defined($error{error}));
  $error{error} = CheckConfigLimits('FilterUDSTimeout', 'uds_timeout') unless(defined($error{error}));

  unless( defined($error{error}) )
  {
    $error{error} = $LANG->{error}{cant_save_config}
          unless( WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $save_map ) );
  }

  unless( defined($error{error}) )
  {
    $error{warning} = $LANG->{error}{cant_restart_server} unless( RestartServer() );
  }
}

# Build page
PageOpen('settings_checks.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );

FormOpen( action=>$param{this}, %{$LANG->{group}{settings_checks}}, 'time'=>1 );

FormEdit( title=>$LANG->{control}{received_headers_limit}, id=>'headers', value=>int($CONFIG->{FilterReceivedHeadersLimit}) );
FormEdit( title=>$LANG->{control}{dnsbl_timeout}, id=>'dnstimeout', value=>int($CONFIG->{FilterDNSTimeout}) );
FormCheck( title=>$LANG->{control}{extract_msoffice}, id=>'msoffice',
           value=>($CONFIG->{FilterParseMSOffice}!~/no/i), one_cell=>0 );
## FormCheck( title=>$LANG->{control}{extract_rtf}, id=>'rtf', value=>($CONFIG->{FilterParseRTF}!~/no/i) );
FormCheck( title=>$LANG->{control}{uds_enabled}, id=>'uds_enabled',
           value=>($CONFIG->{FilterUDSEnabled}!~/no/i), one_cell=>0 );
FormEdit( title=>$LANG->{control}{uds_timeout}, id=>'udstimeout', value=>int($CONFIG->{FilterUDSTimeout}) );

FormSubmitOK();
FormClose();

ErrorBlock( %error );
PageClose();
