#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;

# Save settings
if( defined $param{ok} )
{
  $error{notice} = $LANG->{message}{applied};
  my $save_map = {
                    ClientFilteringSizeLimit  => $CONST->{config_map}{ClientFilteringSizeLimit},
                    ClientOnError             => $CONST->{config_map}{ClientOnError},
                    ClientDefaultDomain       => $CONST->{config_map}{ClientDefaultDomain},
                    ClientConnectTimeout      => $CONST->{config_map}{ClientConnectTimeout},
                    ClientDataTimeout         => $CONST->{config_map}{ClientDataTimeout}
                 };

  $CONFIG->{ClientFilteringSizeLimit} = int($param{limit})          if defined $param{limit};
  $CONFIG->{ClientOnError}            = $param{onerror}             if defined $param{onerror};
  $CONFIG->{ClientDefaultDomain}      = alltrim($param{domain})     if defined $param{domain};
  $CONFIG->{ClientConnectTimeout}     = int($param{connecttimeout}) if defined $param{connecttimeout};
  $CONFIG->{ClientDataTimeout}        = int($param{rwtimeout})      if defined $param{rwtimeout};

  $error{error} = CheckConfigLimits('ClientFilteringSizeLimit', 'client_filtering_size_limit') unless(defined($error{error}));
  $error{error} = CheckConfigLimits('ClientConnectTimeout', 'client_connect_timeout') unless(defined($error{error}));
  $error{error} = CheckConfigLimits('ClientDataTimeout', 'client_rw_timeout') unless(defined($error{error}));

  unless( defined($error{error}) )
  {
    if( $CONFIG->{ClientOnError} !~ /^(accept|reject|tempfail)$/ )
    {
        my $msg = $LANG->{error}{invalid_value};
        $msg =~ s/\$field/$LANG->{control}{client_onerror}/g;
        $error{error} = $msg;
    }
    elsif( $CONFIG->{ClientDefaultDomain} !~ /^[-.A-Za-z0-9]*$/ )
    {
        my $msg = $LANG->{error}{invalid_value};
        $msg =~ s/\$field/$LANG->{control}{client_domain}/g;
        $error{error} = $msg;
    }
  }

  unless( defined($error{error}) )
  {
    $error{error} = $LANG->{error}{cant_save_config}
        unless( WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $save_map ) );
  }
  unless( defined($error{error}) )
  {
    $error{warning} = $LANG->{error}{cant_restart_server} unless( RestartServer() );
  }
}

# Build page
PageOpen('settings_clients.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );

FormOpen( action=>$param{this}, %{$LANG->{group}{settings_clients}}, 'time'=>1 );

FormEdit( title=>$LANG->{control}{client_filtering_size_limit}, id=>'limit', value=>int($CONFIG->{ClientFilteringSizeLimit}) );
FormSelect( %{$LANG->{control}{client_onerror}}, id=>'onerror', selected=>$CONFIG->{ClientOnError} );
FormEdit( title=>$LANG->{control}{client_domain}, id=>'domain', value=>$CONFIG->{ClientDefaultDomain} );
FormEdit( title=>$LANG->{control}{client_connect_timeout}, id=>'connecttimeout', value=>int($CONFIG->{ClientConnectTimeout}) );
FormEdit( title=>$LANG->{control}{client_rw_timeout}, id=>'rwtimeout', value=>int($CONFIG->{ClientDataTimeout}) );

FormSubmitOK();
FormClose();

ErrorBlock( %error );

PageClose();
