#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
  require 'logs.pl';
};

my %param = GetParams();
my %error;

# Save settings
if( defined $param{ok} )
{
  $error{notice} = $LANG->{message}{applied};
  my $save_map = {
                    FilterMaxMessages   => $CONST->{config_map}{FilterMaxMessages},
                    FilterRandMessages  => $CONST->{config_map}{FilterRandMessages},
                    FilterMaxIdle       => $CONST->{config_map}{FilterMaxIdle},
                    FilterDelayedExit   => $CONST->{config_map}{FilterDelayedExit}
                 };

  $CONFIG->{FilterMaxMessages}  = int($param{maxmessages})  if defined $param{maxmessages};
  $CONFIG->{FilterRandMessages} = int($param{randmsg})      if defined $param{randmsg};
  $CONFIG->{FilterMaxIdle}      = int($param{maxidle})      if defined $param{maxidle};
  $CONFIG->{FilterDelayedExit}  = int($param{delayedexit})  if defined $param{delayedexit};

  $error{error} = CheckConfigLimits('FilterMaxMessages', 'maxmessages') unless(defined($error{error}));
  $error{error} = CheckConfigLimits('FilterRandMessages', 'randmsg') unless(defined($error{error}));
  $error{error} = CheckConfigLimits('FilterMaxIdle', 'maxidle') unless(defined($error{error}));
  $error{error} = CheckConfigLimits('FilterDelayedExit', 'delayedexit') unless(defined($error{error}));

  unless( defined($error{error}) )
  {
    $error{error} = $LANG->{error}{cant_save_config}
            unless( WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $save_map ) );
  }
  unless( defined($error{error}) )
  {
    $error{warning} = $LANG->{error}{cant_restart_server} unless( RestartServer() );
  }
}

# Build page
PageOpen('settings_filter.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );

FormOpen( action=>$param{this}, %{$LANG->{group}{settings_filter}}, 'time'=>1 );
FormEdit( title=>$LANG->{control}{maxmessages}, id=>'maxmessages', value=>int($CONFIG->{FilterMaxMessages}) );
FormEdit( title=>$LANG->{control}{randmsg},     id=>'randmsg',     value=>int($CONFIG->{FilterRandMessages}) );
FormEdit( title=>$LANG->{control}{maxidle},     id=>'maxidle',     value=>int($CONFIG->{FilterMaxIdle}) );
FormEdit( title=>$LANG->{control}{delayedexit}, id=>'delayedexit', value=>int($CONFIG->{FilterDelayedExit}) );
FormSubmitOK();
FormClose();

ErrorBlock( %error );
PageClose();
