#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU @ERR);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %error;

# Save settings
if( defined $param{ok} )
{
  $error{notice} = $LANG->{message}{applied};
  my $save_map = {
                    ServerMaxFilters      => $CONST->{config_map}{ServerMaxFilters},
                    ServerStartFilters    => $CONST->{config_map}{ServerStartFilters},
                    ServerSpareFilters    => $CONST->{config_map}{ServerSpareFilters}
                 };

  $CONFIG->{ServerMaxFilters}   = int($param{maxfilters})   if defined $param{maxfilters};
  $CONFIG->{ServerStartFilters} = int($param{startfilters}) if defined $param{startfilters};
  $CONFIG->{ServerSpareFilters} = int($param{sparefilters}) if defined $param{sparefilters};

  $error{error} = CheckConfigLimits('ServerMaxFilters', 'maxfilters') unless(defined($error{error}));

  unless( defined($error{error}) )
  {
    if( $CONFIG->{ServerStartFilters} < 0 || $CONFIG->{ServerStartFilters} > $CONFIG->{ServerMaxFilters} )
    {
        my $msg = $LANG->{error}{invalid_value};
        $msg =~ s/\$field/$LANG->{control}{startfilters}/g;
        $error{error} = $msg;
    }
    elsif( $CONFIG->{ServerSpareFilters} < 0 || $CONFIG->{ServerSpareFilters} > $CONFIG->{ServerMaxFilters} )
    {
        my $msg = $LANG->{error}{invalid_value};
        $msg =~ s/\$field/$LANG->{control}{sparefilters}/g;
        $error{error} = $msg;
    }
  }

  unless( defined($error{error}) )
  {
    $error{error} = $LANG->{error}{cant_save_config} 
                            unless( WriteConfFile( $CONFIG, $CONFIG->{ConfigPath}, $save_map ) );
  }

  unless( defined($error{error}) )
  {
    $error{warning} = $LANG->{error}{cant_restart_server} unless( RestartServer(server=>1) );
  }
}

# Build page
PageOpen('settings_server.cgi');
#ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );

FormOpen( action=>$param{this}, %{$LANG->{group}{settings_server}}, 'time'=>1 );
FormEdit( title=>$LANG->{control}{maxfilters},   id=>'maxfilters',   value=>int($CONFIG->{ServerMaxFilters}) );
FormEdit( title=>$LANG->{control}{startfilters}, id=>'startfilters', value=>int($CONFIG->{ServerStartFilters}) );
FormEdit( title=>$LANG->{control}{sparefilters}, id=>'sparefilters', value=>int($CONFIG->{ServerSpareFilters}) );
FormSubmitOK();
FormClose();

ErrorBlock( %error );
PageClose();
