#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU);

use CGI qw(:standard -private_tempfiles);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
  require 'events.pl';
};

my %param = GetParams();
my %error;

# Load settings
my %map = (
            Keepup2DateConnectTimeout         => $CONST->{kasupdaterini_map}{Keepup2DateConnectTimeout},
            Keepup2DatePassiveFtp             => $CONST->{kasupdaterini_map}{Keepup2DatePassiveFtp},
            Keepup2DateUseProxy               => $CONST->{kasupdaterini_map}{Keepup2DateUseProxy},
            Keepup2DateUseUpdateServerUrl     => $CONST->{kasupdaterini_map}{Keepup2DateUseUpdateServerUrl},
            Keepup2DateUseUpdateServerUrlOnly => $CONST->{kasupdaterini_map}{Keepup2DateUseUpdateServerUrlOnly},
            Keepup2DateProxyAddress           => $CONST->{kasupdaterini_map}{Keepup2DateProxyAddress},
            Keepup2DateRegion                 => $CONST->{kasupdaterini_map}{Keepup2DateRegion},
            Keepup2UpdateServerUrl            => $CONST->{kasupdaterini_map}{Keepup2UpdateServerUrl},
            Keepup2DateLogLevel               => $CONST->{kasupdaterini_map}{Keepup2DateLogLevel},
          );

my %UPDATER_CONFIG;
unless( ReadIniFile( \%UPDATER_CONFIG, $CONFIG->{UpdaterIniPath}, \%map ) )
{
  $error{error} = $LANG->{error}{cant_read_kasupdaterini};
}

# Get boolean values
if( defined($UPDATER_CONFIG{Keepup2DatePassiveFtp}) )
{
  if( $UPDATER_CONFIG{Keepup2DatePassiveFtp} =~ /^((no)|(n)|(false)|(0))$/i )
    { $UPDATER_CONFIG{Keepup2DatePassiveFtp} = 0; }
  elsif( $UPDATER_CONFIG{Keepup2DatePassiveFtp} =~ /^((yes)|(t)|(true)|(1))$/i )
    { $UPDATER_CONFIG{Keepup2DatePassiveFtp} = 1; }
  else
  {
    $UPDATER_CONFIG{Keepup2DatePassiveFtp} = $CONST->{defaults}{Keepup2DatePassiveFtp};
    unless( defined($error{error}) )
    {
      my $msg = $LANG->{error}{invalid_value_keepup2date};
      $msg =~ s/\$field/$LANG->{control}{passive_ftp}/g;
      $error{error} = $msg;
    }
  }
}

if( defined($UPDATER_CONFIG{Keepup2DateUseProxy}) )
{
  if( $UPDATER_CONFIG{Keepup2DateUseProxy} =~ /^((no)|(n)|(false)|(0))$/i )
    { $UPDATER_CONFIG{Keepup2DateUseProxy} = 0; }
  elsif( $UPDATER_CONFIG{Keepup2DateUseProxy} =~ /^((yes)|(t)|(true)|(1))$/i )
    { $UPDATER_CONFIG{Keepup2DateUseProxy} = 1; }
  else
  {
    $UPDATER_CONFIG{Keepup2DateUseProxy} = $CONST->{defaults}{Keepup2DateUseProxy};
    unless( defined($error{error}) )
    {
      my $msg = $LANG->{error}{invalid_value_keepup2date};
      $msg =~ s/\$field/$LANG->{control}{use_proxy}/g;
      $error{error} = $msg;
    }
  }
}

if( defined($UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl}) )
{
  if( $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} =~ /^((no)|(n)|(false)|(0))$/i )
    { $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} = 0; }
  elsif( $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} =~ /^((yes)|(t)|(true)|(1))$/i )
    { $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} = 1; }
  else
  {
    $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} = $CONST->{defaults}{Keepup2DateUseUpdateServerUrl};
    unless( defined($error{error}) )
    {
      my $msg = $LANG->{error}{invalid_value_keepup2date};
      $msg =~ s/\$field/$LANG->{control}{use_updateserverurl}/g;
      $error{error} = $msg;
    }
  }
}

if( defined($UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly}) )
{
  if( $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly} =~ /^((no)|(n)|(false)|(0))$/i )
    { $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly} = 0; }
  elsif( $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly} =~ /^((yes)|(t)|(true)|(1))$/i )
    { $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly} = 1; }
  else
  {
    $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly} = $CONST->{defaults}{Keepup2DateUseUpdateServerUrlOnly};
    unless( defined($error{error}) )
    {
      my $msg = $LANG->{error}{invalid_value_keepup2date};
      $msg =~ s/\$field/$LANG->{control}{use_updateserverurl_only}/g;
      $error{error} = $msg;
    }
  }
}

# Set defaults
$UPDATER_CONFIG{Keepup2DateConnectTimeout} = $CONST->{defaults}{Keepup2DateConnectTimeout}
          unless( defined($UPDATER_CONFIG{Keepup2DateConnectTimeout}) );
$UPDATER_CONFIG{Keepup2DatePassiveFtp} = $CONST->{defaults}{Keepup2DatePassiveFtp}
          unless( defined($UPDATER_CONFIG{Keepup2DatePassiveFtp}) );
$UPDATER_CONFIG{Keepup2DateUseProxy} = $CONST->{defaults}{Keepup2DateUseProxy}
          unless( defined($UPDATER_CONFIG{Keepup2DateUseProxy}) );
$UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} = $CONST->{defaults}{Keepup2DateUseUpdateServerUrl}
          unless( defined($UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl}) );
$UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly} = $CONST->{defaults}{Keepup2DateUseUpdateServerUrlOnly}
          unless( defined($UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly}) );
$UPDATER_CONFIG{Keepup2DateProxyAddress} = $CONST->{defaults}{Keepup2DateProxyAddress}
          unless( defined($UPDATER_CONFIG{Keepup2DateProxyAddress}) );
$UPDATER_CONFIG{Keepup2DateRegion} = $CONST->{defaults}{Keepup2DateRegion}
          unless( defined($UPDATER_CONFIG{Keepup2DateRegion}) );
$UPDATER_CONFIG{Keepup2UpdateServerUrl} = $CONST->{defaults}{Keepup2UpdateServerUrl}
          unless( defined($UPDATER_CONFIG{Keepup2UpdateServerUrl}) );
$UPDATER_CONFIG{Keepup2DateLogLevel} = $CONST->{defaults}{Keepup2DateLogLevel}
          unless( defined($UPDATER_CONFIG{Keepup2DateLogLevel}) );

# Verify settings
my %proxy_addr;
my $proxy_addr_view = "";
my $proxy_user_view = "";
my $proxy_pass_view = "";

if(parse_proxy_addr(\%proxy_addr, $UPDATER_CONFIG{Keepup2DateProxyAddress}))
{
  $proxy_addr_view = $proxy_addr{protocol} . $proxy_addr{url} . $proxy_addr{port};
  $proxy_user_view = $proxy_addr{user};
  $proxy_pass_view = $proxy_addr{pass};
}
else
{
  $proxy_addr_view = $UPDATER_CONFIG{Keepup2DateProxyAddress};

  my $msg = $LANG->{error}{invalid_value_keepup2date};
  $msg =~ s/\$field/$LANG->{control}{proxy}/g;
  $error{error} = $msg;
}

# Read schedule settings
DropEventCache( qw(crontab) );

my $schedule = $CONST->{defaults}{UpdateTime};
{
  my $crontab = ReadCrontab();
  if( $crontab )
  {
    $schedule = ( defined($crontab->{updater}) && $crontab->{updater}>0 )? $crontab->{updater}: 0;
  }
  else
  {
    $error{error} = $LANG->{error}{crontab_get_fault};
  }
}

# Save settings
if( !defined($error{error}) && defined $param{ok} )
{
  $schedule = $param{schedule} if defined($param{schedule});

  $UPDATER_CONFIG{Keepup2DateConnectTimeout} = int($param{timeout})     if defined($param{timeout});

  $UPDATER_CONFIG{Keepup2DatePassiveFtp} = defined($param{pasv}) ? $param{pasv} : 0;
  $UPDATER_CONFIG{Keepup2DateUseProxy} = defined($param{useproxy}) ? $param{useproxy} : 0;
  $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} = defined($param{useurl}) ? $param{useurl} : 0;
  $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly} = defined($param{urlonly}) ? $param{urlonly} : 0;
  
  $UPDATER_CONFIG{Keepup2DateRegion} = alltrim($param{region})          if defined($param{region});
  $UPDATER_CONFIG{Keepup2UpdateServerUrl} = alltrim($param{url})        if defined($param{url});
  $UPDATER_CONFIG{Keepup2DateLogLevel} = $param{loglevel}               if defined($param{loglevel});

  # proxy address
  $proxy_addr_view = $param{proxy}       if defined($param{proxy});
  $proxy_user_view = $param{proxy_user}  if defined($param{proxy_user});
  $proxy_pass_view = $param{proxy_pass}  if defined($param{proxy_pass});

  if(defined($param{proxy}))
  {
    unless(parse_proxy_addr(\%proxy_addr, $param{proxy}))
    {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{proxy}/g;
      $error{error} = $msg;
    }
  }
  if( !defined($error{error}) && defined($param{proxy_user}) )
  {
    $param{proxy_user} = alltrim($param{proxy_user});

    if(!check_user_value($param{proxy_user}))
    {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{proxy_user}/g;
      $error{error} = $msg;
    }
    elsif($param{proxy_user} && $proxy_addr{user} && ($param{proxy_user} ne $proxy_addr{user}))
    {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{proxy_user}/g;
      $error{error} = $msg;
    }
    elsif($param{proxy_user})
    {
      $proxy_addr{user} = $param{proxy_user};
    }
  }
  if( !defined($error{error}) && defined($param{proxy_pass}) )
  {
    $param{proxy_pass} = alltrim($param{proxy_pass});

    if(!check_pass_value($param{proxy_pass}))
    {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{proxy_pass}/g;
      $error{error} = $msg;
    }
    elsif($param{proxy_pass} && $proxy_addr{pass} && ($param{proxy_pass} ne $proxy_addr{pass}))
    {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{proxy_pass}/g;
      $error{error} = $msg;
    }
    elsif($param{proxy_pass})
    {
      $proxy_addr{pass} = $param{proxy_pass};
    }
  }
  if( !defined($error{error}) )
  {
    if( $proxy_addr{user} && !$proxy_addr{url} )
    {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{proxy_user}/g;
      $error{error} = $msg;
    }
    elsif( $proxy_addr{pass} && (!$proxy_addr{url} || !$proxy_addr{user}))
    {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{proxy_pass}/g;
      $error{error} = $msg;
    }
    elsif( $proxy_addr{user} && !$proxy_addr{pass} )
    {
      $error{error} = $LANG->{error}{empty_proxy_pass};
    }
  }
  if( !defined($error{error}) )
  {
    $proxy_addr_view = $proxy_addr{protocol} . $proxy_addr{url} . $proxy_addr{port};
    $proxy_user_view = $proxy_addr{user};
    $proxy_pass_view = $proxy_addr{pass};

    $UPDATER_CONFIG{Keepup2DateProxyAddress} = $proxy_addr{protocol};
    $UPDATER_CONFIG{Keepup2DateProxyAddress} .= $proxy_addr{user};
    $UPDATER_CONFIG{Keepup2DateProxyAddress} .= ':' if($proxy_addr{pass});
    $UPDATER_CONFIG{Keepup2DateProxyAddress} .= $proxy_addr{pass};
    $UPDATER_CONFIG{Keepup2DateProxyAddress} .= '@' if($proxy_addr{user} || $proxy_addr{pass});
    $UPDATER_CONFIG{Keepup2DateProxyAddress} .= $proxy_addr{url};
    $UPDATER_CONFIG{Keepup2DateProxyAddress} .= $proxy_addr{port};
  }

  if( !defined($error{error}) && $UPDATER_CONFIG{Keepup2DateUseProxy} == 1 && !$UPDATER_CONFIG{Keepup2DateProxyAddress} )
  {
      $error{error} = $LANG->{error}{invalid_value_proxy};
  }

  # check settings
  if( !defined($error{error}) &&
        ($UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl} || $UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly}) &&
        (!defined($UPDATER_CONFIG{Keepup2UpdateServerUrl}) || length($UPDATER_CONFIG{Keepup2UpdateServerUrl}) == 0) )
  {
      $error{error} = $LANG->{error}{invalid_value_serverurl};
  }

  if( !defined($error{error}) && 
      ($UPDATER_CONFIG{Keepup2DateConnectTimeout} < 10 || $UPDATER_CONFIG{Keepup2DateConnectTimeout} > 100) )
  {
      my $msg = $LANG->{error}{invalid_value};
      $msg =~ s/\$field/$LANG->{control}{connect_timeout}/g;
      $error{error} = $msg;
  }

  # save
  $error{notice} = $LANG->{message}{applied};
  unless( defined($error{error}) )
  {
    $error{error} = $LANG->{error}{cant_write_kasupdaterini}
      unless( WriteIniFile( \%UPDATER_CONFIG, $CONFIG->{UpdaterIniPath}, \%map ) );
  }
  unless( defined($error{error}) )
  {
    $error{error} = $LANG->{error}{crontab_set_fault}
      unless( WriteCrontab( updater=>$schedule, MAILTO=>$CONFIG->{AlertsTo} ) );
  }
}

#$error{warning} = $LANG->{error}{updater_not_in_crontab} if( !defined($schedule) || $schedule <= 0 );

# Build page
PageOpen('settings_updater.cgi');
ErrorBlock( top=>1, %error ) if( $error{error} || $error{warning} );

FormOpen( action=>$param{this}, title=>$LANG->{group}{settings_updates}{title}, memo=>$LANG->{group}{settings_updates}{memo}, 'time'=>1 );

FormSelect( title=>$LANG->{control}{schedule}{title}, id=>'schedule', selected=>$schedule, options=>$LANG->{control}{schedule}{options} ) unless( $CONFIG->{CCDontModifyCrontab} );

FormSelect( title=>$LANG->{control}{updater_loglevel}{title}, id=>'loglevel', selected=>$UPDATER_CONFIG{Keepup2DateLogLevel}, options=>$LANG->{control}{updater_loglevel}{options} );
FormEdit( title=>$LANG->{control}{connect_timeout}, id=>'timeout', value=>int($UPDATER_CONFIG{Keepup2DateConnectTimeout}) );
FormCheck( title=>$LANG->{control}{passive_ftp}, id=>'pasv',
           value=>$UPDATER_CONFIG{Keepup2DatePassiveFtp}, one_cell=>1 );

FormStaticText( text=>$LANG->{control}{updatesserver_settings}, memo=>'', header=>1 );
FormEdit( title=>$LANG->{control}{region}, id=>'region', value=>$UPDATER_CONFIG{Keepup2DateRegion} );
FormEdit( title=>$LANG->{control}{updateserverurl}, id=>'url', value=>$UPDATER_CONFIG{Keepup2UpdateServerUrl} );

FormControlRowOpen();
FormCheck( title=>$LANG->{control}{use_updateserverurl}, id=>'useurl',
           value=>$UPDATER_CONFIG{Keepup2DateUseUpdateServerUrl}, in_row=>1, one_cell=>1 );
FormCheck( title=>$LANG->{control}{use_updateserverurl_only}, id=>'urlonly',
           value=>$UPDATER_CONFIG{Keepup2DateUseUpdateServerUrlOnly}, in_row=>1, one_cell=>1 );
FormControlRowClose();

FormStaticText( text=>$LANG->{control}{proxy_settings}, memo=>'', header=>1 );
FormEdit( title=>$LANG->{control}{proxy}, id=>'proxy', value=>$proxy_addr_view );
FormEdit( title=>$LANG->{control}{proxy_user}, id=>'proxy_user', value=>$proxy_user_view );
FormEdit( title=>$LANG->{control}{proxy_pass}, id=>'proxy_pass', value=>$proxy_pass_view, password=>1 );
FormCheck( title=>$LANG->{control}{use_proxy}, id=>'useproxy',
           value=>$UPDATER_CONFIG{Keepup2DateUseProxy}, one_cell=>1 );

FormSubmitOK();
FormClose();

ErrorBlock( %error );
PageClose();

sub parse_proxy_addr
{
  my ($hash, $addr) = @_;

  $hash->{protocol} = "";
  $hash->{user}     = "";
  $hash->{pass}     = "";
  $hash->{url}      = "";
  $hash->{port}     = "";

  $addr = alltrim($addr);
  unless($addr)
  {
    return 1;
  }
  elsif(alltrim($addr) =~ 
          m{^(http://)?(?:([^ \@\:/]+):([^ \@\:/]+)\@)?((?:[-a-z0-9]+\.)+[a-z]{2,}|\d+\.\d+\.\d+\.\d+)(:\d+)?$})
  {
    $hash->{protocol} = $1;
    $hash->{user}     = $2;
    $hash->{pass}     = $3;
    $hash->{url}      = $4;
    $hash->{port}     = $5;

    return 0 if( $hash->{url} =~ m|\d+\.\d+\.\d+\.\d+| && ! VerifyString(ip=>\$hash->{url}) );
    return 1;
  }
  else
  {
    return 0;
  }
}

sub check_user_value
{
  my ($val) = @_;
  $val = alltrim($val);
  return 0 if( $val && $val !~ m|^[^ \@\:/]+$| );
  return 1;
}

sub check_pass_value
{
  my ($val) = @_;
  $val = alltrim($val);
  return 0 if( $val && $val !~ m|^[^ \@\:/]+$| );
  return 1;
}
