#!/usr/local/ap-mailfilter3/bin/perl -w
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use vars qw($CONST $CONFIG $LANG $MENU);

use CGI qw(:standard);
use CGI::Carp;
#use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require 'cgiutil.pl';
  require 'config.pl';
  require 'rc/design.pl';
  require 'utils.pl';
};

my %param = GetParams();
my %modes = (
              'd'=>{ id=>'stat.cgi?period=d', title=>$LANG->{group}{stat_d}{title}, memo=>$LANG->{group}{stat_d}{memo}, graph_m=>'g__all_24hM.png', diag_m=>'d__all_24hM.png', graph_s=>'g__all_24hS.png', diag_s=>'d__all_24hS.png', step=>  300, count=>288 },
              'w'=>{ id=>'stat.cgi?period=w', title=>$LANG->{group}{stat_w}{title}, memo=>$LANG->{group}{stat_w}{memo}, graph_m=>'g__all_7dM.png', diag_m=>'d__all_7dM.png', graph_s=>'g__all_7dS.png', diag_s=>'d__all_7dS.png',     step=> 3600, count=>168 },
              'm'=>{ id=>'stat.cgi?period=m', title=>$LANG->{group}{stat_m}{title}, memo=>$LANG->{group}{stat_m}{memo}, graph_m=>'g__all_1mM.png', diag_m=>'d__all_1mM.png', graph_s=>'g__all_1mS.png', diag_s=>'d__all_1mS.png',     step=> 3600, count=>720 },
              'y'=>{ id=>'stat.cgi?period=y', title=>$LANG->{group}{stat_y}{title}, memo=>$LANG->{group}{stat_y}{memo}, graph_m=>'g__all_1yM.png', diag_m=>'d__all_1yM.png', graph_s=>'g__all_1yS.png', diag_s=>'d__all_1yS.png',     step=>86400, count=>366 },
            );

my ($pageid,$title,$memo,$period,$messages,$graph_src,$diag_src,$data,$rnd);

$period = $param{period};
$period = 'd' unless( defined($period) && defined($modes{$period}) );
$messages = ( !defined($param{data}) || $param{data} =~ /^m/i );
$data = $messages? 'm' : 's';

SetStatisticsMode($messages);

$pageid = $modes{$period}{id} . "&data=$data";
$title  = $modes{$period}{title};
$memo   = $modes{$period}{memo};
$rnd    = '?r'.int( (time()-30) / $modes{$period}{step} );
$graph_src = 'stat/'.$modes{$period}{($messages)?'graph_m':'graph_s'};
$diag_src  = 'stat/'.$modes{$period}{($messages)?'diag_m':'diag_s'};


#my @selector = ( { name=>$LANG->{static}{stattype_messages}, uri=>"stat.cgi?period=$period&data=m", active=>$messages },
#                 { name=>$LANG->{static}{stattype_bytes},    uri=>"stat.cgi?period=$period&data=s", active=>(!$messages) } );

my $stat_path = $CONFIG->{CCStatBasePath};
my $stat = load_stat($stat_path, $modes{$period}{step}, $modes{$period}{count});
my %stat;

foreach (qw(normal spam probable formal trust black total)) { $stat{"$_\_m"} = $stat{"$_\_s"} = 0 }

if( $stat )
{
  foreach my $item (@{$stat})
  {
    my ($key,$val);
    while( ($key,$val) = each(%{$item}) )
    {
      next if $key eq 'time';
      $stat{$key} = 0 unless defined $stat{$key};
      $stat{$key} += $val;
    }
  }

  $stat{total_m} = $stat{normal_m} + $stat{spam_m} + $stat{probable_m} + $stat{formal_m} + $stat{trust_m} + $stat{black_m};
  $stat{total_s} = $stat{normal_s} + $stat{spam_s} + $stat{probable_s} + $stat{formal_s} + $stat{trust_s} + $stat{black_s};
}

PageOpen($pageid, reload=>$CONST->{cgi}{refresh_time});
ListOpen( title=>$title, memo=>$memo, 'time'=>1 );
if( defined($stat{normal_m}) )
{
  my $total_bytes = HumanValue($stat{total_s},'&nbsp;');
  print qq|
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td width=16%></td>
    <td width=14% nowrap><b>$CONST->{detected_classes}{spam}</b></td>
    <td width=14% nowrap><b>$CONST->{detected_classes}{probable}</b></td>
    <td width=14% nowrap><b>$CONST->{detected_classes}{black}</b></td>
    <td width=14% nowrap><b>$CONST->{detected_classes}{formal}</b></td>
    <td width=14% nowrap><b>$CONST->{detected_classes}{trust}</b></td>
    <td width=14% nowrap><b>$CONST->{detected_classes}{normal}</b></td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td nowrap><b>$LANG->{static}{stattype_messages}</b></td>
    <td nowrap>|, HumanValue($stat{spam_m},'',1000), qq|</td>
    <td nowrap>|, HumanValue($stat{probable_m},'',1000), qq|</td>
    <td nowrap>|, HumanValue($stat{black_m},'',1000), qq|</td>
    <td nowrap>|, HumanValue($stat{formal_m},'',1000), qq|</td>
    <td nowrap>|, HumanValue($stat{trust_m},'',1000), qq|</td>
    <td nowrap>|, HumanValue($stat{normal_m},'',1000), qq|</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td nowrap><b>$LANG->{static}{stattype_bytes}</b></td>
    <td nowrap>|, HumanValue($stat{spam_s},''), qq|</td>
    <td nowrap>|, HumanValue($stat{probable_s},''), qq|</td>
    <td nowrap>|, HumanValue($stat{black_s},''), qq|</td>
    <td nowrap>|, HumanValue($stat{formal_s},''), qq|</td>
    <td nowrap>|, HumanValue($stat{trust_s},''), qq|</td>
    <td nowrap>|, HumanValue($stat{normal_s},''), qq|</td>
  </tr>
|;

#<!--
#  <tr valign=top align=center bgcolor=#f2f2f2>
#    <td>&nbsp;<b>$LANG->{static}{stat_total}</b>&nbsp;</td>
#    <td colspan=3>&nbsp;|, HumanValue($stat{total_m},'&nbsp;',1000), qq|&nbsp;$LANG->{static}{stattype_messages1}&nbsp;</td>
#    <td colspan=3>&nbsp;$total_bytes|, (($total_bytes =~ /\D/)? '': '&nbsp;'), qq|$LANG->{static}{stattype_bytes1}&nbsp;</td>
#  </tr>
#  <tr valign=middle align=center bgcolor=#f2f2f2>
#    <td>&nbsp;<b>$LANG->{static}{stat_total}</b>&nbsp;</td>
#    <td colspan=2>&nbsp;|, HumanValue($stat{total_m},'&nbsp;',1000), qq|&nbsp;$LANG->{static}{stattype_messages1}&nbsp;<br>&nbsp;$total_bytes|, (($total_bytes =~ /\D/)? '': '&nbsp;'), qq|$LANG->{static}{stattype_bytes1}&nbsp;</td>
#    <td colspan=4 bgcolor=white></td>
#  </tr> -->
}
ListClose();


if( -r $CONFIG->{CCWwwRootPath}.'/'.$graph_src && -r $CONFIG->{CCWwwRootPath}.'/'.$diag_src )
{
  my ($before_msg,$after_msg,$before_size,$after_size) = ( $messages )?
    ( '<span class=sel>', '</span>', "<a href=stat.cgi?period=$period&data=s>", '</a>' ):
    ( "<a href=stat.cgi?period=$period&data=m>", '</a>', '<span class=sel>', '</span>' );

print qq!
  <p align=center><img src=$graph_src$rnd width=500 height=150 vspace=0 border=0>
    <br><img src=$diag_src$rnd width=360 height=200 vspace=0 border=0>
    <br>

  <table cellpadding=3 cellspacing=1 width=100% border=0 bgcolor=#ffffff>
  <tr valign=top>
    <td align=left><b>$LANG->{static}{stat_view}:</b>&nbsp;$before_msg$LANG->{static}{stattype_messages}$after_msg&nbsp;<span class=gray>|</span>&nbsp;$before_size$LANG->{static}{stattype_bytes}$after_size</td>
    <td align=right><b>$LANG->{static}{stat_export}:</b>&nbsp;
          <a href=export.cgi?period=$period&data=$data&format=cvs>$LANG->{static}{stat_export_csv}</a>&nbsp;
          <span class=gray>|</span>&nbsp;
!;
##          <a href=export.cgi?period=$period&data=$data&format=xl>$LANG->{static}{stat_export_excel}</a>&nbsp;
##          <span class=gray>|</span>&nbsp;
print qq!
          <a href=export.cgi?period=$period&data=$data&format=html>$LANG->{static}{stat_export_html}</a>
    </td>
  </tr>
  </table>
!;
}
else
{
  ErrorBlock( info=>$LANG->{message}{no_statistics} );
}

PageClose();

__END__

ListOpen( title=>$title, memo=>$memo, 'time'=>1 );
#FormListTypeSelector( selector=>\@selector, type=>$param{type} );
if( $stat )
{
  print qq|
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td width=20%>&nbsp;</td>
    <td width=40%>&nbsp;<b>$LANG->{static}{stattype_messages}</b>&nbsp;</td>
    <td width=40%>&nbsp;<b>$LANG->{static}{stattype_bytes}</b>&nbsp;</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td>&nbsp;<b>$LANG->{static}{stat_export_normal_m}</b>&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{normal_m},'&nbsp;'), qq|&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{normal_s},'&nbsp;'), qq|&nbsp;</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td>&nbsp;<b>$LANG->{static}{stat_export_spam_m}</b>&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{spam_m},'&nbsp;'), qq|&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{spam_s},'&nbsp;'), qq|&nbsp;</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td>&nbsp;<b>$LANG->{static}{stat_export_probable_m}</b>&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{probable_m},'&nbsp;'), qq|&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{probable_s},'&nbsp;'), qq|&nbsp;</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td>&nbsp;<b>$LANG->{static}{stat_export_formal_m}</b>&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{formal_m},'&nbsp;'), qq|&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{formal_s},'&nbsp;'), qq|&nbsp;</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td>&nbsp;<b>$LANG->{static}{stat_export_trust_m}</b>&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{trust_m},'&nbsp;'), qq|&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{trust_s},'&nbsp;'), qq|&nbsp;</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td>&nbsp;<b>$LANG->{static}{stat_export_black_m}</b>&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{black_m},'&nbsp;'), qq|&nbsp;</td>
    <td>&nbsp;|, SeparateValue($stat{black_s},'&nbsp;'), qq|&nbsp;</td>
  </tr>
  <tr valign=top align=center bgcolor=#f2f2f2>
    <td>&nbsp;<b>$LANG->{static}{stat_total}</b>&nbsp;</td>
    <td>&nbsp;<b>|, SeparateValue($stat{total_m},'&nbsp;'), qq|</b>&nbsp;</td>
    <td>&nbsp;<b>|, SeparateValue($stat{total_s},'&nbsp;'), qq|</b>&nbsp;</td>
  </tr>
|;
}
ListClose();
